@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,26 +18,12 @@
 // 
 //
 
-/* $Id: render_handler.h,v 1.45 2007/07/18 10:06:13 udog Exp $ */
-
 #ifndef RENDER_HANDLER_H
 #define RENDER_HANDLER_H
 
 /// \page render_handler_intro Render handler introduction
 ///
-/// Information for writing new render handlers:
-///
-/// There are two ways to write a new render handler:
-/// - either you write a handler that must only be able to draw line strips and
-///   and triangles (with various fill styles, however). This we call a 
-///   triangulating render handler.
-/// - or your handler can deal with the special fill style Flash uses directly,
-///   which does *not* apply to most hardware accelerations. In this case the
-///   unaltered original shapes are passed to the renderer, which is responsible
-///   of drawing them correctly. 
-///
-/// For triangulating render handlers, see gnash::triangulating_render_handler
-/// (defined in render_handler_tri.h), otherwise read on...
+/// Information for writing new render handlers.
 ///
 /// The most important thing about drawing Flash shapes is to understand how 
 /// their fill styles work. 
@@ -157,11 +143,7 @@
 /// faster than converting between ints and floats all the way.
 
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include "tu_config.h" // for DSOEXPORT
+#include "dsodefs.h" // for DSOEXPORT
 
 #include "shape_character_def.h"  
 #include "generic_character.h"
@@ -224,14 +206,6 @@ public:
 
   // Your handler should return these with a ref-count of 0.  (@@ is that the right policy?)
 
-  /// \brief
-  ///  Create a bitmap_info so that it contains an alpha texture
-  /// with the given data (1 byte per texel).
-  //
-  /// Munges *data (in order to make mipmaps)!!
-  ///
-  virtual bitmap_info*  create_bitmap_info_alpha(int w, int h, unsigned char* data) = 0;
-
   /// \brief
   /// Given an image, returns a pointer to a bitmap_info class
   /// that can later be passed to fill_styleX_bitmap(), to set a
@@ -316,8 +290,8 @@ public:
     point topleft     = pixel_to_world(pixelbounds.getMinX(), pixelbounds.getMinY());
     point bottomright = pixel_to_world(pixelbounds.getMaxX(), pixelbounds.getMaxY());
     
-    return geometry::Range2d<float> (topleft.m_x, topleft.m_y, 
-      bottomright.m_x, bottomright.m_y);
+    return geometry::Range2d<float> (topleft.x, topleft.y, 
+      bottomright.x, bottomright.y);
   }
   
   virtual geometry::Range2d<int> world_to_pixel(const geometry::Range2d<float>& worldbounds)
@@ -353,19 +327,13 @@ public:
 
   virtual void  end_display() = 0;
     
-  /// Geometric transforms for mesh and line_strip rendering.
-  virtual void  set_matrix(const matrix& m) = 0;
-
-  /// Color transforms for mesh and line_strip rendering.
-  virtual void  set_cxform(const cxform& cx) = 0;
-    
   /// Draw a line-strip directly, using a thin, solid line. 
   //
   /// Can be used to draw empty boxes and cursors.
   virtual void  draw_line_strip(const void* coords, int vertex_count,
-      const rgba& color) = 0;
+      const rgba& color, const matrix& mat) = 0;
     
-  /// Draw a simple, solid filled polygon (no outline).
+  /// Draw a simple, solid filled polygon with a thin (~1 pixel) outline.
   //
   /// This can't be used for 
   /// Flash shapes but is intended for internal drawings like bounding boxes 
@@ -376,8 +344,11 @@ public:
   /// The polygon need NOT be closed (ie: this function will automatically
   /// add an additional vertex to close it.
   ///
+  /// When masked==false, then any potential mask currently active will be
+  /// ignored, otherwise it is respected.
+  ///
   virtual void  draw_poly(const point* corners, size_t corner_count, 
-    const rgba& fill, const rgba& outline) = 0;
+    const rgba& fill, const rgba& outline, const matrix& mat, bool masked) = 0;
     
     
   /// Set line and fill styles for mesh & line_strip rendering.
@@ -387,24 +358,25 @@ public:
     WRAP_CLAMP
   };
     
-    
-  /// Special function to draw a rectangular bitmap.
-  //
-  /// Intended for textured glyph rendering.  Ignores
-  /// current transforms.
-  ///
-  virtual void  draw_bitmap(
-    const matrix&   m,
-    const bitmap_info*  bi,
-    const rect&   coords,
-    const rect&   uv_coords,
-    const rgba&   color) = 0;
-    
   virtual void  set_antialiased(bool enable) = 0;
     
+  ///@{ Masks
+  ///
+  /// Masks are defined by drawing calls enclosed by begin_submit_mask()
+  /// and end_submit_mask(). Between these two calls, no drawing is to
+  /// occur. The shapes rendered between the two calls define the
+  /// visible region of the mask. Graphics that are irrelevant in the
+  /// context of a mask (lines and fill styles, for example) should be
+  /// ignored. After use, disable_mask() is called to remove the mask.
+  ///
+  /// Masks may be nested. That is, end_submit_mask() may be followed
+  /// by a call to begin_submit_mask(). The resulting mask shall be an
+  /// intersection of the previously created mask. disable_mask() shall
+  /// result in the disabling or destruction of the last created mask.
   virtual void begin_submit_mask() = 0;
   virtual void end_submit_mask() = 0;
   virtual void disable_mask() = 0;
+  ///@}
   
   /// \brief
   /// Draws the given character definition with the stateful properties
@@ -493,7 +465,9 @@ public:
   //
   /// Glyphs are defined just like shape characters with the difference that
   /// they do not have any fill or line styles.
-  /// Instead, the shape must be drawn using the given color (solid fill). 
+  /// Instead, the shape must be drawn using the given color (solid fill).
+  /// Please note that although the glyph paths may indicate subshapes,
+  /// the renderer is to ignore that information.
   /// 
   /// @param def
   ///
@@ -505,14 +479,7 @@ public:
   ///
   virtual void draw_glyph(shape_character_def *def, const matrix& mat,
     const rgba& color, float pixel_scale) = 0;
-    
-  /// The render handler can choose if it wishes to use textured glyphs 
-  /// (pre-computed bitmaps which are used for small text sizes) or if 
-  /// draw_glyph() should be used in any case. When glyph textures are not
-  /// desired, then draw_bitmap() is never called in the *current* version.  
-  virtual bool allow_glyph_textures() = 0;
-    
-    
+
   /// This function returns the color at any position in the stage. It is used
   /// for automatic testing only, it should not be used for anything else!
   /// x and y are pixel coordinates (<0 won't make any sense) and the color of 
@@ -522,8 +489,8 @@ public:
   virtual bool getPixel(rgba& /*color_return*/, int /*x*/, int /*y*/)
   {
 
-    log_msg("getPixel() not implemented for this renderer");
-    assert(0);    
+    log_debug("getPixel() not implemented for this renderer");
+    abort();    
     return false; // avoid compiler warning    
   }
   
@@ -623,9 +590,11 @@ public:
     // nop
   }
 
-  virtual void get_scale(point& /*scale*/) {
+  /// Sets the x/y offset for the movie  
+  virtual void set_translation(float /*xoff*/, float /*yoff*/) {
+    // nop
   }
-    
+
 protected:
 
   /// Cached fill style list with just one entry used for font rendering
