@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include <map>
@@ -27,6 +27,7 @@
 
 #include <dirent.h> // used by scandir()
 #include <unistd.h> // used by unlink()
+#include <fcntl.h>  // used by asyncmode()
 
 #include "VM.h"
 #include "log.h"
@@ -57,8 +58,10 @@ as_value fileio_getchar(const fn_call& fn);
 as_value fileio_putchar(const fn_call& fn);
 as_value fileio_fflush(const fn_call& fn);
 as_value fileio_ftell(const fn_call& fn);
+as_value fileio_feof(const fn_call& fn);
 as_value fileio_fseek(const fn_call& fn);
 as_value fileio_unlink(const fn_call& fn);
+as_value fileio_asyncmode(const fn_call& fn);
 
 // <Udo> I needed a scandir() function and implemented it here for simplicity.
 // Maybe this should be moved to a dedicated extension and a different class? 
@@ -91,6 +94,8 @@ attachInterface(as_object& obj)
     obj.init_member("fflush", new builtin_function(fileio_fflush));
     obj.init_member("fseek", new builtin_function(fileio_fseek));
     obj.init_member("ftell", new builtin_function(fileio_ftell));
+    obj.init_member("asyncmode", new builtin_function(fileio_asyncmode));
+    obj.init_member("feof", new builtin_function(fileio_feof));
     obj.init_member("fclose", new builtin_function(fileio_fclose));
     
     obj.init_member("unlink", new builtin_function(fileio_unlink));
@@ -191,6 +196,38 @@ Fileio::ftell()
     return -1;
 }
 
+bool
+Fileio::asyncmode(bool async)
+{
+//    GNASH_REPORT_FUNCTION;
+    if (_stream) {
+    
+        int fd = fileno(_stream);
+        
+        long flags = fcntl(fd, F_GETFL);
+        
+        int res;
+    
+        if (async)
+          res = fcntl(fd, F_SETFL, flags|O_NONBLOCK);
+        else
+          res = fcntl(fd, F_SETFL, flags&(~O_NONBLOCK));
+          
+        return res>=0;
+    }
+    return false;
+}
+
+bool
+Fileio::feof()
+{
+//    GNASH_REPORT_FUNCTION;
+    if (_stream) {
+        return ::feof(_stream);
+    }
+    return -1;
+}
+
 bool
 Fileio::fopen(const string &filespec, const string &mode)
 {
@@ -237,7 +274,11 @@ Fileio::fgets(std::string &str)
     if (_stream) {
         char buf[BUFSIZE];
         memset(buf, 0, BUFSIZE);
-        str = ::fgets(buf, BUFSIZE, _stream);
+        char* res = ::fgets(buf, BUFSIZE, _stream);
+        if (res) 
+          str = res;
+        else
+          str = "";  // we might want to return NULL to the VM ?
         return str;
     }
     return str;
@@ -340,8 +381,8 @@ fileio_fopen(const fn_call& fn)
 		return as_value(false);
     }
 
-    string filespec = fn.arg(0).to_string(&fn.env());
-    string mode = fn.arg(1).to_string(&fn.env());
+    string filespec = fn.arg(0).to_string();
+    string mode = fn.arg(1).to_string();
     return as_value(ptr->fopen(filespec, mode));
 
 }
@@ -379,8 +420,15 @@ fileio_fgetc(const fn_call& fn)
     boost::intrusive_ptr<Fileio> ptr = ensureType<Fileio>(fn.this_ptr);
     assert(ptr);
     int i = ptr->fgetc();
-    char *c = reinterpret_cast<char *>(&i);
-    return as_value(c);
+    
+    if ((i==EOF) || (i<0)) 
+    {
+      return as_value(false);  // possible in async mode
+    } else {
+      char c[2]="x"; // set to 1 char to get the zero byte!
+      c[0] = i;
+      return as_value(c);
+    }
 }
 
 as_value
@@ -444,7 +492,7 @@ fileio_fputs(const fn_call& fn)
     //   GNASH_REPORT_FUNCTION;
     boost::intrusive_ptr<Fileio> ptr = ensureType<Fileio>(fn.this_ptr);
 
-    string str = fn.arg(0).to_string(&fn.env());
+    string str = fn.arg(0).to_string();
     return as_value(ptr->fputs(str));
 }
 
@@ -496,6 +544,26 @@ fileio_ftell(const fn_call& fn)
     return as_value(i);
 }
 
+as_value 
+fileio_asyncmode(const fn_call& fn)
+{
+//    GNASH_REPORT_FUNCTION;
+    boost::intrusive_ptr<Fileio> ptr = ensureType<Fileio>(fn.this_ptr);
+    assert(ptr);
+    bool b = (bool) fn.arg(0).to_bool();
+    return as_value(ptr->asyncmode(b));
+}
+
+as_value
+fileio_feof(const fn_call& fn)
+{
+//    GNASH_REPORT_FUNCTION;
+    boost::intrusive_ptr<Fileio> ptr = ensureType<Fileio>(fn.this_ptr);
+    assert(ptr);
+    bool b = ptr->feof();
+    return as_value(b);
+}
+
 as_value
 fileio_unlink(const fn_call& fn)
 {
@@ -542,12 +610,7 @@ extern "C" {
  	    //attachInterface(*cl);
 	}
 	
-	VM& vm = VM::get(); // cache this ?
-	std::string name = "FileIO";
-	if ( vm.getSWFVersion() < 7 ) {
-	    boost::to_lower(name, vm.getLocale());
-	}
-	obj.init_member(name, cl.get());
+	obj.init_member("FileIO", cl.get());
     }
 } // end of extern C
 
