@@ -1,9 +1,9 @@
 //
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
@@ -15,13 +15,17 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-//
-//
+#ifndef __GTK_GLUE_H__
+#define __GTK_GLUE_H__
 
 #include "gnash.h"
 
 #include <gtk/gtk.h>
+#ifndef _WIN32
 #include <gdk/gdkx.h>
+#else
+#include <gdk/gdk.h>
+#endif
 
 namespace gnash
 {
@@ -40,8 +44,12 @@ class GtkGlue
 			{ render();	};
     virtual void configure(GtkWidget *const widget,
                            GdkEventConfigure *const event) = 0;
+    virtual void beforeRendering() { /* nop */ };
   protected:
     GtkWidget *_drawing_area;
 };
 
 } // namespace gnash
+
+// end of __GTK_GLUE_H__
+#endif
