@@ -18,7 +18,11 @@
 #include "gtk_glue.h"
 
 #include <gtk/gtk.h>
+#ifndef _WIN32
 #include <gdk/gdkx.h>
+#else
+#include <gdk/gdk.h>
+#endif
 
 # include <cairo.h>
 
@@ -35,9 +39,7 @@ class GtkCairoGlue : public GtkGlue
     void prepDrawingArea(GtkWidget *drawing_area);
     render_handler* createRenderHandler();
     void render();
-    void render(int /*minx*/, int /*miny*/, int /*maxx*/, int /*maxy*/) {
-	render();
-    };
+    void render(int minx, int miny, int maxx, int maxy);
     void configure(GtkWidget *const widget, GdkEventConfigure *const event);
   private:
     cairo_t     *_cairo_handle;
