@@ -22,13 +22,15 @@
 #include "gnashconfig.h"
 #endif
 
-#include "tu_config.h"
 #include "rect.h"  // for composition
 #include "snappingrange.h"  // for InvalidatedRanges
 #include "gnash.h" // for gnash::key::code type
-#include "tree.hh" // for tree
 #include "smart_ptr.h"
 
+#ifdef USE_SWFTREE
+#include "tree.hh" // for tree
+#endif
+
 #include <string>
 #include <map>
 
@@ -37,6 +39,20 @@
 // This is an experimental feature, so it's off by default
 //#define SKIP_RENDERING_IF_LATE
 
+
+/// Define this to disable region updates debugging altogether. If undefined,
+/// debugging will be a runtime option. The flag and flag-setting functions
+/// will not be disabled (too ugly).
+///
+/// This should go in gnashconfig.h
+///
+/// This has the side effect that all frames will be re-rendered completely
+/// but in contrast to FORCE_REDRAW it won't re-render when no motion
+/// has been detected in the movie (for example when the movie is stopped).
+///
+//#define DISABLE_REGION_UPDATES_DEBUGGING 1
+
+
 /// Define this to support keyboard-based pointer movements
 #define ENABLE_KEYBOARD_MOUSE_MOVEMENTS 1
 
@@ -59,7 +75,7 @@ enum gnash_cursor_type {
 };
 
 /// Parent class from which all GUI implementations will depend.
-class DSOEXPORT Gui {
+class Gui {
 
 public:
 
@@ -159,11 +175,13 @@ public:
     /// Sets the current mouse cursor for the Gui window.
     virtual void setCursor(gnash_cursor_type newcursor);
 
-    /// @return The value to which the movie width should be scaled.
-    float getXScale();
-
-    /// @return The value to which the movie height should be scaled.
-    float getYScale();
+    // Information for System.capabilities to be reimplemented in
+    // each gui.
+    virtual double getPixelAspectRatio() { return 0; }
+    virtual int getScreenResX() { return 0; }
+    virtual int getScreenResY() { return 0; }
+    virtual double getScreenDPI() { return 0; }
+    virtual std::string getScreenColor() { return ""; }
 
     /// @return Whether or not the movie should be looped indefinitely.
     bool loops();
@@ -173,8 +191,8 @@ public:
 
     /// Mouse notification callback to be called when the mouse is moved.
     //
-    /// @param x The mouse coordinate X component in pixels.
-    /// @param y The mouse coordinate Y component in pixels.
+    /// @param x The mouse coordinate X component in user/window coordinate space (pixels).
+    /// @param y The mouse coordinate Y component in user/window coordinate space (pixels).
     ///
     void notify_mouse_moved(int x, int y);
 
@@ -201,6 +219,14 @@ public:
     /// @param height The desired height in pixels.
     void resize_view(int width, int height);
 
+    /// Update stage matrix accordingly to window size and flash Stage
+    /// configuration (scaleMode, alignment)
+    //
+    /// This method should be called from the core lib when Stage configuration
+    /// change or is called by resize_view.
+    ///
+    void updateStageMatrix();
+
     /// \brief
     /// Advances the movie to the next frame. This is to take place after the
     /// interval specified in the call to setInterval().
@@ -219,7 +245,7 @@ public:
     /// Attempt to run in a fullscreen window both for plugin and
     /// standalone player.
     //
-    /// Use gui::isFullscreen() to see if gnash thinks
+    /// Use isFullscreen() to see if gnash thinks
     /// it's running in fullscreen or not. The switch to fullscreen may
     /// fail if, for instance, the window manager refuses to allow it, but
     /// the flag will be set anyway.
@@ -229,6 +255,17 @@ public:
     ///
     virtual void unsetFullscreen();
     
+    /// Sets whether the gui should show the system mouse pointer
+    //
+    /// @param show true if the mouse should be shown.
+    /// @return true if the state changed.
+    virtual bool showMouse(bool show);
+
+    /// Sets whether the menus should be shown (for fscommand)
+    //
+    /// @param show true if the menu bar should be shown.
+    virtual void showMenu(bool show);
+    
     // Toggle between fullscreen and normal mode
     void toggleFullscreen();
 
@@ -259,6 +296,14 @@ public:
 
     /// See stop(), play() and pause()
     bool isStopped() const { return _stopped; }
+    
+    /// Whether gnash is is running as a plugin
+    bool isPlugin() const { return (( _xid )); }
+    
+    void setMaxAdvances(unsigned long ul) { if (ul > 0) _maxAdvances = ul; }
+    
+    void showUpdatedRegions(bool x) { _showUpdatedRegions = x; }
+    bool showUpdatedRegions() { return _showUpdatedRegions; }
 
     /** @name Menu callbacks
      *  These callbacks will be called when a menu item is clicked.
@@ -290,8 +335,10 @@ public:
     }
 #endif // def GNASH_FPS_DEBUG
 
+
+#ifdef USE_SWFTREE
     // TODO: use a tree-like structure (tree.hh?)
-    typedef std::pair<std::string,std::string> StringPair;
+    typedef std::pair<std::string, std::string> StringPair;
     typedef tree<StringPair> InfoTree;
 
     /// \brief
@@ -299,6 +346,7 @@ public:
     /// currently being played (or NULL, if the VM isn't initialized yet)
     ///
     std::auto_ptr<InfoTree> getMovieInfo() const;
+#endif
 
     typedef std::map<std::string, std::string> VariableMap;
 
@@ -348,10 +396,6 @@ protected:
     /// Desired window height.
     int             _height;
 
-    /// The window width scale.
-    float           _xscale;
-    /// The window height scale.
-    float           _yscale;
     /// Desired colour depth in bits.
     int             _depth;
     /// Main loop interval: the time between successive advance_movie calls.
@@ -362,10 +406,32 @@ protected:
     /// window size did change.
     bool            _redraw_flag;
     // True if Gnash is running in fullscreen
-    bool	    _fullscreen;
+    bool _fullscreen;
+    // True if mouse pointer is showing
+    bool _mouseShown;
+    // Maximum number of advances before exit; 0 for no limit.
+    unsigned long _maxAdvances;
+
+    /// Called by Gui::stop().  This can be used by GUIs to implement pause
+    /// widgets (so that resuming a stopped animation is more user-friendly)
+    virtual void stopHook() {}
+    /// Called by Gui::play().
+    virtual void playHook() {}
 
 private:
 
+    /// Width of a window pixel, in stage pseudopixel units.
+    float           _xscale;
+
+    /// Height of a window pixel, in stage pseudopixel units.
+    float           _yscale;
+
+    /// Window pixel X offset of stage origin
+    float           _xoffset;
+
+    /// Window pixel Y offset of stage origin
+    float           _yoffset;
+
     bool display(movie_root* m);
     
 #ifdef GNASH_FPS_DEBUG
@@ -412,6 +478,9 @@ private:
     /// True if the application didn't start yet
     bool            _started;
     
+    /// If true, updated regions (invalidated ranges) are visibly outlined.
+    bool _showUpdatedRegions;
+    
 #ifdef ENABLE_KEYBOARD_MOUSE_MOVEMENTS 
 	int _xpointer;
 	int _ypointer;
@@ -429,6 +498,7 @@ std::auto_ptr<Gui> createFLTKGui(unsigned long xid, float scale, bool loop, unsi
 std::auto_ptr<Gui> createFBGui(unsigned long xid, float scale, bool loop, unsigned int depth);
 std::auto_ptr<Gui> createAQUAGui(unsigned long xid, float scale, bool loop, unsigned int depth);
 std::auto_ptr<Gui> createRISCOSGui(unsigned long xid, float scale, bool loop, unsigned int depth);
+std::auto_ptr<Gui> createDumpGui(unsigned long xid, float scale, bool loop, unsigned int depth);
 
  
 } // end of gnash namespace
