@@ -1,5 +1,5 @@
 //
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,15 +15,17 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-//
-//
-
-
-/* $Id: kde_glue.h,v 1.13 2007/07/01 10:54:03 bjacques Exp $ */
+#ifdef HAVE_CONFIG_H
+#include "gnashconfig.h"
+#endif
 
 #include "gnash.h"
 
-#include <qwidget.h>
+#if GNASH_QT_VERSION == 4
+# include <Qt/qwidget.h>
+#else
+# include <qwidget.h>
+#endif
 #include "snappingrange.h"
 
 namespace gnash
