@@ -1,5 +1,5 @@
 //
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,9 +15,6 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-//
-//
-
 #include "sdl_glue.h"
 #include <cairo.h>
 #include <SDL.h>
@@ -38,8 +35,8 @@ class SdlCairoGlue : public SdlGlue
 
     bool init(int argc, char **argv[]);
     render_handler* createRenderHandler( int depth);
-    bool prepDrawingArea(int width, int height, uint32_t sdl_flags);
-    uint32_t maskFlags(uint32_t sdl_flags);
+    bool prepDrawingArea(int width, int height, boost::uint32_t sdl_flags);
+    boost::uint32_t maskFlags(boost::uint32_t sdl_flags);
     void render();
   private:
     cairo_surface_t *_cairo_surface;
