@@ -17,24 +17,83 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-// $Id: FLVParser.cpp,v 1.27 2008/02/15 20:50:43 rsavoye Exp $
 
 #include <string>
 #include <iosfwd>
 #include "FLVParser.h"
 #include "amf.h"
 #include "log.h"
+#include "utility.h"
 
 using namespace std;
 
-#define PADDING_BYTES 8
+#define PADDING_BYTES 64
+#define READ_CHUNKS 64
 
 // Define the following macro the have seek() operations printed
 //#define GNASH_DEBUG_SEEK 1
 
 namespace gnash {
 
-FLVParser::FLVParser(LoadThread& lt)
+static std::auto_ptr<FLVFrame>
+makeVideoFrame(tu_file& in, const FLVVideoFrame& frameInfo)
+{
+	std::auto_ptr<FLVFrame> frame ( new FLVFrame );
+
+	frame->dataSize = frameInfo.dataSize;
+	frame->timestamp = frameInfo.timestamp;
+	frame->tag = 9;
+
+	if ( in.set_position(frameInfo.dataPosition) )
+	{
+		log_error(_("Failed seeking to videoframe in FLV input"));
+		frame.reset();
+		return frame;
+	}
+
+	unsigned long int dataSize = frameInfo.dataSize;
+	unsigned long int chunkSize = smallestMultipleContaining(READ_CHUNKS, dataSize+PADDING_BYTES);
+
+	frame->data = new boost::uint8_t[chunkSize];
+	size_t bytesread = in.read_bytes(frame->data, dataSize);
+
+	unsigned long int padding = chunkSize-dataSize;
+	assert(padding);
+	memset(frame->data + bytesread, 0, padding);
+
+	return frame;
+}
+
+static std::auto_ptr<FLVFrame>
+makeAudioFrame(tu_file& in, const FLVAudioFrame& frameInfo)
+{
+	std::auto_ptr<FLVFrame> frame ( new FLVFrame );
+	frame->dataSize = frameInfo.dataSize; 
+	frame->timestamp = frameInfo.timestamp;
+
+	if ( in.set_position(frameInfo.dataPosition) )
+	{
+		log_error(_("Failed seeking to audioframe in FLV input"));
+		frame.reset();
+		return frame;
+	}
+
+	unsigned long int dataSize = frameInfo.dataSize;
+	unsigned long int chunkSize = smallestMultipleContaining(READ_CHUNKS, dataSize+PADDING_BYTES);
+
+	frame->data = new boost::uint8_t[chunkSize];
+	size_t bytesread = in.read_bytes(frame->data, dataSize);
+
+	unsigned long int padding = chunkSize-dataSize;
+	assert(padding);
+	memset(frame->data + bytesread, 0, padding);
+
+	frame->tag = 8;
+
+	return frame;
+}
+
+FLVParser::FLVParser(tu_file& lt)
 	:
 	_lt(lt),
 	_lastParsedPosition(0),
@@ -163,32 +222,30 @@ FLVFrame* FLVParser::nextMediaFrame()
 
 	if (useAudio) {
 
-		FLVFrame* frame = new FLVFrame;
-		frame->dataSize = _audioFrames[_nextAudioFrame]->dataSize;
-		frame->timestamp = _audioFrames[_nextAudioFrame]->timestamp;
+		FLVAudioFrame* frameInfo = _audioFrames[_nextAudioFrame];
 
-		_lt.seek(_audioFrames[_nextAudioFrame]->dataPosition);
-		frame->data = new boost::uint8_t[frame->dataSize + PADDING_BYTES];
-		size_t bytesread = _lt.read(frame->data, frame->dataSize);
-		memset(frame->data + bytesread, 0, PADDING_BYTES);
+		std::auto_ptr<FLVFrame> frame = makeAudioFrame(_lt, *frameInfo);
+		if ( ! frame.get() )
+		{
+			log_error("Could not make audio frame %d", _nextAudioFrame);
+			return 0;
+		}
 
-		frame->tag = 8;
 		_nextAudioFrame++;
-		return frame;
+		return frame.release(); // TODO: return by auto_ptr
 
 	} else {
-		FLVFrame* frame = new FLVFrame;
-		frame->dataSize = _videoFrames[_nextVideoFrame]->dataSize;
-		frame->timestamp = _videoFrames[_nextVideoFrame]->timestamp;
 
-		_lt.seek(_videoFrames[_nextVideoFrame]->dataPosition);
-		frame->data = new boost::uint8_t[frame->dataSize + PADDING_BYTES];
-		size_t bytesread  = _lt.read(frame->data, frame->dataSize);
-		memset(frame->data + bytesread, 0, PADDING_BYTES);
+		FLVVideoFrame* frameInfo = _videoFrames[_nextVideoFrame];
+		std::auto_ptr<FLVFrame> frame = makeVideoFrame(_lt, *frameInfo);
+		if ( ! frame.get() )
+		{
+			log_error("Could not make video frame %d", _nextVideoFrame);
+			return 0;
+		}
 
-		frame->tag = 9;
 		_nextVideoFrame++;
-		return frame;
+		return frame.release(); // TODO: return by auto_ptr
 	}
 
 
@@ -209,20 +266,16 @@ FLVFrame* FLVParser::nextAudioFrame()
 	// If the needed frame can't be parsed (EOF reached) return NULL
 	if (_audioFrames.size() <= _nextAudioFrame || _audioFrames.size() == 0) return NULL;
 
-	FLVFrame* frame = new FLVFrame;
-	frame->dataSize = _audioFrames[_nextAudioFrame]->dataSize;
-	frame->timestamp = _audioFrames[_nextAudioFrame]->timestamp;
-	frame->tag = 8;
-
-	_lt.seek(_audioFrames[_nextAudioFrame]->dataPosition);
-	frame->data = new boost::uint8_t[_audioFrames[_nextAudioFrame]->dataSize +
-				  PADDING_BYTES];
-	size_t bytesread = _lt.read(frame->data, 
-				_audioFrames[_nextAudioFrame]->dataSize);
-	memset(frame->data + bytesread, 0, PADDING_BYTES);
+	FLVAudioFrame* frameInfo = _audioFrames[_nextAudioFrame];
+	std::auto_ptr<FLVFrame> frame = makeAudioFrame(_lt, *frameInfo);
+	if ( ! frame.get() )
+	{
+		log_error("Could not make audio frame %d", _nextAudioFrame);
+		return 0;
+	}
 
 	_nextAudioFrame++;
-	return frame;
+	return frame.release(); // TODO: return by auto_ptr
 
 }
 
@@ -250,21 +303,17 @@ FLVFrame* FLVParser::nextVideoFrame()
 		return NULL;
 	}
 
-
-	FLVFrame* frame = new FLVFrame;
-	frame->dataSize = _videoFrames[_nextVideoFrame]->dataSize;
-	frame->timestamp = _videoFrames[_nextVideoFrame]->timestamp;
-	frame->tag = 9;
-
-	_lt.seek(_videoFrames[_nextVideoFrame]->dataPosition);
-	frame->data = new boost::uint8_t[_videoFrames[_nextVideoFrame]->dataSize + 
-				  PADDING_BYTES];
-	size_t bytesread = _lt.read(frame->data, 
-				_videoFrames[_nextVideoFrame]->dataSize);
-	memset(frame->data + bytesread, 0, PADDING_BYTES);
+	// TODO: let a function do this
+	FLVVideoFrame* frameInfo = _videoFrames[_nextVideoFrame];
+	std::auto_ptr<FLVFrame> frame = makeVideoFrame(_lt, *frameInfo);
+	if ( ! frame.get() )
+	{
+		log_error("Could not make video frame %d", _nextVideoFrame);
+		return 0;
+	}
 
 	_nextVideoFrame++;
-	return frame;
+	return frame.release(); // TODO: return by auto_ptr
 }
 
 
@@ -518,21 +567,33 @@ bool FLVParser::parseNextFrame()
 	if (_lastParsedPosition == 0 && !parseHeader()) return false;
 
 	// Check if there is enough data to parse the header of the frame
-	if (!_lt.isPositionConfirmed(_lastParsedPosition+14)) return false;
+	//if (!_lt.isPositionConfirmed(_lastParsedPosition+14)) return false;
 
 	// Seek to next frame and skip the size of the last tag
-	_lt.seek(_lastParsedPosition+4);
+	if ( _lt.set_position(_lastParsedPosition+4) )
+	{
+		log_error("FLVParser::parseNextFrame: can't seek to %d", _lastParsedPosition+4);
+		return false;
+	}
 
 	// Read the tag info
 	boost::uint8_t tag[12];
-	_lt.read(tag, 12);
+	int actuallyRead = _lt.read_bytes(tag, 12);
+	if ( actuallyRead < 12 )
+	{
+		if ( actuallyRead )
+			log_error("FLVParser::parseNextTag: can't read tag info (needed 12 bytes, only got %d)", actuallyRead);
+		// else { assert(_lt.get_eof(); } ?
+		_parsingComplete=true;
+		return false;
+	}
 
 	// Extract length and timestamp
 	boost::uint32_t bodyLength = getUInt24(&tag[1]);
 	boost::uint32_t timestamp = getUInt24(&tag[4]);
 
 	// Check if there is enough data to parse the body of the frame
-	if (!_lt.isPositionConfirmed(_lastParsedPosition+15+bodyLength)) return false;
+	//if (!_lt.isPositionConfirmed(_lastParsedPosition+15+bodyLength)) return false;
 
 	_lastParsedPosition += 15 + bodyLength;
 
@@ -543,7 +604,7 @@ bool FLVParser::parseNextFrame()
 		FLVAudioFrame* frame = new FLVAudioFrame;
 		frame->dataSize = bodyLength - 1;
 		frame->timestamp = timestamp;
-		frame->dataPosition = _lt.tell();
+		frame->dataPosition = _lt.get_position();
 		_audioFrames.push_back(frame);
 
 		// If this is the first audioframe no info about the
@@ -567,7 +628,7 @@ bool FLVParser::parseNextFrame()
 		FLVVideoFrame* frame = new FLVVideoFrame;
 		frame->dataSize = bodyLength - 1;
 		frame->timestamp = timestamp;
-		frame->dataPosition = _lt.tell();
+		frame->dataPosition = _lt.get_position();
 		frame->frameType = (tag[11] & 0xf0) >> 4;
 		_videoFrames.push_back(frame);
 
@@ -581,9 +642,18 @@ bool FLVParser::parseNextFrame()
 
 			// Extract the video size from the videodata header
 			if (codec == VIDEO_CODEC_H263) {
-				_lt.seek(frame->dataPosition);
+				if ( _lt.set_position(frame->dataPosition) )
+				{
+					log_error(" Couldn't seek to VideoTag data position");
+					return false;
+				}
 				boost::uint8_t videohead[12];
-				_lt.read(videohead, 12);
+				int actuallyRead = _lt.read_bytes(videohead, 12);
+				if ( actuallyRead < 12 )
+				{
+		log_error("FLVParser::parseNextFrame: can't read H263 video header (needed 12 bytes, only got %d)", actuallyRead);
+		return false;
+				}
 
 				bool sizebit1 = (videohead[3] & 0x02);
 				bool sizebit2 = (videohead[3] & 0x01);
@@ -626,16 +696,26 @@ bool FLVParser::parseNextFrame()
 		}
 
 	} else if (tag[0] == META_TAG) {
+		LOG_ONCE( log_unimpl("FLV MetaTag parser") );
 		// Extract information from the meta tag
 		/*_lt.seek(_lastParsedPosition+16);
 		char* metaTag = new char[bodyLength];
-		_lt.read(metaTag, bodyLength);
+		size_t actuallyRead = _lt.read(metaTag, bodyLength);
+		if ( actuallyRead < bodyLength )
+		{
+			log_error("FLVParser::parseNextTag: can't read metaTag (%d) body (needed %d bytes, only got %d)",
+				META_TAG, bodyLength, actuallyRead);
+			_parsingComplete=true;
+			return false;
+		}
 		amf::AMF* amfParser = new amf::AMF();
 		amfParser->parseAMF(metaTag);*/
 
 	} else {
-		_parsingComplete = true;
-		return false;
+		log_error("Unknown FLV tag type %d", tag[0]);
+		// don't give up, isn't a short read...
+		//_parsingComplete = true;
+		//return false;
 	}
 
 	return true;
@@ -644,11 +724,15 @@ bool FLVParser::parseNextFrame()
 bool FLVParser::parseHeader()
 {
 	// seek to the begining of the file
-	_lt.seek(0);
+	_lt.set_position(0); // seek back ? really ?
 
 	// Read the header
 	boost::uint8_t header[9];
-	_lt.read(header, 9);
+	if ( _lt.read_bytes(header, 9) != 9 )
+	{
+		log_error("FLVParser::parseHeader: couldn't read 9 bytes of header");
+		return false;
+	}
 
 	// Check if this is really a FLV file
 	if (header[0] != 'F' || header[1] != 'L' || header[2] != 'V') return false;
@@ -680,3 +764,4 @@ inline boost::uint32_t FLVParser::getUInt24(boost::uint8_t* in)
 } // end of gnash namespace
 
 #undef PADDING_BYTES
+#undef READ_CHUNKS 
