@@ -8,11 +8,8 @@
 #ifndef GRID_INDEX_H
 #define GRID_INDEX_H
 
-
-#include "tu_config.h"
 #include "container.h"
 
-
 //
 // grid_index_point
 //
@@ -325,7 +322,7 @@ public:
 		}
 
 		// Didn't find entry!  Something is wrong.
-		assert(0);
+		abort();
 	}
 
 	iterator	find(const point_t& location, payload p)
