@@ -26,30 +26,39 @@
 # include <io.h>
 #endif
 
-// Support compilation with (or without) native language support
-#include "gettext.h"	// for internationalization (GNU gettext)
-#define	_(String) gettext (String)
-#define N_(String) gettext_noop (String)
-
 #include "rc.h" // for IF_VERBOSE_* implementation
+#include "dsodefs.h" // for DSOEXPORT
 
 #include <fstream>
 #include <sstream>
 #include <boost/thread/thread.hpp>
 #include <boost/thread/mutex.hpp>
-//#include <boost/format.hpp>
+#include <boost/format.hpp>
 
 // the default name for the debug log
 #define DEFAULT_LOGFILE "gnash-dbg.log"
 #define TIMESTAMP_LENGTH 24             // timestamp length
 #define TIMESTAMP_FORMAT "%Y-%m-%d %H:%M:%S     " // timestamp format
 
+// Support compilation with (or without) native language support
+#include "gettext.h"
+#define	_(String) gettext (String)
+#define N_(String) gettext_noop (String)
+
+// Macro to prevent repeated logging calls for the same
+// event
+#define LOG_ONCE(x) { static bool warned = false; if (!warned) { warned = true; x; } }
+
+// Define to switch between printf-style log formatting
+// and boost::format
+# include <boost/preprocessor/arithmetic/inc.hpp>
+# include <boost/preprocessor/repetition/enum_params.hpp>
+# include <boost/preprocessor/repetition/repeat.hpp>
+# include <boost/preprocessor/repetition/repeat_from_to.hpp>
+# include <boost/preprocessor/seq/for_each.hpp>
+
 namespace gnash {
 
-extern std::ostream& stampon(std::ostream& x);
-extern std::ostream& stampoff(std::ostream& x);
-extern std::ostream& timestamp(std::ostream& x);
-extern std::ostream& datetimestamp(std::ostream& x);
 #define DEBUGLEVEL 2
 
 // This is a basic file logging class
@@ -60,7 +69,7 @@ public:
 
     ~LogFile();
 
-    enum fileState {
+    enum FileState {
         CLOSED,
         OPEN,
         INPROGRESS,
@@ -96,6 +105,14 @@ public:
     ///
     bool closeLog();
 
+    /// Set log filename 
+    //
+    /// If a log file is opened already, it will be closed
+    /// by this call, and will be reopened on next use
+    /// if needed.
+    ///
+    void setLogFilename(const std::string& fname);
+
     // accessors for the verbose level
     void setVerbosity () {
         _verbose++;
@@ -126,21 +143,24 @@ public:
     }
     
     void setStamp (bool b) {
-	_stamp = b;
+        _stamp = b;
     }
 
     bool getStamp () {
         return _stamp;
     }
 
-    void setWriteDisk (bool b) {
-	_write = b;
-    }
+    /// Set whether to write logs to file
+    void setWriteDisk (bool b);
 
     bool getWriteDisk () {
-	return _write;
+        return _write;
     }
     
+    typedef void (*logListener)(const std::string& s);
+    
+    void registerLogCallback(logListener l) { _listener = l; }
+
 private:
     
     /// Open the specified file to write logs on disk
@@ -173,139 +193,124 @@ private:
     std::ofstream	 _outstream;
 
     /// How much output is required: 2 or more gives debug output.
-    static int		 _verbose;
+    int		 _verbose;
 
     /// Whether to dump all SWF actions
-    static bool		 _actiondump;
+    bool		 _actiondump;
 
     /// Whether to dump parser output
-    static bool		 _parserdump;
+    bool		 _parserdump;
 
     /// The state of the log file.
-    fileState _state;
+    FileState _state;
 
     bool		 _stamp;
 
     /// Whether to write the log file to disk.
     bool		 _write;
 
-    std::string		 _filespec;
+    std::string _filespec;
 
-    /// For the ostream << operator
-    friend std::ostream & operator << (std::ostream &os, LogFile& e);
-
-    LogFile& operator << (const std::string &s);
-    LogFile& operator << (std::ostream & (&)(std::ostream &));
-
-    /// Print anything that can be printed on a stringstream
-    //
-    /// This template function could replace ALL but
-    /// operator << (const std::string&) members of
-    /// LogFile class.
-    ///
-    template <class T>
-    LogFile& operator << (const T& any)
-    {
-	    std::stringstream ss;
-	    ss << any;
-	    return *this << ss.str();
-    }
+    std::string _logFilename;
+    
+    logListener _listener;
 
 };
 
-
-DSOEXPORT unsigned char *hexify(unsigned char *p, const unsigned char *s, int length, bool ascii);
-
-#ifdef __GNUC__
-#define GNUC_LOG_ATTRS __attribute__((format (printf, 1, 2)))
-#else
-#define GNUC_LOG_ATTRS
-#endif
-
-/// Log a runtime error
+/// This heap of steaming preprocessor code magically converts
+/// printf-style statements into boost::format messages using templates.
 //
-/// Runtime errors, such as un-openable files, un-allocatable memory,
-/// etc, are logged (for convenience of the user or debugger) this way.
-/// This function is not used to report coding errors; use log_aserror
-/// or log_swferror for that.
-///
-DSOEXPORT void log_error(const char* fmt, ...) GNUC_LOG_ATTRS;
-//DSOEXPORT void logError(const boost::format& fmt);
+/// Macro to feed boost::format strings to the boost::format object,
+/// producing code like this: "% t1 % t2 % t3 ..."
+#define TOKENIZE_FORMAT(z, n, t) % t##n
 
-/// Log a message about unimplemented features.
+/// Macro to add a number of arguments to the templated function
+/// corresponding to the number of template arguments. Produces code
+/// like this: "const T0& t0, const T1& t1, const T2& t2 ..."
+#define TOKENIZE_ARGS(z, n, t) BOOST_PP_COMMA_IF(n) const T##n& t##n
+
+/// This is a sequence of different log message types to be used in
+/// the code. Append the name to log_ to call the function, e.g. 
+/// log_error, log_unimpl.
+#define LOG_TYPES (error) (debug) (unimpl) (aserror) (swferror) (security) (action) (parse) (trace)
+
+/// This actually creates the template functions using the TOKENIZE
+/// functions above. The templates look like this:
 //
-/// This function must be used to warn user about missing Gnash features.
-/// We expect all calls to this function to disappear over time, as we
-/// implement those features of Flash.
-///
-DSOEXPORT void log_unimpl(const char* fmt, ...) GNUC_LOG_ATTRS;
-//DSOEXPORT void logUnimpl(const boost::format& fmt);
-
-/// Use only for explicit user traces
+/// template< typename T0 , typename T1 , typename T2 , typename T3 > 
+/// void
+/// log_security (const T0& t0, const T1& t1, const T2& t2, const T3& t3)
+/// {
+///     if (_verbosity == 0) return;
+///     processLog_security(myFormat(t0) % t1 % t2 % t3);
+/// }
 //
-/// Current users are Global.cpp for _global.trace() and
-/// ASHandlers.cpp for ActionTrace
+/// Only not as nicely indented.
 ///
-DSOEXPORT void log_trace(const char* fmt, ...) GNUC_LOG_ATTRS;
-//DSOEXPORT void logTrace(const boost::format& fmt);
+/// Use "g++ -E log.h" or "gcc log.h" to check.
+#define LOG_TEMPLATES(z, n, data)\
+    template< \
+         BOOST_PP_ENUM_PARAMS(\
+         BOOST_PP_INC(n), typename T)\
+     >\
+    void log_##data (\
+        BOOST_PP_REPEAT(\
+        BOOST_PP_INC(n), \
+        TOKENIZE_ARGS, t)\
+    ) { \
+    if (LogFile::getDefaultInstance().getVerbosity() == 0) return; \
+    processLog_##data(logFormat(t0) \
+    BOOST_PP_REPEAT_FROM_TO(1, \
+        BOOST_PP_INC(n), \
+        TOKENIZE_FORMAT, t));\
+    }\
 
-/// Log debug info
+/// Defines the maximum number of template arguments
 //
-/// Used for function entry/exit tracing.
-///
-DSOEXPORT void log_debug(const char* fmt, ...) GNUC_LOG_ATTRS;
-//DSOEXPORT void logDebug(const boost::format& fmt);
+/// The preprocessor generates templates with 1..ARG_NUMBER
+/// arguments.
+#define ARG_NUMBER 16
 
-/// Log action execution info
+/// Calls the macro LOG_TEMPLATES an ARG_NUMBER number
+/// of times, each time adding an extra typename argument to the
+/// template.
+#define GENERATE_LOG_TYPES(r, _, t) \
+    BOOST_PP_REPEAT(ARG_NUMBER, LOG_TEMPLATES, t)
+
+/// Calls the template generator for each log type in the
+/// sequence LOG_TYPES.
+BOOST_PP_SEQ_FOR_EACH(GENERATE_LOG_TYPES, _, LOG_TYPES)
+
+#undef TOKENIZE_FORMAT
+#undef GENERATE_LOG_TYPES
+#undef LOG_TEMPLATES
+#undef ARG_NUMBER
+
+DSOEXPORT void processLog_error(const boost::format& fmt);
+DSOEXPORT void processLog_unimpl(const boost::format& fmt);
+DSOEXPORT void processLog_trace(const boost::format& fmt);
+DSOEXPORT void processLog_debug(const boost::format& fmt);
+DSOEXPORT void processLog_action(const boost::format& fmt);
+DSOEXPORT void processLog_parse(const boost::format& fmt);
+DSOEXPORT void processLog_security(const boost::format& fmt);
+DSOEXPORT void processLog_swferror(const boost::format& fmt);
+DSOEXPORT void processLog_aserror(const boost::format& fmt);
+
+/// A fault-tolerant boost::format object for logging
 //
-/// Wrap all calls to this function (and other related statements)
-/// into an IF_VERBOSE_ACTION macro, so to allow completely
-/// removing all the overhead at compile time and reduce it
-/// at runtime.
-///
-DSOEXPORT void log_action(const char* fmt, ...) GNUC_LOG_ATTRS;
-//DSOEXPORT void logAction(const boost::format& fmt);
+/// Generally to be used in the LogFile macro BF(), which will also
+/// be recognized by gettext for internationalization and is less
+/// effort to type.
+DSOEXPORT boost::format logFormat(const std::string &str);
 
-/// Log parsing information
+/// Convert a sequence of bytes to hex or ascii format.
 //
-/// Wrap all calls to this function (and other related statements)
-/// into an IF_VERBOSE_PARSE macro, so to allow completely
-/// removing all the overhead at compile time and reduce it
-/// at runtime.
-///
-DSOEXPORT void log_parse(const char* fmt, ...) GNUC_LOG_ATTRS;
-//DSOEXPORT void logParse(const boost::format& fmt);
-
-/// Log security information
-DSOEXPORT void log_security(const char* fmt, ...) GNUC_LOG_ATTRS;
-//DSOEXPORT void logSecurity(const boost::format& fmt);
-
-/// Log a malformed SWF error
-//
-/// This indicates an error in how the binary SWF file was constructed, i.e.
-/// probably a bug in the tools used to build the SWF file.
-///
-/// Wrap all calls to this function (and other related statements)
-/// into an IF_VERBOSE_MALFORMED_SWF macro, so to allow completely
-/// removing all the overhead at compile time and reduce it
-/// at runtime.
-///
-DSOEXPORT void log_swferror(const char* fmt, ...) GNUC_LOG_ATTRS;
-//DSOEXPORT void logSWFError(const boost::format& fmt);
-
-/// Log an ActionScript error
-//
-/// This indicates an error by the programmer who wrote the ActionScript
-/// code, such as too few or too many arguments to a function.
-///
-/// Wrap all calls to this function (and other related statements)
-/// into an IF_VERBOSE_ASCODING_ERRORS macro, so to allow completely
-/// removing all the overhead at compile time and reduce it
-/// at runtime.
-///
-DSOEXPORT void log_aserror(const char* fmt, ...) GNUC_LOG_ATTRS;
-//DSOEXPORT void logASError(const boost::format& fmt);
-
+/// @param bytes    the array of bytes to process
+/// @param length   the number of bytes to read. Callers are responsible
+///                 for checking that length does not exceed the array size.
+/// @param ascii    whether to return in ascii or space-separated hex format.
+/// @return         a string representation of the byte sequence.
+DSOEXPORT std::string hexify(const unsigned char *bytes, size_t length, bool ascii);
 
 // Define to 0 to completely remove parse debugging at compile-time
 #ifndef VERBOSE_PARSE
@@ -375,7 +380,7 @@ public:
     }
 
     ~__Host_Function_Report__(void) {
-	if (LogFile::getDefaultInstance().getVerbosity() >= DEBUGLEVEL+1) {
+	if (LogFile::getDefaultInstance().getVerbosity() >= DEBUGLEVEL + 1) {
 	    log_debug("returning");
 	}
     }
