@@ -17,7 +17,6 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-/* $Id: sharedlib.cpp,v 1.20 2008/02/19 19:20:50 bwy Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "gnashconfig.h"
@@ -108,13 +107,11 @@ SharedLib::SharedLib(const char *filespec)
 //     } else {
 //         log_debug ("Initialized ltdl");
     }
-    char *pluginsdir;
+    const char *pluginsdir = PLUGINSDIR;
     char *env = getenv ("GNASH_PLUGINS");
-    if (env == 0) {
-        pluginsdir = PLUGINSDIR;
-    } else {
+    if (env != NULL)
         pluginsdir = env;
-    }
+   
 
     lt_dlsetsearchpath(pluginsdir);
 }
