@@ -15,7 +15,6 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-/* $Id: plugin.cpp,v 1.97.2.2 2008/02/27 08:47:39 strk Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "gnashconfig.h"
@@ -61,6 +60,7 @@
 #include <string>
 #include <vector>
 #include <iostream>
+#include <sstream>
 
 // Mozilla SDK headers
 #include "prinit.h"
@@ -113,6 +113,14 @@ NPP_GetMIMEDescription(void)
 NPError
 NS_PluginInitialize()
 {
+	if ( plugInitialized )
+	{
+		cout << "NS_PluginInitialize called, but ignored (we already initialized)" << endl;
+		return NPERR_NO_ERROR;
+	}
+
+	cout << "NS_PluginInitialize call ---------------------------------------------------" << endl;
+
 
 	/* Browser Functionality Checks */
 
@@ -166,7 +174,7 @@ NS_PluginInitialize()
 	Check for environment variables.
 	*/
 	char* opts = getenv("GNASH_OPTIONS");
-	if ( opts )
+	if (opts != NULL)
 	{
 		cout << "GNASH_OPTIONS : " << opts << endl;
 		
@@ -178,6 +186,43 @@ NS_PluginInitialize()
 
 	}
 
+	// Append SYSCONFDIR/gnashpluginrc and ~/.gnashpluginrc to GNASHRC
+	do {
+		// TODO: extract content in a set, add to set
+		//       and serialize back (to avoid duplicates)
+
+		std::string newGnashRc;
+		char *gnashrc = getenv("GNASHRC");
+		if ( gnashrc )
+		{
+			newGnashRc.assign(gnashrc);
+			newGnashRc.append(":");
+		}
+
+		newGnashRc.append(SYSCONFDIR);
+		newGnashRc.append("/gnashpluginrc");
+
+		char *home = getenv("HOME");
+		if ( home )
+		{
+			newGnashRc.append(":");
+			newGnashRc.append(home);
+			newGnashRc.append("/.gnashpluginrc");
+		}
+		else
+		{
+			cerr << "WARNING: NPAPI plugin could not find user home dir" << endl;
+		}
+
+		if ( setenv("GNASHRC", newGnashRc.c_str(), 1) )
+		{
+			cerr << "WARNING: NPAPI plugin could not append to the GNASHRC env variable" << endl;
+		}
+		else cout << "NOTE: NPAPI plugin set GNASHRC to " << newGnashRc << endl;
+
+	} while (0);
+
+
 	/* Success */
 
 	plugInitialized = TRUE;
@@ -194,6 +239,7 @@ NS_PluginInitialize()
 void
 NS_PluginShutdown()
 {
+#if 0
 	if (!plugInitialized)
 	{
 		cout << "Plugin already shut down" << endl;
@@ -201,6 +247,7 @@ NS_PluginShutdown()
 	}
 
 	plugInitialized = FALSE;
+#endif
 }
 
 
@@ -218,7 +265,7 @@ NS_PluginGetValue(NPPVariable aVariable, void *aValue)
 	switch (aVariable)
 	{
 		case NPPVpluginNameString:
-			*static_cast<char **> (aValue) = PLUGIN_NAME;
+			*static_cast<const char **> (aValue) = PLUGIN_NAME;
 			break;
 
 		// This becomes the description field you see below the opening
@@ -287,7 +334,8 @@ nsPluginInstance::nsPluginInstance(nsPluginCreateData* data)
 	_ichan(0),
 	_ichanWatchId(0),
 	_childpid(0),
-	_filefd(-1)
+	_filefd(-1),
+	_name()
 {
 	for (size_t i=0, n=data->argc; i<n; ++i)
 	{
@@ -302,9 +350,16 @@ nsPluginInstance::nsPluginInstance(nsPluginCreateData* data)
 		{
 			val = data->argv[i];
 		}
-		//log_debug("PARAM: %s = %s", name.c_str(), val.c_str());
+
+		if ( ! strcasecmp(name.c_str(), "name") )
+		{
+			_name = val;
+		}
+
+		cerr << "PARAM: " << name << " = " << val << endl;
 		_params[name] = val;
 	}
+
 }
 
 /// \brief Destructor
@@ -623,34 +678,66 @@ nsPluginInstance::processPlayerRequest(gchar* buf, gsize linelen)
 		return false;
 	}
 
-	if ( strncmp(buf, "GET ", 4) )
+	if ( ! strncmp(buf, "GET ", 4) )
 	{
-		cout << "Unknown player request: " << buf << endl;
-		return false;
-	}
+		char* target = buf+4;
+		if ( ! *target )
+		{
+			cout << "No target found after GET request" << endl;
+			return false;
+		}
+		char* url = target;
+		while (*url && *url != ':') ++url;
+		if ( *url )
+		{
+			*url='\0';
+			++url;
+		}
+		else
+		{
+			cout << "No colon found after GETURL target string" << endl;
+			return false;
+		}
+
+		cout << "Asked to get URL '" << url << "' in target '" << target << "'" << endl;
+		NPN_GetURL(_instance, url, target);
+		return true;
 
-	char* target = buf+4;
-	if ( ! *target )
-	{
-		cout << "No target found after GET request" << endl;
-		return false;
 	}
-	char* url = target;
-	while (*url && *url != ':') ++url;
-	if ( *url )
+	else if ( ! strncmp(buf, "INVOKE ", 7) )
 	{
-		*url='\0';
-		++url;
+		char* command = buf+7;
+		if ( ! *command ) {
+			cout << "No command found after INVOKE request" << endl;
+			return false;
+		}
+		char* arg = command;
+		while (*arg && *arg != ':') ++arg;
+		if ( *arg ) {
+			*arg='\0';
+			++arg;
+		} else {
+			cout << "No colon found after INVOKE command string" << endl;
+			return false;
+		}
+
+		std::string name = _name; 
+
+		std::stringstream jsurl;
+		jsurl << "javascript:" << name << "_DoFSCommand('" << command << "','" << arg <<"')";
+
+		// TODO: check if _self is a good target for this
+		static const char* tgt = "_self";
+
+		cout << "Calling NPN_GetURL(" << jsurl.str() << ", '" << tgt << "');" << endl;
+		NPN_GetURL(_instance, jsurl.str().c_str(), tgt);
+		return true;
 	}
 	else
 	{
-		cout << "No colon found after target string" << endl;
+		cout << "Unknown player request: '" << buf << "'" << endl;
 		return false;
 	}
-
-	cout << "Asked to get URL '" << url << "' in target '" << target << "'" << endl;
-	NPN_GetURL(_instance, url, target);
-	return true;
 }
 
 void
@@ -658,7 +745,7 @@ nsPluginInstance::startProc(Window win)
 {
 	string procname;
 	char *gnash_env = getenv("GNASH_PLAYER");
-	if (!gnash_env) {
+	if (gnash_env == NULL) {
 		procname = GNASHBINDIR;
 		procname += "/gtk-gnash";
 	}
@@ -966,7 +1053,7 @@ static const char* getPluginDescription()
 	if (!desc)
 	{
 		desc = getenv("GNASH_PLUGIN_DESCRIPTION");
-		if (!desc) desc = PLUGIN_DESCRIPTION;
+		if (desc == NULL) desc = PLUGIN_DESCRIPTION;
 	}
 	return desc;
 }
