@@ -29,12 +29,16 @@
 #include "as_function.h"
 #include "as_value.h" // for enumerateValues
 #include "VM.h" // For string_table
+#include "string_table.h"
 
 #include <utility> // for std::make_pair
 
 // Define the following to enable printing address of each property added
 //#define DEBUG_PROPERTY_ALLOC
 
+// Define this to get verbosity of properties insertion and flags setting
+//#define GNASH_DEBUG_PROPERTY 1
+
 namespace gnash {
 
 PropertyList::PropertyList(const PropertyList& pl)
@@ -136,6 +140,11 @@ PropertyList::reserveSlot(unsigned short slotId, string_table::key name,
 	Property a(name, nsId, as_value());
 	a.setOrder(slotId + 1);
 	_props.insert(a);
+#ifdef GNASH_DEBUG_PROPERTY
+	string_table& st = VM::get().getStringTable();
+	log_debug("Slot for AS property %s in namespace %s inserted with flags %s",
+		st.value(name), st.value(nsId), a.getFlags());
+#endif
 	return true;
 }
 
@@ -153,27 +162,40 @@ PropertyList::getValue(const string_table::key key, as_value& val,
 
 bool
 PropertyList::setValue(string_table::key key, as_value val,
-		as_object& this_ptr, string_table::key nsId)
+		as_object& this_ptr, string_table::key nsId,
+		const as_prop_flags& flagsIfMissing)
 {
 	container::iterator found = iterator_find(_props, key, nsId);
 	
 	if (found == _props.end())
 	{
 		// create a new member
-		Property a(key, nsId, val);
+		Property a(key, nsId, val, flagsIfMissing);
 		// Non slot properties are negative ordering in insertion order
 		a.setOrder(- ++mDefaultOrder - 1);
 		_props.insert(a);
+#ifdef GNASH_DEBUG_PROPERTY
+		string_table& st = VM::get().getStringTable();
+		log_debug("Simple AS property %s in namespace %s inserted with flags %s",
+			st.value(key), st.value(nsId), a.getFlags());
+#endif
 		return true;
 	}
-	if (found->isReadOnly())
+
+	const Property& prop = *found;
+	if (prop.isReadOnly() && ! prop.isDestructive())
 	{
-		log_error(_("Property %s is read-only, not setting it to %s"), 
-			VM::get().getStringTable().value(key).c_str(), val.to_string().c_str());
+		string_table& st = VM::get().getStringTable();
+		log_error(_("Property %s (key %d) in namespace %s (key %d) is read-only %s, not setting it to %s"), 
+			st.value(key), key, st.value(nsId), nsId, prop.getFlags(), val);
 		return false;
 	}
 
-	const_cast<Property*>(&(*found))->setValue(this_ptr, val);
+	// Property is const because the container uses its members
+	// for indexing. We don't use value (only name and namespace)
+	// so this const_cast is safe
+	const_cast<Property&>(prop).setValue(this_ptr, val);
+
 	return true;
 }
 
@@ -184,8 +206,16 @@ PropertyList::setFlags(string_table::key key,
 	container::iterator found = iterator_find(_props, key, nsId);
 	if ( found == _props.end() ) return false;
 
+	as_prop_flags oldFlags = found->getFlags();
+
 	as_prop_flags& f = const_cast<as_prop_flags&>(found->getFlags());
 	return f.set_flags(setFlags, clearFlags);
+
+#ifdef GNASH_DEBUG_PROPERTY
+	string_table& st = VM::get().getStringTable();
+	log_debug("Flags of property %s in namespace %s changed from %s to  %s",
+		st.value(key), st.value(nsId), oldFlags, found->getFlags());
+#endif
 }
 
 std::pair<size_t,size_t>
@@ -194,13 +224,26 @@ PropertyList::setFlagsAll(int setFlags, int clearFlags)
 	size_t success=0;
 	size_t failure=0;
 
+#ifdef GNASH_DEBUG_PROPERTY
+	string_table& st = VM::get().getStringTable();
+#endif
+
 	for (container::iterator it=_props.begin(), far=_props.end(); it != far; ++it)
 	{
+#ifdef GNASH_DEBUG_PROPERTY
+		as_prop_flags oldFlags = it->getFlags();
+#endif
+
 		as_prop_flags& f = const_cast<as_prop_flags&>(it->getFlags());
 		if (f.set_flags(setFlags, clearFlags))
 			++success;
 		else
 			++failure;
+
+#ifdef GNASH_DEBUG_PROPERTY
+		log_debug("Flags of property %s in namespace %s changed from %s to  %s",
+			st.value(it->getName()), st.value(it->getNamespace()), oldFlags, it->getFlags());
+#endif
 	}
 
 	return std::make_pair(success,failure);
@@ -325,21 +368,73 @@ PropertyList::import(const PropertyList& o)
 			Property a = *it;
 			a.setOrder(found->getOrder());
 			_props.replace(found, a);
+#ifdef GNASH_DEBUG_PROPERTY
+			string_table& st = VM::get().getStringTable();
+			log_debug("Property %s in namespace %s replaced on import: new flags %s",
+				st.value(a.getName()), st.value(a.getNamespace()), a.getFlags());
+#endif
 		}
 		else
 		{
 			Property a = *it;
 			a.setOrder(- ++mDefaultOrder - 1);
 			_props.insert(a);
+#ifdef GNASH_DEBUG_PROPERTY
+			string_table& st = VM::get().getStringTable();
+			log_debug("Property %s in namespace %s imported with flags %s",
+				st.value(a.getName()), st.value(a.getNamespace()), a.getFlags());
+#endif
 		}
 	}
 }
 
 bool
 PropertyList::addGetterSetter(string_table::key key, as_function& getter,
-	as_function& setter, string_table::key nsId)
+	as_function* setter, const as_value& cacheVal,
+	const as_prop_flags& flagsIfMissing, string_table::key nsId)
 {
-	Property a(key, nsId, &getter, &setter);
+	Property a(key, nsId, &getter, setter, flagsIfMissing);
+	a.setOrder(- ++mDefaultOrder - 1);
+
+	container::iterator found = iterator_find(_props, key, nsId);
+	if (found != _props.end())
+	{
+		// copy flags from previous member (even if it's a normal member ?)
+		as_prop_flags& f = a.getFlags();
+		f = found->getFlags();
+		a.setCache(found->getCache());
+
+		_props.replace(found, a);
+		//assert ( iterator_find(_props, key, nsId) != _props.end() );
+#ifdef GNASH_DEBUG_PROPERTY
+		string_table& st = VM::get().getStringTable();
+		log_debug("AS GetterSetter %s in namespace %s replaced copying flags %s",
+			st.value(key), st.value(nsId), a.getFlags());
+#endif
+
+	}
+	else
+	{
+		a.setCache(cacheVal);
+		_props.insert(a);
+        	//assert ( iterator_find(_props, key, nsId) != _props.end() );
+#ifdef GNASH_DEBUG_PROPERTY
+		string_table& st = VM::get().getStringTable();
+		log_debug("AS GetterSetter %s in namespace %s inserted with flags %s",
+			st.value(key), st.value(nsId), a.getFlags());
+#endif
+	}
+
+
+	return true;
+}
+
+bool
+PropertyList::addGetterSetter(string_table::key key, as_c_function_ptr getter,
+	as_c_function_ptr setter, const as_prop_flags& flagsIfMissing,
+	string_table::key nsId)
+{
+	Property a(key, nsId, getter, setter, flagsIfMissing);
 	a.setOrder(- ++mDefaultOrder - 1);
 
 	container::iterator found = iterator_find(_props, key, nsId);
@@ -350,13 +445,23 @@ PropertyList::addGetterSetter(string_table::key key, as_function& getter,
 		f = found->getFlags();
 
 		_props.replace(found, a);
-		assert ( iterator_find(_props, key, nsId) != _props.end() );
+		//assert ( iterator_find(_props, key, nsId) != _props.end() );
+#ifdef GNASH_DEBUG_PROPERTY
+		string_table& st = VM::get().getStringTable();
+		log_debug("Native GetterSetter %s in namespace %s replaced copying flags %s",
+			st.value(key), st.value(nsId), a.getFlags());
+#endif
 
 	}
 	else
 	{
 		_props.insert(a);
-        	assert ( iterator_find(_props, key, nsId) != _props.end() );
+        	//assert ( iterator_find(_props, key, nsId) != _props.end() );
+#ifdef GNASH_DEBUG_PROPERTY
+		string_table& st = VM::get().getStringTable();
+		log_debug("Native GetterSetter %s in namespace %s inserted with flags %s",
+			st.value(key), st.value(nsId), a.getFlags());
+#endif
 	}
 
 
@@ -364,16 +469,51 @@ PropertyList::addGetterSetter(string_table::key key, as_function& getter,
 }
 
 bool
-PropertyList::addDestructiveGetterSetter(string_table::key key,
-	as_function& getter, as_function& setter, string_table::key nsId)
+PropertyList::addDestructiveGetter(string_table::key key,
+	as_function& getter, string_table::key nsId,
+	const as_prop_flags& flagsIfMissing)
+{
+	container::iterator found = iterator_find(_props, key, nsId);
+	if (found != _props.end())
+	{
+		string_table& st = VM::get().getStringTable();
+		log_error("Property %s in namespace %s already exists, can't addDestructiveGetter",
+			st.value(key), st.value(nsId));
+		return false; // Already exists.
+	}
+
+	// destructive getter don't need a setter
+	Property a(key, nsId, &getter, (as_function*)0, flagsIfMissing, true);
+	a.setOrder(- ++mDefaultOrder - 1);
+	_props.insert(a);
+
+#ifdef GNASH_DEBUG_PROPERTY
+	string_table& st = VM::get().getStringTable();
+	log_debug("Destructive AS property %s (key %d) in namespace %s (key %d) inserted with flags %s",
+		st.value(key), key, st.value(nsId), nsId, a.getFlags());
+#endif
+
+	return true;
+}
+
+bool
+PropertyList::addDestructiveGetter(string_table::key key,
+	as_c_function_ptr getter, string_table::key nsId,
+	const as_prop_flags& flagsIfMissing)
 {
 	container::iterator found = iterator_find(_props, key, nsId);
 	if (found != _props.end())
 		return false; // Already exists.
 
-	Property a(key, nsId, &getter, &setter, 1);
+	// destructive getter don't need a setter
+	Property a(key, nsId, getter, (as_c_function_ptr)0, flagsIfMissing, true);
 	a.setOrder(- ++mDefaultOrder - 1);
 	_props.insert(a);
+#ifdef GNASH_DEBUG_PROPERTY
+	string_table& st = VM::get().getStringTable();
+	log_debug("Destructive native property %s in namespace %s inserted with flags %s",
+		st.value(key), st.value(nsId), a.getFlags());
+#endif
 	return true;
 }
 
