@@ -18,9 +18,7 @@
 #ifndef GNASH_PROPERTYLIST_H
 #define GNASH_PROPERTYLIST_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
+#include "dsodefs.h" // DSOEXPORT
 
 #include "Property.h" // for templated functions
 #include "as_value.h" // for templated functions
@@ -196,14 +194,18 @@ public:
 	///	so we can't promise constness.
 	///
 	/// @param namespaceId
-	/// The namespace in which this should be entered. If 0 is given,
-	/// this will use the first value found, if it exists.
+	///	The namespace in which this should be entered. If 0 is given,
+	///	this will use the first value found, if it exists.
+	///
+	/// @param flagsIfMissing
+	///	Flags to associate to the property if a new one is created.
 	///
 	/// @return true if the value was successfully set, false
 	///         otherwise (found a read-only property, most likely).
 	///
 	bool setValue(string_table::key key, as_value value,
-			as_object& this_ptr, string_table::key namespaceId = 0);
+			as_object& this_ptr, string_table::key namespaceId = 0,
+			const as_prop_flags& flagsIfMissing=0);
 
 	/// Reserves a slot number for a property
 	///
@@ -277,14 +279,44 @@ public:
 	///	A function to invoke when setting this property's value.
 	///	add_ref will be called on the function.
 	///
+	/// @param cacheVal
+	///	The value to use as a cache. If null uses any cache
+	///	from pre-existing property with same name.
+	///
+	/// @param flagsIfMissing
+	///	Flags to associate to the property if a new one is created.
+	///
 	/// @return true if the property was successfully added, false
 	///         otherwise (property already existent?)
 	///
 	bool addGetterSetter(string_table::key key, as_function& getter,
-		as_function& setter, string_table::key ns = 0);
+		as_function* setter, const as_value& cacheVal,
+		const as_prop_flags& flagsIfMissing=0, string_table::key ns=0);
 
 	/// \brief
-	/// Add a destructive getter/setter property, if not already extant.
+	/// Add a getter/setter property, if not already existing
+	/// (or should we allow override ?)
+	//
+	/// @param key
+	///	Name of the property. Search is case-*sensitive*
+	///
+	/// @param getter
+	///	A function to invoke when this property value is requested.
+	///	add_ref will be called on the function.
+	///
+	/// @param setter
+	///	A function to invoke when setting this property's value.
+	///	add_ref will be called on the function.
+	///
+	/// @return true if the property was successfully added, false
+	///         otherwise (property already existent?)
+	///
+	bool addGetterSetter(string_table::key key, as_c_function_ptr getter,
+		as_c_function_ptr setter, const as_prop_flags& flagsIfMissing,
+		string_table::key ns = 0);
+
+	/// \brief
+	/// Add a destructive getter property, if not already existant.
 	///
 	/// @param key
 	/// Name of the property. Case-sensitive search.
@@ -292,14 +324,34 @@ public:
 	/// @param getter
 	/// A function to invoke when this property value is requested.
 	///
-	/// @param setter
-	/// A function to invoke when setting this property's value.
+	/// @param flagsIfMissing
+	///	Flags to associate to the property if a new one is created.
 	///
 	/// @return true if the property was successfully added, false
 	/// otherwise.
 	///
-	bool addDestructiveGetterSetter(string_table::key key,
-		as_function& getter, as_function& setter, string_table::key ns = 0);
+	bool addDestructiveGetter(string_table::key key,
+		as_function& getter, string_table::key ns = 0,
+		const as_prop_flags& flagsIfMissing=0);
+
+	/// \brief
+	/// Add a destructive getter property, if not already existant.
+	///
+	/// @param key
+	/// Name of the property. Case-sensitive search.
+	///
+	/// @param getter
+	/// A function to invoke when this property value is requested.
+	///
+	/// @param flagsIfMissing
+	///	Flags to associate to the property if a new one is created.
+	///
+	/// @return true if the property was successfully added, false
+	/// otherwise.
+	///
+	bool addDestructiveGetter(string_table::key key,
+		as_c_function_ptr getter, string_table::key ns = 0,
+		const as_prop_flags& flagsIfMissing=0);
 
 	/// Set the flags of a property.
 	//
@@ -440,7 +492,7 @@ public:
 
 	/// Mark all simple properties, getters and setters
 	/// as being reachable (for the GC)
-	void setReachable() const;
+	void DSOEXPORT setReachable() const;
 
 private:
 	container _props;
