@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,9 +15,6 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-
 #ifndef _GNASH_URLACCESSMANAGER_H
 #define _GNASH_URLACCESSMANAGER_H
 
@@ -36,13 +33,14 @@ namespace URLAccessManager {
 
 /// Return true if access to given url is allowed, false otherwise.
 //
-/// Will use rc file for whitelist/blacklist.
+/// Will use rc file for whitelist/blacklist and localSendbox.
 ///
 bool allow(const URL& url);
 
-bool allowHost(const std::string& str);
+//bool allowHost(const std::string& str);
 
-bool allowHost(const std::string& str, short port);
+// Used by XMLSocket::connect (and others?)
+bool allowXMLSocket(const std::string& str, short port);
 
 } // AccessManager
 
