@@ -17,13 +17,7 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-/* $Id: as_environment.cpp,v 1.125 2008/02/19 19:20:52 bwy Exp $ */
-
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
-#include "smart_ptr.h"
+#include "smart_ptr.h" // GNASH_USE_GC
 #include "as_environment.h"
 #include "sprite_instance.h"
 #include "shape_character_def.h"
@@ -356,7 +350,7 @@ as_environment::set_variable_raw(
         {
             // const_cast needed due to non-const as_object::get_member 
             as_object* obj = const_cast<as_object*>(scopeStack[i-1].get());
-            if (obj && obj->update_member(varkey, val).first )
+            if (obj && obj->set_member(varkey, val, 0, true) )
             {
 		return;
             }
@@ -375,7 +369,7 @@ as_environment::set_variable_raw(
         {
             // const_cast needed due to non-const as_object::get_member 
             as_object* obj = const_cast<as_object*>(scopeStack[i-1].get());
-            if (obj && obj->update_member(varkey, val).first )
+            if (obj && obj->set_member(varkey, val, 0, true))
             {
 		return;
             }
@@ -559,8 +553,8 @@ as_environment::find_object(const std::string& path_in, const ScopeStack* scopeS
     const char*	p = path.c_str();
     if (*p == '/')
     {
-	// Absolute path.  Start at the root.
-	sprite_instance* root = m_target->get_root();
+	// Absolute path.  Start at the (AS) root (handle _lockroot)
+	sprite_instance* root = const_cast<sprite_instance*>(m_target->getAsRoot());
 	if ( ! *(++p) )
 	{
 #ifdef DEBUG_TARGET_FINDING 
@@ -743,6 +737,9 @@ as_environment::dump_local_registers(std::ostream& out) const
 {
 	if ( _localFrames.empty() ) return;
 	out << "Local registers: ";
+#ifndef DUMP_LOCAL_REGISTERS_IN_ALL_CALL_FRAMES
+	dump(_localFrames.back().registers, out);
+#else
 	for (CallStack::const_iterator it=_localFrames.begin(),
 			itEnd=_localFrames.end();
 			it != itEnd; ++it)
@@ -750,6 +747,7 @@ as_environment::dump_local_registers(std::ostream& out) const
 		if ( it != _localFrames.begin() ) out << " | ";
 		dump(it->registers, out);
 	}
+#endif
 	out << std::endl;
 }
 
@@ -776,6 +774,9 @@ as_environment::dump_local_variables(std::ostream& out) const
 {
 	if ( _localFrames.empty() ) return;
 	out << "Local variables: ";
+#ifndef DUMP_LOCAL_VARIABLES_IN_ALL_CALL_FRAMES
+	dump(_localFrames.back().locals, out);
+#else
 	for (CallStack::const_iterator it=_localFrames.begin(),
 			itEnd=_localFrames.end();
 			it != itEnd; ++it)
@@ -783,6 +784,7 @@ as_environment::dump_local_variables(std::ostream& out) const
 		if ( it != _localFrames.begin() ) out << " | ";
 		dump(it->locals, out);
 	}
+#endif
 	out << std::endl;
 }
 
@@ -918,6 +920,28 @@ as_environment::CallFrame::CallFrame(as_function* funcPtr)
 {
 }
 
+void
+as_environment::dump_stack(std::ostream& out, unsigned int limit) const
+{
+	unsigned int si=0, n=m_stack.size();
+	if ( limit && n > limit )
+	{
+		si=n-limit;
+		out << "Stack (last " << limit << " of " << n << " items): ";
+	}
+	else
+	{
+		out << "Stack: ";
+	}
+
+	for (unsigned int i=si; i<n; i++)
+	{
+		if (i!=si) out << " | ";
+		out << '"' << m_stack[i] << '"';
+	}
+	out << std::endl;
+}
+
 #ifdef GNASH_USE_GC
 /// Mark all reachable resources
 //
