@@ -1,10 +1,10 @@
 // as_function.cpp:  ActionScript Functions, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -17,10 +17,7 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 // 
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
+#include "smart_ptr.h" // GNASH_USE_GC
 #include "log.h"
 #include "as_function.h"
 #include "builtin_function.h" // for _global.Function
@@ -30,6 +27,8 @@
 #include "fn_call.h"
 #include "GnashException.h"
 #include "VM.h"
+#include "Object.h" // for getObjectInterface
+#include "namedStrings.h"
 
 #include <typeinfo>
 #include <iostream>
@@ -71,9 +70,18 @@ static as_object* getFunctionPrototype()
 	static boost::intrusive_ptr<as_object> proto;
 
 	if ( proto.get() == NULL ) {
+
 		// Initialize Function prototype
 		proto = new as_object();
 
+		// We initialize the __proto__ member separately, as getObjectInterface
+		// will end up calling getFunctionPrototype again and we want that
+		// call to return the still-not-completely-constructed prototype rather
+		// then create a new one. 
+		proto->set_prototype(getObjectInterface());
+
+		VM::get().addStatic(proto.get());
+
 		if ( VM::get().getSWFVersion() >= 6 )
 		{
 			proto->init_member("apply", new builtin_function(function_apply));
@@ -89,48 +97,60 @@ static as_value
 function_ctor(const fn_call& /* fn */)
 {
 	boost::intrusive_ptr<as_object> func = new as_object(getFunctionPrototype());
-	//log_msg(_("User tried to invoke new Function()"));
+	//log_debug(_("User tried to invoke new Function()"));
 	return as_value(func.get());
 }
 
+as_function::as_function()
+	:
+	// all functions inherit from global Function class
+	//as_object(getFunctionPrototype())
+	as_object()
+{
+	int flags = as_prop_flags::dontDelete|as_prop_flags::dontEnum|as_prop_flags::onlySWF6Up;
+	init_member(NSV::PROP_uuPROTOuu, as_value(getFunctionPrototype()), flags);
+}
 
 
 // What if we want a function to inherit from Object instead ?
 as_function::as_function(as_object* iface)
 	:
 	// all functions inherit from global Function class
-	as_object(getFunctionPrototype()),
-	_properties(iface)
+	//as_object(getFunctionPrototype())
+	as_object()
 {
-	/// TODO: create properties lazily, on getPrototype() call
-	if ( ! _properties )
+	int flags = as_prop_flags::dontDelete|as_prop_flags::dontEnum|as_prop_flags::onlySWF6Up;
+	init_member(NSV::PROP_uuPROTOuu, as_value(getFunctionPrototype()), flags);
+
+	if ( iface )
 	{
-		_properties = new as_object();
+		iface->init_member(NSV::PROP_CONSTRUCTOR, this); 
+		init_member(NSV::PROP_PROTOTYPE, as_value(iface));
 	}
-	_properties->init_member("constructor", this); 
-	init_member("prototype", as_value(_properties.get()));
 }
 
 void
 as_function::setPrototype(as_object* proto)
 {
-	_properties = proto;
-	init_member("prototype", as_value(_properties.get()));
+	//_properties = proto;
+	init_member(NSV::PROP_PROTOTYPE, as_value(proto));
 }
 
 void
 as_function::extends(as_function& superclass)
 {
-	_properties = new as_object(superclass.getPrototype());
-	_properties->init_member("constructor", &superclass); 
+	as_object* newproto = new as_object(superclass.getPrototype().get());
+	newproto->init_member(NSV::PROP_uuPROTOuu, superclass.getPrototype().get());
+
 	if ( VM::get().getSWFVersion() > 5 )
 	{
-		_properties->init_member("__constructor__", &superclass); 
+		newproto->init_member(NSV::PROP_uuCONSTRUCTORuu, &superclass); 
 	}
-	init_member("prototype", as_value(_properties.get()));
+
+	init_member(NSV::PROP_PROTOTYPE, as_value(newproto));
 }
 
-as_object*
+boost::intrusive_ptr<as_object>
 as_function::getPrototype()
 {
 	// TODO: create if not available ?
@@ -138,26 +158,24 @@ as_function::getPrototype()
 	//               this function should likely return the *new*
 	//               prototype, not the old !!
 	as_value proto;
-	get_member("prototype", &proto);
-	if ( proto.to_object() != _properties.get() )
-	{
-		log_debug(_("Exported interface of function %p "
-				"has been overwritten (from %p to %p)"),
-				this, _properties.get(),
-				(void*)proto.to_object().get());
-		_properties = proto.to_object();
-	}
-	return _properties.get();
+	get_member(NSV::PROP_PROTOTYPE, &proto);
+	return proto.to_object();
 }
 
 /* static public */
 boost::intrusive_ptr<builtin_function>
 as_function::getFunctionConstructor()
 {
-	static boost::intrusive_ptr<builtin_function> func=new builtin_function(
-		function_ctor, // function constructor doesn't do anything
-		getFunctionPrototype() // exported interface
-		);
+	static boost::intrusive_ptr<builtin_function> func = NULL;
+	if ( ! func )
+	{
+		func = new builtin_function(
+			function_ctor, // function constructor doesn't do anything
+			getFunctionPrototype(), // exported interface
+			true // use "this" as constructor
+			);
+		VM::get().addStatic(func.get());
+	}
 	return func;
 }
 
@@ -249,11 +267,9 @@ function_apply(const fn_call& fn)
 			unsigned int nelems = arg_array->size();
 
 			//log_error(_("Function.apply(this_ref, array[%d])\n"), nelems);
-			as_value index, value;
 			for (unsigned int i=nelems; i; i--)
 			{
-				value=arg_array->at(i-1);
-				fn.env().push_val(value);
+				fn.env().push(arg_array->at(i-1));
 				pushed++;
 			}
 
@@ -286,14 +302,46 @@ function_call(const fn_call& fn)
 
 	if ( ! fn.nargs )
 	{
-                log_msg (_("Function.call() with no args"));
+                log_debug (_("Function.call() with no args"));
 		new_fn_call.nargs=0;
 	}
 	else
 	{
 		// Get the object to use as 'this' reference
-		boost::intrusive_ptr<as_object> this_ptr = fn.arg(0).to_object();
-		new_fn_call.this_ptr = this_ptr;
+		as_value this_val = fn.arg(0);
+		boost::intrusive_ptr<as_object> this_ptr = this_val.to_object();
+
+		if ( ! this_ptr )
+		{
+			// If the first argument is not an object, we should
+			// not pass an object to the function, which I belive
+			// should be allowed (but gnash code is not ready).
+			// Anyway, the 'this' label inside the function should 
+			// then be a strange object in that typeof() would return
+			// 'object' but when compared to undefined matches !!
+			// See actionscript.all/Function.as
+			IF_VERBOSE_ASCODING_ERRORS(
+			log_aserror(_("First argument to Function.call(%s) doesn't cast to object. "
+				"Gnash will keep the current 'this' pointer as it is, "
+				"but this is known to not be the correct way to handle "
+				"such a malformed call."), this_val.to_debug_string().c_str());
+			);
+		}
+		else
+		{
+			new_fn_call.this_ptr = this_ptr;
+			as_object* proto = this_ptr->get_prototype().get();
+			if ( proto )
+			{
+				new_fn_call.super = proto->get_super();
+			}
+			else
+			{
+				// TODO: check this !
+				log_debug("No prototype in 'this' pointer passed to Function.call");
+				new_fn_call.super = function_obj->get_super();
+			}
+		}
 		new_fn_call.nargs--;
 		new_fn_call.set_offset(new_fn_call.offset()-1);
 	}
@@ -301,9 +349,9 @@ function_call(const fn_call& fn)
 	// Call the function 
 	return (*function_obj)(new_fn_call);
 
-	//log_msg(_("at function_call exit, stack: \n")); fn.env->dump_stack();
+	//log_debug(_("at function_call exit, stack: \n")); fn.env->dump_stack();
 
-	//log_msg(_("%s: tocheck \n"), __FUNCTION__);
+	//log_debug(_("%s: tocheck \n"), __FUNCTION__);
 }
 
 boost::intrusive_ptr<as_object>
@@ -312,12 +360,22 @@ as_function::constructInstance( as_environment& env,
 {
 //	GNASH_REPORT_FUNCTION;
 
+#ifndef GNASH_USE_GC
 	assert(get_ref_count() > 0);
+#endif // GNASH_USE_GC
 
 	int swfversion = VM::get().getSWFVersion();
 
 	boost::intrusive_ptr<as_object> newobj;
 
+	as_value us;
+	bool has_proto = false;
+	get_member(NSV::PROP_PROTOTYPE, &us);
+	if (!us.is_undefined())
+	{
+		has_proto = true;
+	}
+
         // a built-in class takes care of assigning a prototype
 	// TODO: change this
         if ( isBuiltin() )
@@ -328,20 +386,23 @@ as_function::constructInstance( as_environment& env,
 		);
 
 		fn_call fn(NULL, &env, nargs, first_arg_index);
-		newobj = call(fn).to_object();
+		try {
+			newobj = call(fn).to_object();
+		} catch (std::exception& ex) {
+			log_debug("Native function called as constructor returned %s", ex.what());
+			newobj = new as_object();
+		}
 		assert(newobj); // we assume builtin functions do return objects !!
 
 		// Add a __constructor__ member to the new object, but only for SWF6 up
 		// (to be checked). NOTE that we assume the builtin constructors
 		// won't set __constructor__ to some other value...
-		if ( swfversion > 5 )
-		{
-			newobj->init_member("__constructor__", as_value(this));
+		int flags = as_prop_flags::dontEnum|as_prop_flags::onlySWF6Up; // can delete, hidden in swf5
+		newobj->init_member(NSV::PROP_uuCONSTRUCTORuu, as_value(this), flags);
 
-			if ( swfversion == 6 )
-			{
-				newobj->init_member("constructor", as_value(this));
-			}
+		if ( swfversion < 7 )
+		{
+			newobj->init_member(NSV::PROP_CONSTRUCTOR, as_value(this), flags);
 		}
 
         }
@@ -349,12 +410,16 @@ as_function::constructInstance( as_environment& env,
 	{
 		// Set up the prototype.
 		as_value	proto;
-		// We can safaly call as_object::get_member here as member name is 
+
+		// We can safely call as_object::get_member here as member name is 
 		// a literal string in lowercase. (we should likely avoid calling
 		// get_member as a whole actually, and use a getProto() or similar
 		// method directly instead) TODO
-		bool func_has_prototype = get_member("prototype", &proto);
-		assert(func_has_prototype);
+		/*bool func_has_prototype=*/ get_member(NSV::PROP_PROTOTYPE, &proto);
+
+		// user could have dropped the prototype..
+		// see construct-properties-#.swf from swfdec testsuite
+		//assert(func_has_prototype);
 
 		IF_VERBOSE_ACTION (
 		log_action(_("constructor prototype is %s"), proto.to_debug_string().c_str());
@@ -365,20 +430,27 @@ as_function::constructInstance( as_environment& env,
 
 		// Add a __constructor__ member to the new object, but only for SWF6 up
 		// (to be checked)
-		if ( swfversion > 5 )
-		{
-			newobj->init_member("__constructor__", as_value(this));
+		int flags = as_prop_flags::dontEnum|as_prop_flags::onlySWF6Up; // can delete, hidden in swf5
+		newobj->init_member(NSV::PROP_uuCONSTRUCTORuu, as_value(this), flags);
 
-			if ( swfversion == 6 )
-			{
-				newobj->init_member("constructor", as_value(this));
-			}
+		if ( swfversion < 7 )
+		{
+			newobj->init_member(NSV::PROP_CONSTRUCTOR, as_value(this), flags);
 		}
 
+		// Super is constructed from this function's prototype
+		as_object* super = NULL;
+		as_object* iface = getPrototype().get(); // this function's prototype
+		if ( iface ) super = iface->get_super();
+
 		// Call the actual constructor function; new_obj is its 'this'.
+
 		// We don't need the function result.
-		call(fn_call(newobj.get(), &env, nargs, first_arg_index));
+		call(fn_call(newobj.get(), &env, nargs, first_arg_index, super));
 	}
+
+	if (!has_proto)
+		set_member(NSV::PROP_PROTOTYPE, as_value(newobj));
     
 	return newobj;
 }
