@@ -1,10 +1,10 @@
 // LoadVars.cpp:  ActionScript "LoadVars" class (HTTP variables), for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
@@ -17,18 +17,13 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "LoadVars.h"
 #include "as_object.h" // for inheritance
 #include "log.h"
 #include "fn_call.h"
-#include "smart_ptr.h" // for boost intrusive_ptr
+#include "smart_ptr.h" // GNASH_USE_GC
 #include "builtin_function.h" // need builtin_function
 #include "as_function.h" // for calling event handlers
-#include "as_environment.h" // for setting up a fn_call
 #include "as_value.h" // for setting up a fn_call
 #include "StreamProvider.h"
 #include "URL.h"
@@ -36,21 +31,24 @@
 #include "tu_file.h"
 #include "timers.h"
 #include "VM.h"
-#include "LoadVariablesThread.h"
+#include "Object.h" // for getObjectInterface
+#include "LoadThread.h"
+#include "namedStrings.h"
+#include "utf8.h"
 
 #include <list>
+#include <boost/algorithm/string/case_conv.hpp>
+
+//#define DEBUG_LOADS 1
 
 namespace gnash {
 
 static as_value loadvars_addrequestheader(const fn_call& fn);
-static as_value loadvars_decode(const fn_call& fn);
 static as_value loadvars_load(const fn_call& fn);
 static as_value loadvars_send(const fn_call& fn);
 static as_value loadvars_sendandload(const fn_call& fn);
 static as_value loadvars_tostring(const fn_call& fn);
 static as_value loadvars_ctor(const fn_call& fn);
-//static as_object* getLoadVarsInterface();
-//static void attachLoadVarsInterface(as_object& o);
 
 //--------------------------------------------
 
@@ -64,7 +62,7 @@ public:
 	/// @param env
 	/// 	Environment to use for event handlers calls
 	///
-	LoadVars(as_environment* env);
+	LoadVars();
 
 	~LoadVars();
 
@@ -115,21 +113,32 @@ public:
 		return _bytesTotal;
 	}
 
-	// Retur number of completed loads
-	unsigned int loaded() const {
-		return _loaded;
+protected:
+
+#ifdef GNASH_USE_GC
+	/// Mark all reachable resources, for the GC
+	//
+	/// There are no special reachable resources here
+	///
+	virtual void markReachableResources() const
+	{
+
+		// Invoke generic as_object marker
+		markAsObjectReachable();
 	}
 
+#endif // GNASH_USE_GC
+
 private:
 
 	/// Forbid copy
 	LoadVars(const LoadVars&)
 		:
 		as_object()
-	{ assert(0); }
+	{ abort(); }
 
 	/// Forbid assignment
-	LoadVars& operator=(const LoadVars&) { assert(0); return *this; }
+	LoadVars& operator=(const LoadVars&) { abort(); return *this; }
 
 	/// Return enumerable property pairs in url-encoded form
 	//
@@ -138,15 +147,6 @@ private:
 	///
 	std::string getURLEncodedProperties();
 
-	/// Return true if a load is currently in progress.
-	//
-	/// NOTE: doesn't lock the _loadRequestsMutex !
-	///
-	bool isLoading() const
-	{
-		return _currentLoad != _loadRequests.end();
-	}
-
 	/// Check for completed loading threads, fire
 	/// new threads if needed.
 	void checkLoads();
@@ -160,149 +160,155 @@ private:
 	///
 	void addLoadVariablesThread(const std::string& urlstr, const char* postdata=NULL);
 
-	/// Process a completed load
-	size_t processLoaded(LoadVariablesThread& lr);
-
-	/// Dispatch load event, if any
-	as_value dispatchLoadEvent();
-
-	/// Dispatch data event, if any
-	as_value dispatchDataEvent();
-
-	void setLoadHandler(as_function* fn) {
-		_onLoad = fn;
-	}
-
-	as_function* getLoadHandler() {
-		return _onLoad.get();
-	}
-
-	as_function* getDataHandler() {
-		return _onData.get();
-	}
-
-	void setDataHandler(as_function* fn) {
-		_onData = fn;
-	}
-
 	static as_value checkLoads_wrapper(const fn_call& fn);
 
-	static as_value loaded_get(const fn_call& fn);
-
-	static as_value onData_getset(const fn_call& fn);
-
 	static as_value onLoad_getset(const fn_call& fn);
 
 	static as_value getBytesLoaded_method(const fn_call& fn);
 
 	static as_value getBytesTotal_method(const fn_call& fn);
 
+	static as_value decode_method(const fn_call& fn);
+
+	static as_value onData_method(const fn_call& fn);
+
+	static as_value onLoad_method(const fn_call& fn);
+
 	boost::intrusive_ptr<as_function> _onLoad;
 
-	boost::intrusive_ptr<as_function> _onData;
-
-	as_environment* _env;
-
 	size_t _bytesTotal;
 
 	size_t _bytesLoaded;
 
 	/// List of load requests
-	typedef std::list<LoadVariablesThread*> LoadVariablesThreads;
-	//typedef boost::ptr_list<LoadVariablesThread> LoadVariablesThreads;
+	typedef std::list<LoadThread*> LoadThreadList;
 
 	/// Load requests queue
 	//
-	/// Scheduling loads are needed because LoadVars
-	/// exposes a getBytesLoaded() and getBytesTotal()
-	/// which prevent parallel loads to properly work
+	/// TODO: 
+	/// Is serialization of loads needed to properly 
+	/// implament getBytesLoaded() and getBytesTotal()
+	/// methods ?
 	/// (ie: values from *which* load should be reported?)
 	///
-	LoadVariablesThreads _loadRequests;
-
-	/// The load currently in progress.
-	//
-	/// When _currentLoad == _loadRequests.end()
-	/// no load is in progress.
-	///
-	LoadVariablesThreads::iterator _currentLoad;
+	LoadThreadList _loadThreads; 
 
 	unsigned int _loadCheckerTimer;
 
-	/// Number of clompleted loads
-	unsigned int _loaded;
 };
 
-LoadVars::LoadVars(as_environment* env)
+LoadVars::LoadVars()
 		:
 		as_object(getLoadVarsInterface()),
-		_env(env),
 		_bytesTotal(0),
 		_bytesLoaded(0),
-		_loadRequests(),
-		_currentLoad(_loadRequests.end()),
-		_loaded(0)
+		_loadThreads(),
+		_loadCheckerTimer(0)
 {
-	//log_msg("LoadVars %p created", this);
+	//log_debug("LoadVars %p created", this);
 }
 
 LoadVars::~LoadVars()
 {
-	//log_msg("Deleting LoadVars %p", this);
-	for ( LoadVariablesThreads::iterator i=_loadRequests.begin(), e=_loadRequests.end();
-			i!=e; ++i)
+	for (LoadThreadList::iterator it=_loadThreads.begin(); it != _loadThreads.end(); ++it)
 	{
-		delete *i;
+		delete *it; // supposedly joins the thread
+	}
+
+	if ( _loadCheckerTimer )
+	{
+		VM& vm = getVM();
+        	vm.getRoot().clear_interval_timer(_loadCheckerTimer);
 	}
 }
 
 void
 LoadVars::checkLoads()
 {
-	/// Process a completed load if any
-	if ( isLoading() && (*_currentLoad)->completed() )
-	{
-		processLoaded(*(*_currentLoad));
-		_loadRequests.pop_front();
-		_currentLoad = _loadRequests.end();
-	}
+#ifdef DEBUG_LOADS
+    static int call=0;
+    log_debug("LoadVars %p checkLoads call %d, _loadThreads: %d", (void *)this, ++call, _loadThreads.size());
+#endif
 
-	if ( ! isLoading() )
-	{
-		if ( ! _loadRequests.empty() )
-		{
-			_currentLoad = _loadRequests.begin();
-			(*_currentLoad)->process();
-		}
-		else
-		{
-			VM::get().getRoot().clear_interval_timer(_loadCheckerTimer);
-		}
-	}
+    if ( _loadThreads.empty() ) return; // nothing to do
+
+    for (LoadThreadList::iterator it=_loadThreads.begin();
+            it != _loadThreads.end(); )
+    {
+        LoadThread* lt = *it;
+
+#ifdef DEBUG_LOADS
+        log_debug("LoadVars loads thread %p got %ld/%ld bytes", (void*)lt, lt->getBytesLoaded(), lt->getBytesTotal() );
+#endif
+        if ( lt->completed() )
+        {
+            size_t xmlsize = _bytesTotal = _bytesLoaded = lt->getBytesTotal();
+            boost::scoped_array<char> buf(new char[xmlsize+1]);
+            size_t actuallyRead = lt->read(buf.get(), xmlsize);
+            if ( actuallyRead != xmlsize )
+			{
+				// This would be either a bug of LoadThread or an expected
+				// possibility which lacks documentation (thus a bug in documentation)
+				//
+#ifdef DEBUG_LOADS
+				log_debug("LoadThread::getBytesTotal() returned %d but ::read(%d) returned %d",
+					xmlsize, xmlsize, actuallyRead);
+#endif
+			}
+            buf[actuallyRead] = '\0';
+
+            // Strip BOM, if any.
+            // See http://savannah.gnu.org/bugs/?19915
+            utf8::TextEncoding encoding;
+            // NOTE: the call below will possibly change 'xmlsize' parameter
+            char* bufptr = utf8::stripBOM(buf.get(), xmlsize, encoding);
+            if ( encoding != utf8::encUTF8 && encoding != utf8::encUNSPECIFIED )
+            {
+                log_unimpl("%s to utf8 conversion in LoadVars input parsing", utf8::textEncodingName(encoding));
+            }
+            as_value dataVal(bufptr); // memory copy here (optimize?)
+
+            it = _loadThreads.erase(it);
+            delete lt; // supposedly joins the thread...
+
+            // might push_front on the list..
+            callMethod(NSV::PROP_ON_DATA, dataVal);
+
+#ifdef DEBUG_LOADS
+            log_debug("Completed load, _loadThreads have now %d elements", _loadThreads.size());
+#endif
+        }
+        else
+        {
+            _bytesLoaded = lt->getBytesLoaded();
+            ++it;
+        }
+    }
+
+    if ( _loadThreads.empty() ) 
+    {
+#ifdef DEBUG_LOADS
+        log_debug("Clearing LoadVars load checker interval timer");
+#endif
+	VM& vm = getVM();
+        vm.getRoot().clear_interval_timer(_loadCheckerTimer);
+        _loadCheckerTimer=0;
+    }
 }
 
 void
 LoadVars::attachLoadVarsInterface(as_object& o)
 {
 	o.init_member("addRequestHeader", new builtin_function(loadvars_addrequestheader));
-	o.init_member("decode", new builtin_function(loadvars_decode));
+	o.init_member("decode", new builtin_function(LoadVars::decode_method));
 	o.init_member("getBytesLoaded", new builtin_function(LoadVars::getBytesLoaded_method));
 	o.init_member("getBytesTotal", new builtin_function(LoadVars::getBytesTotal_method));
 	o.init_member("load", new builtin_function(loadvars_load));
 	o.init_member("send", new builtin_function(loadvars_send));
 	o.init_member("sendAndLoad", new builtin_function(loadvars_sendandload));
 	o.init_member("toString", new builtin_function(loadvars_tostring));
-
-	boost::intrusive_ptr<builtin_function> gettersetter;
-
-	gettersetter = new builtin_function(&LoadVars::onLoad_getset, NULL);
-	o.init_property("onLoad", *gettersetter, *gettersetter);
-
-	gettersetter = new builtin_function(&LoadVars::onData_getset, NULL);
-	o.init_property("onData", *gettersetter, *gettersetter);
-
-	gettersetter = new builtin_function(&LoadVars::loaded_get, NULL);
-	o.init_readonly_property("loaded", *gettersetter);
+	o.init_member("onData", new builtin_function(LoadVars::onData_method));
+	o.init_member("onLoad", new builtin_function(LoadVars::onLoad_method));
 }
 
 as_object*
@@ -311,84 +317,64 @@ LoadVars::getLoadVarsInterface()
 	static boost::intrusive_ptr<as_object> o;
 	if ( ! o )
 	{
-		o = new as_object();
+		o = new as_object(getObjectInterface());
 		attachLoadVarsInterface(*o);
 	}
 	return o.get();
 }
 
-/*private*/
-as_value
-LoadVars::dispatchDataEvent()
-{
-	if ( ! _onData ) return as_value();
-	
-	//log_msg("Calling _onData func");
-	// This would be the function calls "context"
-	// will likely be the same to all events
-	fn_call fn(this, _env, 0, 0);
-
-	return _onData->call(fn);
-}
-
-/* private */
-as_value
-LoadVars::dispatchLoadEvent()
-{
-	if ( ! _onLoad ) return as_value();
-	
-	//log_msg("Calling _onLoad func");
-	// This would be the function calls "context"
-	// will likely be the same to all events
-	fn_call fn(this, _env, 0, 0);
-
-	return _onLoad->call(fn);
-}
-
-/* private */
-size_t
-LoadVars::processLoaded(LoadVariablesThread& lr)
-{
-	typedef LoadVariablesThread::ValuesMap ValuesMap;
-	using std::string;
-
-	ValuesMap& vals = lr.getValues();
-	for  (ValuesMap::iterator it=vals.begin(), itEnd=vals.end();
-			it != itEnd; ++it)
-	{
-		set_member(it->first, as_value(it->second.c_str()));
-		//log_msg("Setting %s == %s", it->first.c_str(), it->second.c_str());
-	}
-
-	_bytesLoaded = lr.getBytesLoaded();
-	_bytesTotal = lr.getBytesTotal();
-	++_loaded;
-
-	dispatchLoadEvent();
-
-	return vals.size();
-}
-
 void
 LoadVars::addLoadVariablesThread(const std::string& urlstr, const char* postdata)
 {
-	if ( _loadRequests.empty() )
-	{
-		//log_msg("addLoadVariablesThread(): new requests, starting timer");
-
-		using boost::intrusive_ptr;
-		intrusive_ptr<builtin_function> loadsChecker = new builtin_function(
-			&LoadVars::checkLoads_wrapper, NULL);
-		Timer timer; timer.setInterval(*loadsChecker, 50, this);
-		_loadCheckerTimer = VM::get().getRoot().add_interval_timer(timer);
-	}
+	set_member(NSV::PROP_LOADED, false);
 
 	URL url(urlstr, get_base_url());
-	if ( postdata ) {
-		_loadRequests.insert( _loadRequests.end(), new LoadVariablesThread(url, postdata) );
-	} else {
-		_loadRequests.insert( _loadRequests.end(), new LoadVariablesThread(url) );
+
+	std::auto_ptr<tu_file> str;
+	if ( postdata ) str.reset ( StreamProvider::getDefaultInstance().getStream(url, std::string(postdata)) );
+	else str.reset ( StreamProvider::getDefaultInstance().getStream(url) );
+
+	if ( ! str.get() ) 
+	{
+		log_error(_("Can't load variables from %s (security?)"), url.str().c_str());
+		return;
+		// TODO: check if this is correct
+		//as_value nullValue; nullValue.set_null();
+		//callMethod(VM::get().getStringTable().find(PROPNAME("onData")), nullValue);
 	}
+
+	log_security(_("Loading variables file from url: '%s'"), url.str().c_str());
+
+	bool startTimer = _loadThreads.empty();
+
+	std::auto_ptr<LoadThread> lt ( new LoadThread );
+	lt->setStream(str);
+
+	// we push on the front to avoid invalidating
+	// iterators when queueLoad is called as effect
+	// of onData invocation.
+	// Doing so also avoids processing queued load
+	// request immediately
+	// 
+	_loadThreads.push_front(lt.get());
+#ifdef DEBUG_LOADS
+	log_debug("Pushed thread %p to _loadThreads, number of LoadVars load threads now: %d", (void*)lt.get(),  _loadThreads.size());
+#endif
+	lt.release();
+
+
+	if ( startTimer )
+	{
+		boost::intrusive_ptr<builtin_function> loadsChecker = \
+			new builtin_function(&LoadVars::checkLoads_wrapper);
+		std::auto_ptr<Timer> timer(new Timer);
+		timer->setInterval(*loadsChecker, 50, this);
+		_loadCheckerTimer = getVM().getRoot().add_interval_timer(timer, true);
+#ifdef DEBUG_LOADS
+		log_debug("Registered LoadVars loads interval %d", _loadCheckerTimer);
+#endif
+    }
+
 }
 
 void
@@ -439,26 +425,6 @@ LoadVars::sendAndLoad(const std::string& urlstr, LoadVars& target, bool post)
 	}
 }
 
-/* private static */
-as_value
-LoadVars::onLoad_getset(const fn_call& fn)
-{
-	boost::intrusive_ptr<LoadVars> ptr = ensureType<LoadVars>(fn.this_ptr);
-
-	if ( fn.nargs == 0 ) // getter
-	{
-		as_function* h = ptr->getLoadHandler();
-		if ( h ) return as_value(h);
-		else return as_value();
-	}
-	else // setter
-	{
-		as_function* h = fn.arg(0).to_as_function();
-		if ( h ) ptr->setLoadHandler(h);
-	}
-	return as_value();
-}
-
 /* private static */
 as_value
 LoadVars::checkLoads_wrapper(const fn_call& fn)
@@ -468,37 +434,6 @@ LoadVars::checkLoads_wrapper(const fn_call& fn)
 	return as_value();
 }
 
-/* private static */
-as_value
-LoadVars::onData_getset(const fn_call& fn)
-{
-
-	boost::intrusive_ptr<LoadVars> ptr = ensureType<LoadVars>(fn.this_ptr);
-
-	if ( fn.nargs == 0 ) // getter
-	{
-		as_function* h = ptr->getDataHandler();
-		if ( h ) return as_value(h);
-		else return as_value();
-	}
-	else // setter
-	{
-		as_function* h = fn.arg(0).to_as_function();
-		if ( h ) ptr->setDataHandler(h);
-	}
-	return as_value();
-}
-
-/* private static */
-as_value
-LoadVars::loaded_get(const fn_call& fn)
-{
-
-	boost::intrusive_ptr<LoadVars> ptr = ensureType<LoadVars>(fn.this_ptr);
-
-	return as_value(ptr->loaded() > 0);
-}
-
 static as_value
 loadvars_addrequestheader(const fn_call& fn)
 {
@@ -508,12 +443,26 @@ loadvars_addrequestheader(const fn_call& fn)
 	return as_value(); 
 }
 
-static as_value
-loadvars_decode(const fn_call& fn)
+as_value
+LoadVars::decode_method(const fn_call& fn)
 {
 	boost::intrusive_ptr<LoadVars> ptr = ensureType<LoadVars>(fn.this_ptr);
-	UNUSED(ptr);
-	log_unimpl (__FUNCTION__);
+
+	if ( ! fn.nargs ) return as_value(false);
+
+	typedef std::map<std::string, std::string> ValuesMap;
+
+	ValuesMap vals;
+
+	URL::parse_querystring(fn.arg(0).to_string(), vals);
+
+	string_table& st = ptr->getVM().getStringTable();
+	for  (ValuesMap::const_iterator it=vals.begin(), itEnd=vals.end();
+			it != itEnd; ++it)
+	{
+		ptr->set_member(st.find(it->first), as_value(it->second.c_str()));
+	}
+
 	return as_value(); 
 }
 
@@ -531,6 +480,47 @@ LoadVars::getBytesTotal_method(const fn_call& fn)
 	return as_value(ptr->getBytesTotal());
 }
 
+as_value
+LoadVars::onData_method(const fn_call& fn)
+{
+	//GNASH_REPORT_FUNCTION;
+
+	as_object* thisPtr = fn.this_ptr.get();
+	if ( ! thisPtr ) return as_value();
+
+	// See http://gitweb.freedesktop.org/?p=swfdec/swfdec.git;a=blob;f=libswfdec/swfdec_initialize.as
+
+	as_value src; src.set_null();
+	if ( fn.nargs ) src = fn.arg(0);
+
+	if ( ! src.is_null() )
+	{
+		VM& vm = thisPtr->getVM();
+		string_table& st = vm.getStringTable();
+		string_table::key decodeKey = st.find("decode"); // add to namedStrings ?
+
+		as_value tmp(true);
+		thisPtr->set_member(NSV::PROP_LOADED, tmp);
+		thisPtr->callMethod(decodeKey, src);
+		thisPtr->callMethod(NSV::PROP_ON_LOAD, tmp);
+	}
+	else
+	{
+		as_value tmp(true);
+		thisPtr->set_member(NSV::PROP_LOADED, tmp);
+		thisPtr->callMethod(NSV::PROP_ON_LOAD, tmp);
+	}
+
+	return as_value();
+}
+
+as_value
+LoadVars::onLoad_method(const fn_call& /*fn*/)
+{
+	//GNASH_REPORT_FUNCTION;
+	return as_value();
+}
+
 static as_value
 loadvars_load(const fn_call& fn)
 {
@@ -602,7 +592,7 @@ loadvars_sendandload(const fn_call& fn)
 	bool post = true;
 	if ( fn.nargs > 2 && fn.arg(2).to_string() == "GET" ) post = false;
 
-	//log_msg("LoadVars.sendAndLoad(%s, %p) called, and returning TRUE", urlstr.c_str(), target.get());
+	//log_debug("LoadVars.sendAndLoad(%s, %p) called, and returning TRUE", urlstr.c_str(), target.get());
 
 	ptr->sendAndLoad(urlstr, *target, post);
 	return as_value(true);
@@ -620,7 +610,14 @@ loadvars_tostring(const fn_call& fn)
 static as_value
 loadvars_ctor(const fn_call& fn)
 {
-	boost::intrusive_ptr<as_object> obj = new LoadVars(&fn.env());
+	boost::intrusive_ptr<as_object> obj = new LoadVars();
+
+	if ( fn.nargs )
+	{
+		std::stringstream ss;
+		fn.dump_args(ss);
+		log_unimpl("new LoadVars(%s) - arguments discarded", ss.str().c_str()); // or ASERROR ?
+	}
 	
 	return as_value(obj.get()); // will keep alive
 }
