@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,19 +15,9 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-//
-
 #ifndef __XML_ATTRS_H__
 #define __XML_ATTRS_H__
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include "tu_config.h"
-
 //#define DEBUG_MEMORY_ALLOCATION 1
 #include "as_object.h" // for inheritance
 
@@ -91,10 +81,10 @@ public:
     int   padding;
 #ifdef DEBUG_MEMORY_ALLOCATION
     xmlattr_as_object() {
-        log_msg("\t\tCreating xmlattr_as_object at %p\n", this);
+        log_debug("\t\tCreating xmlattr_as_object at %p\n", this);
     };
     ~xmlattr_as_object() {
-        log_msg("\tDeleting xmlattr_as_object at %p \n", this);
+        log_debug("\tDeleting xmlattr_as_object at %p \n", this);
     };
 #endif
 };
