@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,14 +18,10 @@
 #ifndef __XMLSOCKET_H__
 #define __XMLSOCKET_H__
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include "tu_config.h"
 #include "xml.h"
 #include "impl.h"
 #include "network.h"
+#include "dsodefs.h"
 
 #include <string>
 
