@@ -18,10 +18,10 @@
 //
 
 #ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
+#include "gnashconfig.h" // USE_SWF_TREE
 #endif
 
-
+#include "smart_ptr.h" // GNASH_USE_GC
 #include "button_character_instance.h"
 #include "button_character_def.h"
 #include "action.h" // for as_standard_member enum
@@ -36,6 +36,7 @@
 #include "GnashException.h" // for shared ActionScript getter-setters
 #include "ExecutableCode.h"
 #include "namedStrings.h"
+#include "Object.h" // for getObjectInterface
 
 /** \page buttons Buttons and mouse behaviour
 
@@ -167,6 +168,51 @@ frame loop:
 
 namespace gnash {
 
+namespace {
+
+class ButtonActionExecutor {
+public:
+	ButtonActionExecutor(as_environment& env)
+		:
+		_env(env)
+	{}
+
+	void operator() (const action_buffer& ab)
+	{
+		ActionExec exec(ab, _env);
+		exec();
+	}
+private:
+	as_environment& _env;
+};
+
+class ButtonActionPusher {
+public:
+	ButtonActionPusher(movie_root& mr, character* this_ptr)
+		:
+		called(false),
+		_mr(mr),
+		_tp(this_ptr)
+	{}
+
+	void operator() (const action_buffer& ab)
+	{
+		_mr.pushAction(ab, boost::intrusive_ptr<character>(_tp));
+		called = true;
+	}
+
+	bool called;
+
+private:
+	movie_root& _mr;
+	character* _tp;
+};
+
+}
+
+// Forward declarations
+static as_object* getButtonInterface();
+
 /// A couple of typedefs to make code neater
 typedef button_character_instance Button;
 typedef boost::intrusive_ptr<Button> ButtonPtr;
@@ -181,75 +227,55 @@ attachButtonInterface(as_object& o)
 {
 	//int target_version = o.getVM().getSWFVersion();
 
-	boost::intrusive_ptr<builtin_function> gettersetter;
+	as_c_function_ptr gettersetter;
 
 	//
 	// Properties (TODO: move to appropriate SWF version section)
 	//
 
-	gettersetter = new builtin_function(&character::x_getset, NULL);
+	gettersetter = &character::x_getset;
 	o.init_property("_x", *gettersetter, *gettersetter);
 
-	gettersetter = new builtin_function(&character::y_getset, NULL);
+	gettersetter = &character::y_getset;
 	o.init_property("_y", *gettersetter, *gettersetter);
 
-	gettersetter = new builtin_function(&character::xscale_getset, NULL);
+	gettersetter = &character::xscale_getset;
 	o.init_property("_xscale", *gettersetter, *gettersetter);
 
-	gettersetter = new builtin_function(&character::yscale_getset, NULL);
+	gettersetter = &character::yscale_getset;
 	o.init_property("_yscale", *gettersetter, *gettersetter);
 
-	gettersetter = new builtin_function(&character::xmouse_get, NULL);
+	gettersetter = &character::xmouse_get;
 	o.init_readonly_property("_xmouse", *gettersetter);
 
-	gettersetter = new builtin_function(&character::ymouse_get, NULL);
+	gettersetter = &character::ymouse_get;
 	o.init_readonly_property("_ymouse", *gettersetter);
 
-	gettersetter = new builtin_function(&character::alpha_getset, NULL);
+	gettersetter = &character::alpha_getset;
 	o.init_property("_alpha", *gettersetter, *gettersetter);
 
-	gettersetter = new builtin_function(&character::visible_getset, NULL);
+	gettersetter = &character::visible_getset;
 	o.init_property("_visible", *gettersetter, *gettersetter);
 
-	gettersetter = new builtin_function(&character::width_getset, NULL);
+	gettersetter = &character::width_getset;
 	o.init_property("_width", *gettersetter, *gettersetter);
 
-	gettersetter = new builtin_function(&character::height_getset, NULL);
+	gettersetter = &character::height_getset;
 	o.init_property("_height", *gettersetter, *gettersetter);
 
-	gettersetter = new builtin_function(&character::rotation_getset, NULL);
+	gettersetter = &character::rotation_getset;
 	o.init_property("_rotation", *gettersetter, *gettersetter);
 
-	gettersetter = new builtin_function(&character::parent_getset, NULL);
+	gettersetter = &character::parent_getset;
 	o.init_property("_parent", *gettersetter, *gettersetter);
 	
-	gettersetter = new builtin_function(&character::target_getset, NULL);
+	gettersetter = &character::target_getset;
 	o.init_property("_target", *gettersetter, *gettersetter);
+
+	gettersetter = character::name_getset;
+	o.init_property("_name", gettersetter, gettersetter);
 	
-
-#if 0
-	gettersetter = new builtin_function(&character::onrollover_getset, NULL);
-	o.init_property("onRollOver", *gettersetter, *gettersetter);
-
-	gettersetter = new builtin_function(&character::onrollout_getset, NULL);
-	o.init_property("onRollOut", *gettersetter, *gettersetter);
-
-	gettersetter = new builtin_function(&character::onpress_getset, NULL);
-	o.init_property("onPress", *gettersetter, *gettersetter);
-
-	gettersetter = new builtin_function(&character::onrelease_getset, NULL);
-	o.init_property("onRelease", *gettersetter, *gettersetter);
-
-	gettersetter = new builtin_function(&character::onreleaseoutside_getset, NULL);
-	o.init_property("onReleaseOutside", *gettersetter, *gettersetter);
-
-	gettersetter = new builtin_function(&character::onload_getset, NULL);
-	o.init_property("onLoad", *gettersetter, *gettersetter);
-#endif
-
-	//--
-
-	gettersetter = new builtin_function(&button_character_instance::enabled_getset, NULL);
+	gettersetter = &button_character_instance::enabled_getset;
 	o.init_property("enabled", *gettersetter, *gettersetter);
 
 }
@@ -267,18 +293,12 @@ button_character_instance::button_character_instance(
 {
 	assert(m_def);
 
-	attachButtonInterface(*this);
+	set_prototype(getButtonInterface());
 
 	// check up presence Key events
-	// TODO: use a service of button_character_def, not this hard-coded thing here
-	for (size_t i = 0, e = m_def->m_button_actions.size(); i < e; ++i)
+	if ( m_def->hasKeyPressHandler() )
 	{
-		// TODO: use labels, not magic numbers here !!
-		if (m_def->m_button_actions[i]->m_conditions & 0xFE00)	// check up on CondKeyPress: UB[7]
-		{
-			_vm.getRoot().add_key_listener(this);
-			break;
-		}
+		_vm.getRoot().add_key_listener(this);
 	}
 
 }
@@ -330,48 +350,31 @@ button_character_instance::enabled_getset(const fn_call& fn)
 bool
 button_character_instance::on_event(const event_id& id)
 {
-
-	if( (id.m_id==event_id::KEY_PRESS) && (id.keyCode == key::INVALID) )
+	if ( isUnloaded() )
 	{
-		// onKeypress only responds to valid key code
-		return false;
+		// We dont' respond to events while unloaded
+		// See bug #22982
+		log_debug("Button %s received %s event while unloaded: ignored",
+			getTarget(), id.get_function_name());
+		return false; 
 	}
 
-	bool called = false;
+	// We only respond keypress events
+	if ( id.m_id != event_id::KEY_PRESS ) return false;
 
-	// Add appropriate actions to the global action list ...
-	// TODO: should we execute immediately instead ?
-	for (size_t i = 0, ie=m_def->m_button_actions.size(); i<ie; ++i)
-	{
-		button_action& ba = *(m_def->m_button_actions[i]);
+	// We only respond to valid key code (should we assert here?)
+	if ( id.keyCode == key::INVALID ) return false;
 
-		int keycode = (ba.m_conditions & 0xFE00) >> 9;
-		
-		// Test match between button action conditions and the SWF code
-		// that maps to id.keyCode (the gnash unique key code). 
-		if (id.m_id == event_id::KEY_PRESS && gnash::key::codeMap[id.keyCode][key::SWF] == keycode)
-		{
-			// Matching action.
-			VM::get().getRoot().pushAction(ba.m_actions, boost::intrusive_ptr<character>(this));
-			called = true;
-		}
-	}
+	ButtonActionPusher xec(getVM().getRoot(), this); 
+	m_def->forEachTrigger(id, xec);
 
-	return called;
+	return xec.called;
 }
 
 void
 button_character_instance::restart()
 {
-  set_invalidated();
-	m_last_mouse_flags = IDLE;
-	m_mouse_flags = IDLE;
-	m_mouse_state = UP;
-	size_t r, r_num =  m_record_character.size();
-	for (r = 0; r < r_num; r++)
-	{
-		m_record_character[r]->restart();
-	}
+	log_error("button_character_instance::restart called, from whom??");
 }
 
 void
@@ -381,6 +384,10 @@ button_character_instance::display()
 
 	std::vector<character*> actChars;
 	get_active_characters(actChars);
+
+	//log_debug("At display time, button %s got %d currently active chars", getTarget(), actChars.size());
+
+	// TODO: by keeping chars sorted by depth we'd avoid the sort on display
 	std::sort(actChars.begin(), actChars.end(), charDepthLessThen);
 
 	std::for_each(actChars.begin(), actChars.end(), std::mem_fun(&character::display)); 
@@ -429,8 +436,7 @@ button_character_instance::get_topmost_mouse_entity(float x, float y)
 	//-------------------------------------------------
 
 	// Find hit characters
-	Chars hitChars;
-	get_active_characters(hitChars, HIT);
+	const CharsVect& hitChars = getHitCharacters();
 	if ( hitChars.empty() ) return 0;
 
 	// point is in parent's space,
@@ -444,7 +450,7 @@ button_character_instance::get_topmost_mouse_entity(float x, float y)
 
 	for (size_t i=0, e=hitChars.size(); i<e; ++i)
 	{
-		character* ch = hitChars[i];
+		const character* ch = hitChars[i];
 
 		if ( ch->pointInVisibleShape(wp.x, wp.y) )
 		{
@@ -460,7 +466,16 @@ button_character_instance::get_topmost_mouse_entity(float x, float y)
 void
 button_character_instance::on_button_event(const event_id& event)
 {
-  e_mouse_state new_state = m_mouse_state;
+	if ( isUnloaded() )
+	{
+		// We dont' respond to events while unloaded
+		// See bug #22982
+		log_debug("Button %s received %s button event while unloaded: ignored",
+			getTarget(), event.get_function_name());
+		return;
+	}
+
+	e_mouse_state new_state = m_mouse_state;
   
 	// Set our mouse state (so we know how to render).
 	switch (event.m_id)
@@ -540,134 +555,201 @@ button_character_instance::on_button_event(const event_id& event)
 		}
 	}
 
-
-	// @@ eh, should just be a lookup table.
-	int	c = 0;
-	if (event.m_id == event_id::ROLL_OVER) c |= (button_action::IDLE_TO_OVER_UP);
-	else if (event.m_id == event_id::ROLL_OUT) c |= (button_action::OVER_UP_TO_IDLE);
-	else if (event.m_id == event_id::PRESS) c |= (button_action::OVER_UP_TO_OVER_DOWN);
-	else if (event.m_id == event_id::RELEASE) c |= (button_action::OVER_DOWN_TO_OVER_UP);
-	else if (event.m_id == event_id::DRAG_OUT) c |= (button_action::OVER_DOWN_TO_OUT_DOWN);
-	else if (event.m_id == event_id::DRAG_OVER) c |= (button_action::OUT_DOWN_TO_OVER_DOWN);
-	else if (event.m_id == event_id::RELEASE_OUTSIDE) c |= (button_action::OUT_DOWN_TO_IDLE);
-	//IDLE_TO_OVER_DOWN = 1 << 7,
-	//OVER_DOWN_TO_IDLE = 1 << 8,
-
 	// From: "ActionScript - The Definiteve Guide" by Colin Moock
 	// (chapter 10: Events and Event Handlers)
-
+	//
 	// "Event-based code [..] is said to be executed asynchronously
 	//  because the triggering of events can occur at arbitrary times."
+	//
+	// We'll push to the global list. The movie_root will process
+	// the action queue on mouse event.
+	//
 
-	// Immediately execute all events actions (don't append to
-	// parent's action buffer for later execution!)
+	movie_root& mr = getVM().getRoot();
 
-	for (size_t i = 0; i < m_def->m_button_actions.size(); i++)
-	{
-		button_action& ba = *(m_def->m_button_actions[i]);
-
-		if (ba.m_conditions & c)
-		{
-			// Matching action.
-			action_buffer& ab = ba.m_actions;
-			IF_VERBOSE_ACTION(
-				log_action(_("Executing actions for "
-					"button condition %d"), c);
-			);
-			ActionExec exec(ab, get_environment());
-			exec();
-		}
-	}
+	ButtonActionPusher xec(mr, this); 
+	m_def->forEachTrigger(event, xec);
 
 	// check for built-in event handler.
 	std::auto_ptr<ExecutableCode> code ( get_event_handler(event) );
 	if ( code.get() )
 	{
-		code->execute();
+		//log_debug(_("Got statically-defined handler for event: %s"), event.get_function_name().c_str());
+		mr.pushAction(code, movie_root::apDOACTION);
+		//code->execute();
 	}
-	else
-	{
-		//log_debug(_("No handler for event: %s"), event.get_function_name().c_str());
-	}
-
+	//else log_debug(_("No statically-defined handler for event: %s"), event.get_function_name().c_str());
 
 	// Call conventional attached method.
 	boost::intrusive_ptr<as_function> method = getUserDefinedEventHandler(event.get_function_key());
 	if ( method )
 	{
-		call_method0(as_value(method.get()), &(get_environment()), this);
+		//log_debug(_("Got user-defined handler for event: %s"), event.get_function_name().c_str());
+		mr.pushAction(method, this, movie_root::apDOACTION);
+		//call_method0(as_value(method.get()), &(get_environment()), this);
 	}
+	//else log_debug(_("No statically-defined handler for event: %s"), event.get_function_name().c_str());
 }
 
 void 
-button_character_instance::get_active_characters(std::vector<character*>& list)
-{
-	get_active_characters(list, m_mouse_state);
-}
-
-void 
-button_character_instance::get_active_characters(std::vector<character*>& list,
-  e_mouse_state state)
+button_character_instance::get_active_characters(std::vector<character*>& list, bool includeUnloaded)
 {
 	list.clear();
 	
-	for (size_t i = 0; i < m_def->m_button_records.size(); i++)
+	for (size_t i=0,e=m_record_character.size(); i<e; ++i)
+	{
+		character* ch = m_record_character[i];
+		if (ch == NULL) continue;
+		if ( ! includeUnloaded && ch->isUnloaded() ) continue;
+		list.push_back(ch);
+	} 
+}
+
+void 
+button_character_instance::get_active_records(RecSet& list, e_mouse_state state)
+{
+	list.clear();
+	
+	size_t nrecs = m_def->m_button_records.size();
+
+	//log_debug("%s.get_active_records(%s) - def has %d records", getTarget(), mouseStateName(state), m_def->m_button_records.size());
+	for (size_t i=0; i<nrecs; ++i)
 	{
 		button_record&	rec = m_def->m_button_records[i];
-		assert(m_record_character.size() > i);
-		if (m_record_character[i] == NULL)
-		{
-			continue;
-		}
+		//log_debug(" rec %d has hit:%d down:%d over:%d up:%d", i, rec.m_hit_test, rec.m_down, rec.m_over, rec.m_up);
+
 		if ((state == UP && rec.m_up)
 		    || (state == DOWN && rec.m_down)
 		    || (state == OVER && rec.m_over)
 		    || (state == HIT && rec.m_hit_test))
 		{
-			list.push_back(m_record_character[i].get());
+			list.insert(i);
 		}
-	} // for button record	
+	}
 }
 
+#ifdef GNASH_DEBUG_BUTTON_DISPLAYLIST
+static void dump(std::vector< character* >& chars, std::stringstream& ss)
+{
+	for (size_t i=0, e=chars.size(); i<e; ++i)
+	{
+		ss << "Record" << i << ": ";
+		character* ch = chars[i];
+		if ( ! ch ) ss << "NULL.";
+		else
+		{
+			ss << ch->getTarget() << " (depth:" << ch->get_depth()-character::staticDepthOffset-1 << " unloaded:" << ch->isUnloaded() << " destroyed:" << ch->isDestroyed() << ")";
+		}
+		ss << std::endl;
+	}
+}
+#endif
+
 void
 button_character_instance::set_current_state(e_mouse_state new_state)
 {
 	if (new_state == m_mouse_state)
 		return;
 		
-	// save current "display list"
-	std::vector<character*> old_list;
-	get_active_characters(old_list, m_mouse_state);
-	
-	// load new "display list" 
-	// NOTE: We don't change state yet, so that set_invalidated() can 
-	// load the current bounds first.
-	std::vector<character*> new_list;
-	get_active_characters(new_list, new_state);
-		
-	// see if the two lists differ and restart characters if needed
-	if (new_list.size() != old_list.size())
-		set_invalidated();		// something changed 
-  
-  size_t old_count = old_list.size();
-  size_t new_count = new_list.size();
-  for (size_t i=0; i<new_count; i++) {
+#ifdef GNASH_DEBUG_BUTTON_DISPLAYLIST
+	std::stringstream ss;
+	ss << "at set_current_state enter: " << std::endl;
+	dump(m_record_character, ss);
+	log_debug("%s", ss.str());
+#endif
+
+	// Get new state records
+	RecSet newChars;
+	get_active_records(newChars, new_state);
+
+	// For each possible record, check if it should still be there
+	for (size_t i=0, e=m_record_character.size(); i<e; ++i)
+	{
+		character* oldch = m_record_character[i];
+		bool shouldBeThere = ( newChars.find(i) != newChars.end() );
+
+		if ( ! shouldBeThere )
+		{
+			// is there, but is unloaded: destroy, clear slot and go on
+			if ( oldch && oldch->isUnloaded() )
+			{
+				if ( ! oldch->isDestroyed() ) oldch->destroy();
+				m_record_character[i] = NULL;
+				oldch = NULL;
+			}
+
+			if ( oldch ) // the one we have should not be there... unload!
+			{
+				set_invalidated();
+
+				if ( ! oldch->unload() )
+				{
+					// No onUnload handler: destroy and clear slot
+					if ( ! oldch->isDestroyed() ) oldch->destroy();
+					m_record_character[i] = NULL;
+				}
+				else
+				{
+					// onUnload handler: shift depth and keep slot
+					int oldDepth = oldch->get_depth();
+					int newDepth = character::removedDepthOffset - oldDepth;
+#ifdef GNASH_DEBUG_BUTTON_DISPLAYLIST
+					log_debug("Removed button record shifted from depth %d to depth %d", oldDepth, newDepth);
+#endif
+					oldch->set_depth(newDepth);
+				}
+			}
+		}
+		else // should be there
+		{
+			// Is there already, but is unloaded: destroy and consider as gone
+			if ( oldch && oldch->isUnloaded() )
+			{
+				if ( ! oldch->isDestroyed() ) oldch->destroy();
+				m_record_character[i] = NULL;
+				oldch = NULL;
+			}
+
+			if ( ! oldch )
+			{
+				// Not there, instantiate
+				button_record& bdef = m_def->m_button_records[i];
+
+				const matrix&	mat = bdef.m_button_matrix;
+				const cxform&	cx = bdef.m_button_cxform;
+				int ch_depth = bdef.m_button_layer+character::staticDepthOffset+1;
+				int ch_id = bdef.m_character_id;
+
+				character* ch = bdef.m_character_def->create_character_instance(this, ch_id);
+				ch->set_matrix(mat); 
+				ch->set_cxform(cx); 
+				ch->set_depth(ch_depth); 
+				assert(ch->get_parent() == this);
+				assert(ch->get_name().empty()); // no way to specify a name for button chars anyway...
+
+				if ( ch->wantsInstanceName() )
+				{
+					//std::string instance_name = getNextUnnamedInstanceName();
+					ch->set_name(getNextUnnamedInstanceName());
+				}
+
+				set_invalidated();
+
+				m_record_character[i] = ch;
+				ch->stagePlacementCallback(); // give this character a life
 
-  	bool found=false;
-  	for (size_t j=0; j<old_count; j++) { 
-	  	if (new_list[i] == old_list[j]) {
-				found=true;
-				break; 
 			}
 		}
-		if (!found) {
-			// character (re-)appeared on stage -> restart!
-			new_list[i]->restart();
-			set_invalidated();
-		} 
 	}
 
-	// effectively change state
+#ifdef GNASH_DEBUG_BUTTON_DISPLAYLIST
+	ss.str("");
+	ss << "at set_current_state end: " << std::endl;
+	dump(m_record_character, ss);
+	log_debug("%s", ss.str());
+#endif
+
+	// Remember current state
 	m_mouse_state=new_state;
 	 
 }
@@ -682,77 +764,59 @@ void
 button_character_instance::add_invalidated_bounds(InvalidatedRanges& ranges, 
 	bool force)
 {
-  if (!m_visible) return; // not visible anyway
+	if (!m_visible)
+	{
+		//log_debug("button %s not visible on add_invalidated_bounds", getTarget());
+		return; // not visible anyway
+	}
+	//log_debug("button %s add_invalidated_bounds called", getTarget());
 
 	ranges.add(m_old_invalidated_ranges);  
 
-  // TODO: Instead of using these for loops again and again, wouldn't it be a
-  // good idea to have a generic "get_record_character()" method?
-	for (size_t i = 0; i < m_def->m_button_records.size(); i++)
-	{
-		button_record&	rec = m_def->m_button_records[i];
-		assert(m_record_character.size() > i);
-		if (m_record_character[i] == NULL)
-		{
-			continue;
-		}
-		if ((m_mouse_state == UP && rec.m_up)
-		    || (m_mouse_state == DOWN && rec.m_down)
-		    || (m_mouse_state == OVER && rec.m_over))
-		{
-				/*bounds->expand_to_transformed_rect(get_world_matrix(), 
-          m_record_character[i]->get_bound());*/
-        m_record_character[i]->add_invalidated_bounds(ranges, 
-          force||m_invalidated);        
-		}
-	}
-
+	std::vector<character*> actChars;
+	get_active_characters(actChars);
+	std::for_each(actChars.begin(), actChars.end(),
+		boost::bind(&character::add_invalidated_bounds, _1,
+			    boost::ref(ranges), force||m_invalidated)
+	);
 }
 
 geometry::Range2d<float>
 button_character_instance::getBounds() const
 {
-	for (size_t i = 0; i < m_def->m_button_records.size(); i++)
+	typedef geometry::Range2d<float> Range;
+	Range allBounds(geometry::nullRange);
+
+	typedef std::vector<character*> CharVect;
+	CharVect actChars;
+	const_cast<button_character_instance*>(this)->get_active_characters(actChars);
+	for(CharVect::iterator i=actChars.begin(),e=actChars.end(); i!=e; ++i)
 	{
-		button_record&	rec = m_def->m_button_records[i];
-		assert(m_record_character.size() > i);
-		if (m_record_character[i] == NULL)
-		{
-			continue;
-		}
-		if ((m_mouse_state == UP && rec.m_up)
-		    || (m_mouse_state == DOWN && rec.m_down)
-		    || (m_mouse_state == OVER && rec.m_over))
-		{
-			// TODO: should we consider having multiple characters
-			//       for a single state ?
-			return m_record_character[i]->getBounds();
-		}
+		const character* ch = *i;
+
+		// Child bounds need be transformed in our coordinate space
+		Range lclBounds = ch->getBounds();
+		matrix m = ch->get_matrix();
+		m.transform(lclBounds);
+
+		allBounds.expandTo(lclBounds);
 	}
-	return geometry::Range2d<float>(geometry::nullRange);
+
+	return allBounds;
 }
 
 bool
 button_character_instance::pointInShape(float x, float y) const
 {
-	for (size_t i = 0; i < m_def->m_button_records.size(); i++)
+	typedef std::vector<character*> CharVect;
+	CharVect actChars;
+	const_cast<button_character_instance*>(this)->get_active_characters(actChars);
+	for(CharVect::iterator i=actChars.begin(),e=actChars.end(); i!=e; ++i)
 	{
-		button_record&	rec = m_def->m_button_records[i];
-		assert(m_record_character.size() > i);
-		if (m_record_character[i] == NULL)
-		{
-			continue;
-		}
-		if ((m_mouse_state == UP && rec.m_up)
-		    || (m_mouse_state == DOWN && rec.m_down)
-		    || (m_mouse_state == OVER && rec.m_over))
-		{
-			// TODO: should we consider having multiple characters
-			//       for a single state ?
-			return m_record_character[i]->pointInShape(x, y);
-		}
+		const character* ch = *i;
+		if ( ch->pointInShape(x,y) ) return true;
 	}
-	return false; // no shape
+	return false; 
 }
 
 as_object*
@@ -768,10 +832,17 @@ button_character_instance::get_path_element(string_table::key key)
 character *
 button_character_instance::getChildByName(const std::string& name) const
 {
-	// See if we have a match on the button records list
-	for (size_t i=0, n=m_record_character.size(); i<n; ++i)
+	// Get all currently active characters, including unloaded
+	CharsVect actChars;
+	// TODO: fix the const_cast
+	const_cast<button_character_instance*>(this)->get_active_characters(actChars, true);
+
+	// Lower depth first for duplicated names, so we sort
+	std::sort(actChars.begin(), actChars.end(), charDepthLessThen);
+
+	for (CharsVect::iterator i=actChars.begin(), e=actChars.end(); i!=e; ++i)
 	{
-		character* child = m_record_character[i].get();
+		character* child = *i;
 		const char* pat_c = child->get_name().c_str();
 		const char* nam_c = name.c_str();
 
@@ -797,33 +868,64 @@ button_character_instance::stagePlacementCallback()
 	// do we need this???
 	//_vm.getRoot().addLiveChar(this);
 
-	size_t r, r_num =  m_def->m_button_records.size();
-	m_record_character.resize(r_num);
-
-	for (r = 0; r < r_num; r++)
+	// Instantiate the hit characters
+	RecSet hitChars;
+	get_active_records(hitChars, HIT);
+	for (RecSet::iterator i=hitChars.begin(),e=hitChars.end(); i!=e; ++i)
 	{
-		button_record& bdef = m_def->m_button_records[r];
+		button_record& bdef = m_def->m_button_records[*i];
 
 		const matrix&	mat = bdef.m_button_matrix;
 		const cxform&	cx = bdef.m_button_cxform;
-		int ch_depth = bdef.m_button_layer;
+		int ch_depth = bdef.m_button_layer+character::staticDepthOffset+1;
 		int ch_id = bdef.m_character_id;
 
-		boost::intrusive_ptr<character> ch = bdef.m_character_def->create_character_instance(this, ch_id);
-		ch->set_matrix(mat);
-		ch->set_cxform(cx);
-		ch->set_depth(ch_depth);
+		character* ch = bdef.m_character_def->create_character_instance(this, ch_id);
+		ch->set_matrix(mat); 
+		ch->set_cxform(cx); // TODO: who cares about color ?
+		ch->set_depth(ch_depth); // TODO: check if we care about depth, and why ...
 		assert(ch->get_parent() == this);
+		assert(ch->get_name().empty()); // no way to specify a name for button chars anyway...
 
-		if (ch->get_name().empty() && ch->wantsInstanceName()) 
+		_hitCharacters.push_back(ch);
+	}
+
+	// Setup the state characters container
+	// It will have a slot for each character record.
+	// Some slots will probably be never used (consider HIT-only records)
+	// but for now this direct corrispondence between record number
+	// and active character will be handy.
+	m_record_character.resize(m_def->m_button_records.size());
+
+	// Instantiate the default state characters 
+	RecSet upChars;
+	get_active_records(upChars, UP);
+	log_debug("At StagePlacementCallback, button %s got %d active chars for state UP", getTarget(), upChars.size());
+	for (RecSet::iterator i=upChars.begin(),e=upChars.end(); i!=e; ++i)
+	{
+		int rno = *i;
+		button_record& bdef = m_def->m_button_records[rno];
+
+		const matrix&	mat = bdef.m_button_matrix;
+		const cxform&	cx = bdef.m_button_cxform;
+		int ch_depth = bdef.m_button_layer+character::staticDepthOffset+1;
+		int ch_id = bdef.m_character_id;
+
+		character* ch = bdef.m_character_def->create_character_instance(this, ch_id);
+		ch->set_matrix(mat); 
+		ch->set_cxform(cx); 
+		ch->set_depth(ch_depth); 
+		assert(ch->get_parent() == this);
+		assert(ch->get_name().empty()); // no way to specify a name for button chars anyway...
+
+		if ( ch->wantsInstanceName() )
 		{
-			std::string instance_name = getNextUnnamedInstanceName();
-			ch->set_name(instance_name.c_str());
+			//std::string instance_name = getNextUnnamedInstanceName();
+			ch->set_name(getNextUnnamedInstanceName());
 		}
 
-		m_record_character[r] = ch;
-
-		ch->stagePlacementCallback(); // give this character life (TODO: they aren't on stage, are them ?)
+		m_record_character[rno] = ch;
+		ch->stagePlacementCallback(); // give this character a life
 	}
 
 	// there's no INITIALIZE/CONSTRUCT/LOAD/ENTERFRAME/UNLOAD events for buttons
@@ -837,11 +939,21 @@ button_character_instance::markReachableResources() const
 
 	m_def->setReachable();
 
-	// Markstate characters as reachable
+	// Mark state characters as reachable
 	for (CharsVect::const_iterator i=m_record_character.begin(), e=m_record_character.end();
 			i!=e; ++i)
 	{
-		(*i)->setReachable();
+		character* ch = *i;
+		if ( ch ) ch->setReachable();
+	}
+
+	// Mark hit characters as reachable
+	for (CharsVect::const_iterator i=_hitCharacters.begin(), e=_hitCharacters.end();
+			i!=e; ++i)
+	{
+		character* ch = *i;
+		assert ( ch );
+		ch->setReachable();
 	}
 
 	// character class members
@@ -852,22 +964,59 @@ button_character_instance::markReachableResources() const
 bool
 button_character_instance::unload()
 {
+	//log_debug("Button %s being unloaded", getTarget());
+
 	bool childsHaveUnload = false;
 
 	// We need to unload all childs, or the global instance list will keep growing forever !
 	//std::for_each(m_record_character.begin(), m_record_character.end(), boost::bind(&character::unload, _1));
 	for (CharsVect::iterator i=m_record_character.begin(), e=m_record_character.end(); i!=e; ++i)
 	{
-		boost::intrusive_ptr<character> ch = *i;
+		character* ch = *i;
+		if ( ! ch ) continue;
+		if ( ch->isUnloaded() ) continue;
 		if ( ch->unload() ) childsHaveUnload = true;
 		//log_debug("Button child %s (%s) unloaded", ch->getTarget().c_str(), typeName(*ch).c_str());
 	}
 
+	// NOTE: we don't need to ::unload or ::destroy here
+	//       as the _hitCharacters are never placed on stage.
+	//       As an optimization we might not even instantiate
+	//       them, and only use the definition and the 
+	//       associated transform matrix... (would take
+	//       hit instance off the GC).
+	_hitCharacters.clear();
+
 	bool hasUnloadEvent = character::unload();
 
 	return hasUnloadEvent || childsHaveUnload;
 }
 
+void
+button_character_instance::destroy()
+{
+	//log_debug("Button %s being destroyed", getTarget());
+
+	for (CharsVect::iterator i=m_record_character.begin(), e=m_record_character.end(); i!=e; ++i)
+	{
+		character* ch = *i;
+		if ( ! ch ) continue;
+		if ( ch->isDestroyed() ) continue;
+		ch->destroy();
+		*i = NULL;
+	}
+
+	// NOTE: we don't need to ::unload or ::destroy here
+	//       as the _hitCharacters are never placed on stage.
+	//       As an optimization we might not even instantiate
+	//       them, and only use the definition and the 
+	//       associated transform matrix... (would take
+	//       hit instance off the GC).
+	_hitCharacters.clear();
+
+	character::destroy();
+}
+
 bool
 button_character_instance::get_member(string_table::key name_key, as_value* val,
   string_table::key nsname)
@@ -879,10 +1028,8 @@ button_character_instance::get_member(string_table::key name_key, as_value* val,
   //
   if (name_key == NSV::PROP_uROOT)
   {
-
-    // Let ::get_root() take care of _lockroot
-    movie_instance* relRoot = get_root();
-    val->set_as_object( relRoot );
+    // getAsRoot() will take care of _lockroot
+    val->set_as_object( const_cast<sprite_instance*>( getAsRoot() )  );
     return true;
   }
 
@@ -975,6 +1122,76 @@ button_character_instance::getSWFVersion() const
 	return m_def->getSWFVersion();
 }
 
+static as_object*
+getButtonInterface()
+{
+  static boost::intrusive_ptr<as_object> proto;
+  if ( proto == NULL )
+  {
+    proto = new as_object(getObjectInterface());
+    VM::get().addStatic(proto.get());
+
+    attachButtonInterface(*proto);
+  }
+  return proto.get();
+}
+
+static as_value
+button_ctor(const fn_call& /* fn */)
+{
+  boost::intrusive_ptr<as_object> clip = new as_object(getButtonInterface());
+  return as_value(clip.get());
+}
+
+void
+button_class_init(as_object& global)
+{
+  // This is going to be the global Button "class"/"function"
+  static boost::intrusive_ptr<builtin_function> cl=NULL;
+
+  if ( cl == NULL )
+  {
+    cl=new builtin_function(&button_ctor, getButtonInterface());
+    VM::get().addStatic(cl.get());
+  }
+
+  // Register _global.MovieClip
+  global.init_member("Button", cl.get());
+}
+
+#ifdef USE_SWFTREE
+character::InfoTree::iterator 
+button_character_instance::getMovieInfo(InfoTree& tr, InfoTree::iterator it)
+{
+	InfoTree::iterator selfIt = character::getMovieInfo(tr, it);
+	std::ostringstream os;
+
+	std::vector<character*> actChars;
+	get_active_characters(actChars, true);
+	std::sort(actChars.begin(), actChars.end(), charDepthLessThen);
+
+	os << actChars.size() << " active characters for state " << mouseStateName(m_mouse_state);
+	InfoTree::iterator localIter = tr.append_child(selfIt, StringPair(_("Button state"), os.str()));	    
+	std::for_each(actChars.begin(), actChars.end(), boost::bind(&character::getMovieInfo, _1, tr, localIter)); 
+
+	return selfIt;
+
+}
+#endif
+
+const char*
+button_character_instance::mouseStateName(e_mouse_state s)
+{
+	switch (s)
+	{
+		case UP: return "UP";
+		case DOWN: return "DOWN";
+		case OVER: return "OVER";
+		case HIT: return "HIT";
+		default: return "UNKNOWN (error?)";
+	}
+}
+
 } // end of namespace gnash
 
 
