@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,20 +15,18 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
-// 
-//
-//
 
 // SWF buttons.  Mouse-sensitive update/display, actions, etc.
 
-/* $Id: button_character_instance.h,v 1.23 2007/07/01 10:54:21 bjacques Exp $ */
 
 #ifndef GNASH_BUTTON_CHARACTER_INSTANCE_H
 #define GNASH_BUTTON_CHARACTER_INSTANCE_H
 
-
+#include "smart_ptr.h" // GNASH_USE_GC
 #include "character.h" // for inheritance
 
+#include <vector>
+#include <set>
 
 namespace gnash {
 
@@ -40,13 +38,6 @@ namespace gnash {
 // button characters
 //
 
-enum mouse_state
-{
-	MOUSE_UP,
-	MOUSE_DOWN,
-	MOUSE_OVER
-};
-
 //
 // button_character_instance
 //
@@ -56,9 +47,8 @@ class button_character_instance : public character
 public:
 	button_character_definition*	m_def;
 
-	typedef std::vector< boost::intrusive_ptr<character> > CharsVect;
-
-	CharsVect m_record_character;
+	typedef std::vector< character* > CharsVect;
+	typedef std::set<int> RecSet;
 
 	enum mouse_flags
 	{
@@ -72,12 +62,17 @@ public:
 		OUT_DOWN = FLAG_DOWN
 	};
 	int	m_last_mouse_flags, m_mouse_flags;
+
 	enum e_mouse_state
 	{
 		UP = 0,
 		DOWN,
-		OVER
+		OVER,
+		HIT
 	};
+
+	static const char* mouseStateName(e_mouse_state s);
+
 	e_mouse_state m_mouse_state;
 
 	button_character_instance(button_character_definition* def,
@@ -85,6 +80,10 @@ public:
 
 	~button_character_instance();
 
+	// See dox in as_object.h
+	bool get_member(string_table::key name, as_value* val, 
+		string_table::key nsname = 0);
+
 	bool can_handle_mouse_event() const { return true; }
 
 	// called from keypress listener only
@@ -92,17 +91,9 @@ public:
 
 	void	restart();
 
-	virtual void	advance(float delta_time);
-
 	void	display();
 	
 	void set_current_state(e_mouse_state new_state);
-	
-	/// Returns all characters that are currently visible based on the
-	/// current button state. The "_visible" property does not matter here. 
-	void get_active_characters(std::vector<character*>& list);
-	void get_active_characters(std::vector<character*>& list, e_mouse_state state);
-	
 
 	/// Combine the flags to avoid a conditional.
 	//  It would be faster with a macro.
@@ -120,18 +111,16 @@ public:
 	///
 	virtual character* get_topmost_mouse_entity(float x, float y);
 	
-	virtual bool wantsInstanceName()
+	virtual bool wantsInstanceName() const
 	{
 		return true; // buttons can be referenced 
 	}
 	
 	/// Overridden to look in button records for a match
-	virtual character* get_relative_target(const std::string& name);
+	virtual as_object* get_path_element(string_table::key key);
 
 	virtual void	on_button_event(const event_id& event);
 
-	void restart_characters(int condition);
-
 	//
 	// ActionScript overrides
 	//
@@ -148,8 +137,25 @@ public:
 	bool get_enabled();
 	void set_enabled(bool value);
 	
-	/// Properly initialize instance names for contained sprites
-	virtual void construct();
+	/// Receive a stage placement notification
+	//
+	/// This callback will:
+	///
+	/// (1) Register this button instance as a live character
+	/// (2) Setup the state characters calling stagePlacementCallback on all [WRONG]
+	///
+	virtual void stagePlacementCallback();
+
+	/// Properly unload contained characters
+	bool unload();
+
+	/// Properly destroy contained characters
+	void destroy();
+
+#ifdef USE_SWFTREE
+	// Override to append button characters info, see dox in character.h
+	virtual InfoTree::iterator getMovieInfo(InfoTree& tr, InfoTree::iterator it);
+#endif
 
 protected:
 
@@ -159,15 +165,67 @@ protected:
 	/// These are:
 	///	- this char's definition (m_def)
 	///	- the vector of state characters (m_record_character)
+	///	- the vector of hit characters (m_record_character)
 	///
 	void markReachableResources() const;
 #endif // GNASH_USE_GC
 
 private:
+
+	CharsVect m_record_character;
+
+	CharsVect _hitCharacters;
+
+	/// Returns all characters that are active based on the current state.
+	//
+	/// The "_visible" property does not matter here. 
+	///
+	/// @param list
+	///	The vector to push active characters into
+	///
+	/// @param includeUnloaded
+	///	If true, include unloaded but still reachable chars in the records slot.
+	///
+	void get_active_characters(std::vector<character*>& list, bool includeUnloaded=false);
+
+	/// Returns all characters (record nums) that should be active on the given state.
+	//
+	/// @param list
+	///	The set to push active characters record number into
+	///
+	/// @param state
+	///	The state we're interested in
+	///
+	void get_active_records(RecSet& list, e_mouse_state state);
+
+	const CharsVect& getHitCharacters() const
+	{
+		return _hitCharacters;
+	}
+
+
+	/// Return any state character whose name matches the given string
+	//
+	/// NOTE: both active and inactive childs are scanned for
+	///
+	/// @param name
+	///	Name to match, search is case sensitive for SWF7 and higher,
+	///     case insensitive up to SWF6.
+	///
+	character * getChildByName(const std::string& name) const;
+
+	/// \brief
+	/// Return version of the SWF containing
+	/// the button definition this is an instance of.
+        int getSWFVersion() const;
+
 	bool m_enabled;
 
 };
 
+/// Initialize the global Button class
+void button_class_init(as_object& global);
+
 }	// end namespace gnash
 
 
