@@ -1,6 +1,6 @@
 // font.cpp:  ActionScript font handling, for Gnash.
 // 
-//   Copyright (C) 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,10 +17,10 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-/* $Id: font.cpp,v 1.45 2007/07/24 19:43:30 strk Exp $ */
 
 // Based on the public domain work of Thatcher Ulrich <tu@tulrich.com> 2003
 
+#include "smart_ptr.h" // GNASH_USE_GC
 #include "font.h"
 #include "stream.h"
 #include "log.h"
@@ -37,21 +37,18 @@ namespace gnash {
 GlyphInfo::GlyphInfo()
 	:
 	glyph(),
-	textureGlyph(),
 	advance(0)
 {}
 
 GlyphInfo::GlyphInfo(boost::intrusive_ptr<shape_character_def> nGlyph, float nAdvance)
 	:
 	glyph(nGlyph.get()),
-	textureGlyph(),
 	advance(nAdvance)
 {}
 
 GlyphInfo::GlyphInfo(const GlyphInfo& o)
 	:
 	glyph(o.glyph.get()),
-	textureGlyph(o.textureGlyph),
 	advance(o.advance)
 {}
 
@@ -59,15 +56,15 @@ GlyphInfo::GlyphInfo(const GlyphInfo& o)
 void
 GlyphInfo::markReachableResources() const
 {
-	textureGlyph.markReachableResources();
 	if ( glyph ) glyph->setReachable();
 }
 #endif
 
 	font::font()
 		:
-		m_texture_glyph_nominal_size(96),	// Default is not important; gets overridden during glyph generation
 		m_name(),
+                m_display_name(),
+                m_copyright_name(),
 		m_owning_movie(NULL),
 		m_unicode_chars(false),
 		m_shift_jis_chars(false),
@@ -75,23 +72,26 @@ GlyphInfo::markReachableResources() const
 		m_is_italic(false),
 		m_is_bold(false),
 		m_wide_codes(false),
+		m_subpixel_font(false),
 		m_ascent(0.0f),
 		m_descent(0.0f),
 		m_leading(0.0f)
 	{
 	}
 
-	font::font(const std::string& name)
+	font::font(const std::string& name, bool bold, bool italic)
 		:
-		m_texture_glyph_nominal_size(96),	// Default is not important; gets overridden during glyph generation
 		m_name(name),
+                m_display_name(),
+                m_copyright_name(),
 		m_owning_movie(NULL),
 		m_unicode_chars(false),
 		m_shift_jis_chars(false),
 		m_ansi_chars(true),
-		m_is_italic(false),
-		m_is_bold(false),
+		m_is_italic(italic),
+		m_is_bold(bold),
 		m_wide_codes(false),
+		m_subpixel_font(false),
 		m_ascent(0.0f),
 		m_descent(0.0f),
 		m_leading(0.0f)
@@ -102,6 +102,10 @@ GlyphInfo::markReachableResources() const
 		{
 			log_error(_("Could not initialize device font face '%s'"), m_name.c_str());
 		}
+		else
+		{
+			log_debug("Initialized device font face '%s'%s%s", m_name, bold ? " bold" : "", italic ? " italic" : "");
+		}
 	}
 
 	font::~font()
@@ -125,50 +129,6 @@ GlyphInfo::markReachableResources() const
 	}
 
 
-	const texture_glyph&	font::get_texture_glyph(int glyph_index, bool embedded) const
-	{
-		const GlyphInfoVect& lookup = embedded ? _embedGlyphTable : _deviceGlyphTable;
-
-		if (glyph_index < 0 || (size_t)glyph_index >= lookup.size())
-		{
-			// TODO: should we log an error here ?
-			static const texture_glyph	s_dummy_texture_glyph;
-			return s_dummy_texture_glyph;
-		}
-
-		return lookup[glyph_index].textureGlyph;
-	}
-
-
-	void	font::add_texture_glyph(int glyph_index, const texture_glyph& glyph, bool embedded)
-	{
-		GlyphInfoVect& lookup = embedded ? _embedGlyphTable : _deviceGlyphTable;
-
-		assert(glyph_index >= 0 && (size_t)glyph_index < lookup.size());
-		assert(glyph.is_renderable());
-
-		assert(lookup[glyph_index].textureGlyph.is_renderable() == false);
-
-		lookup[glyph_index].textureGlyph = glyph;
-	}
-
-
-	void	font::wipe_texture_glyphs()
-	{
-
-		// Replace with default (empty) glyph info.
-		texture_glyph	default_tg;
-		for (size_t i = 0, n = _embedGlyphTable.size(); i < n; i++)
-		{
-			_embedGlyphTable[i].textureGlyph = default_tg;
-		}
-		for (size_t i = 0, n = _deviceGlyphTable.size(); i < n; i++)
-		{
-			_deviceGlyphTable[i].textureGlyph = default_tg;
-		}
-	}
-
-
 	void	font::read(stream* in, SWF::tag_type tag, movie_definition* m)
 	{
 		// No add_ref() here, to avoid cycle. 
@@ -183,8 +143,13 @@ GlyphInfo::markReachableResources() const
 		{
 			assert (tag == SWF::DEFINEFONT2 || tag == SWF::DEFINEFONT3);
 			readDefineFont2_or_3(in, m);
+			if (tag == SWF::DEFINEFONT3)
+			{
+				m_subpixel_font = true;
+			}
 		}
 
+		// TODO: initialize the deviceFontProvider only when needed ?
 		if ( ! m_name.empty() && ! initDeviceFontProvider() )
 		{
 			log_error("Could not initialize device font face '%s'", m_name.c_str());
@@ -204,6 +169,7 @@ GlyphInfo::markReachableResources() const
 		// are measured from the start of the
 		// offset table.
 		std::vector<unsigned>	offsets;
+		in->ensureBytes(2);
 		offsets.push_back(in->read_u16());
 
 		IF_VERBOSE_PARSE (
@@ -211,36 +177,41 @@ GlyphInfo::markReachableResources() const
 		);
 
 		int	count = offsets[0] >> 1;
-		for (int i = 1; i < count; i++)
+		if ( count > 0 )
 		{
-			offsets.push_back(in->read_u16());
+			in->ensureBytes(count*2);
+			for (int i = 1; i < count; i++)
+			{
+				offsets.push_back(in->read_u16());
 
-			IF_VERBOSE_PARSE (
-			log_parse("offset[%d] = %d", i, offsets[i]);
-			);
+				IF_VERBOSE_PARSE (
+				log_parse("offset[%d] = %d", i, offsets[i]);
+				);
+			}
+		}
+		else
+		{
+			log_error("Negative embedded glyph table size: %d", count);
 		}
 
 		_embedGlyphTable.resize(count);
 
-		if (m->get_create_font_shapes() == DO_LOAD_FONT_SHAPES)
+		// Read the glyph shapes.
+		for (int i = 0; i < count; i++)
 		{
-			// Read the glyph shapes.
-			{for (int i = 0; i < count; i++)
+			// Seek to the start of the shape data.
+			unsigned long new_pos = table_base + offsets[i];
+
+			if ( ! in->set_position(new_pos) )
 			{
-				// Seek to the start of the shape data.
-				unsigned long new_pos = table_base + offsets[i];
+				throw ParserException(_("Glyphs offset table corrupted in DefineFont tag"));
+			}
 
-				if ( ! in->set_position(new_pos) )
-				{
-        				throw ParserException(_("Glyphs offset table corrupted in DefineFont tag"));
-				}
+			// Create & read the shape.
+			shape_character_def* s = new shape_character_def;
+			s->read(in, SWF::DEFINEFONT, false, m); 
 
-				// Create & read the shape.
-				shape_character_def* s = new shape_character_def;
-				s->read(in, SWF::DEFINEFONT, false, m); 
-
-				_embedGlyphTable[i].glyph = s;
-			}}
+			_embedGlyphTable[i].glyph = s;
 		}
 	}
 
@@ -251,15 +222,17 @@ GlyphInfo::markReachableResources() const
 		log_parse(_("reading DefineFont2 or DefineFont3"));
 		);
 
-		bool	has_layout = (in->read_uint(1) != 0);
-		m_shift_jis_chars = (in->read_uint(1) != 0);
-		m_unicode_chars = (in->read_uint(1) != 0);
-		m_ansi_chars = (in->read_uint(1) != 0);
-		bool	wide_offsets = (in->read_uint(1) != 0);
-		m_wide_codes = (in->read_uint(1) != 0);
-		m_is_italic = (in->read_uint(1) != 0);
-		m_is_bold = (in->read_uint(1) != 0);
-		uint8_t	reserved = in->read_u8();
+		// TODO: should this be aligned ?
+		in->ensureBytes(2); // 1 for the flags, 1 reserved
+		bool	has_layout = in->read_bit();
+		m_shift_jis_chars = in->read_bit();
+		m_unicode_chars = in->read_bit();
+		m_ansi_chars = in->read_bit();
+		bool	wide_offsets = in->read_bit();
+		m_wide_codes = in->read_bit();
+		m_is_italic = in->read_bit();
+		m_is_bold = in->read_bit();
+		boost::uint8_t	reserved = in->read_u8();
 
 		IF_VERBOSE_PARSE (
 		log_parse(" has_layout = %d", has_layout);
@@ -275,28 +248,25 @@ GlyphInfo::markReachableResources() const
 		// Inhibit compiler warning.
 		reserved = reserved;
 
-		char* name = in->read_string_with_length();
-		if ( name )
-		{
-			m_name = name;
-			delete [] name;
-		}
+		in->read_string_with_length(m_name);
 
-		uint16_t glyph_count = in->read_u16();
+		in->ensureBytes(2); 
+		boost::uint16_t glyph_count = in->read_u16();
 		
 		unsigned long table_base = in->get_position();
 
 		// Read the glyph offsets.  Offsets
 		// are measured from the start of the
 		// offset table. Make sure wide offsets fit into elements
-		std::vector<uint32_t>	offsets;
+		std::vector<boost::uint32_t>	offsets;
 		int	font_code_offset;
 		if (wide_offsets)
 		{
 			// 32-bit offsets.
+			in->ensureBytes(4*glyph_count + 4); 
 			for (unsigned int i = 0; i < glyph_count; i++)
 			{
-				uint32_t off = in->read_u32();	
+				boost::uint32_t off = in->read_u32();	
 
 				IF_VERBOSE_PARSE (
 				log_parse(_("Glyph %d at offset %u"), i, off);
@@ -309,9 +279,10 @@ GlyphInfo::markReachableResources() const
 		else
 		{
 			// 16-bit offsets.
+			in->ensureBytes(2*glyph_count + 2); 
 			for (unsigned int i = 0; i < glyph_count; i++)
 			{
-				uint16_t off = in->read_u16();	
+				boost::uint16_t off = in->read_u16();	
 
 				IF_VERBOSE_PARSE (
 				log_parse(_("Glyph %d at offset %u"), i, off);
@@ -324,51 +295,36 @@ GlyphInfo::markReachableResources() const
 
 		_embedGlyphTable.resize(glyph_count);
 
-		if (m->get_create_font_shapes() == DO_LOAD_FONT_SHAPES)
+		// Read the glyph shapes.
+		for (int i = 0; i < glyph_count; i++)
 		{
-			// Read the glyph shapes.
-			{for (int i = 0; i < glyph_count; i++)
+			// Seek to the start of the shape data.
+			unsigned long new_pos = table_base + offsets[i];
+
+			// It seems completely possible to
+			// have such seeks-back, see bug #16311
+			//assert(new_pos >= in->get_position());
+
+			if ( ! in->set_position(new_pos) )
 			{
-				// Seek to the start of the shape data.
-				unsigned long new_pos = table_base + offsets[i];
-
-				// It seems completely possible to
-				// have such seeks-back, see bug #16311
-				//assert(new_pos >= in->get_position());
-
-				if ( ! in->set_position(new_pos) )
-				{
-        				throw ParserException(_("Glyphs offset table corrupted in DefineFont2/3 tag"));
-				}
-
-				// Create & read the shape.
-				shape_character_def* s = new shape_character_def;
-				s->read(in, SWF::DEFINEFONT2, false, m); // .. or DEFINEFONT3 actually..
-
-				_embedGlyphTable[i].glyph = s;
-			}}
-
-			unsigned long current_position = in->get_position();
-			if (font_code_offset + table_base != current_position)
-			{
-				// Bad offset!  Don't try to read any more.
-				IF_VERBOSE_MALFORMED_SWF(
-				log_swferror(_("Bad offset in DefineFont2"));
-				);
-				return;
+				throw ParserException(_("Glyphs offset table corrupted in DefineFont2/3 tag"));
 			}
+
+			// Create & read the shape.
+			shape_character_def* s = new shape_character_def;
+			s->read(in, SWF::DEFINEFONT2, false, m); // .. or DEFINEFONT3 actually..
+
+			_embedGlyphTable[i].glyph = s;
 		}
-		else
-		{
-			// Skip the shape data.
-			unsigned long new_pos = table_base + font_code_offset;
-			if (new_pos >= in->get_tag_end_position())
-			{
-				// No layout data!
-				return;
-			}
 
-			in->set_position(new_pos);
+		unsigned long current_position = in->get_position();
+		if (font_code_offset + table_base != current_position)
+		{
+			// Bad offset!  Don't try to read any more.
+			IF_VERBOSE_MALFORMED_SWF(
+			log_swferror(_("Bad offset in DefineFont2"));
+			);
+			return;
 		}
 
 		read_code_table(in);
@@ -376,30 +332,41 @@ GlyphInfo::markReachableResources() const
 		// Read layout info for the glyphs.
 		if (has_layout)
 		{
+			in->ensureBytes(6);
 			m_ascent = (float) in->read_s16();
 			m_descent = (float) in->read_s16();
 			m_leading = (float) in->read_s16();
 			
 			// Advance table; i.e. how wide each character is.
-			for (int i = 0, n = _embedGlyphTable.size(); i < n; i++)
+			size_t nGlyphs = _embedGlyphTable.size();
+			in->ensureBytes(nGlyphs*2);
+			for (size_t i = 0; i < nGlyphs; i++)
 			{
 				_embedGlyphTable[i].advance = (float) in->read_s16();
 			}
 
 			// Bounds table.
-			//m_bounds_table.resize(m_glyphs.size());	// kill
-			rect	dummy_rect;
-			{for (size_t i = 0, n = _embedGlyphTable.size(); i < n; i++)
 			{
-				//m_bounds_table[i].read(in);	// kill
-				dummy_rect.read(in);
-			}}
+				rect	dummy_rect;
+				// TODO: shouldn't we log_unimpl here ??
+				for (size_t i = 0; i < nGlyphs; i++) dummy_rect.read(in);
+			}
 
 			// Kerning pairs.
+			in->ensureBytes(2);
 			int	kerning_count = in->read_u16();
+			if ( m_wide_codes )
+			{
+				in->ensureBytes(6*kerning_count); // includes the adjustment 
+			}
+			else
+			{
+				in->ensureBytes(4*kerning_count); // includes the adjustment 
+			}
+
 			for (int i = 0; i < kerning_count; i++)
 			{
-				uint16_t	char0, char1;
+				boost::uint16_t	char0, char1;
 				if (m_wide_codes)
 				{
 					char0 = in->read_u16();
@@ -429,6 +396,14 @@ GlyphInfo::markReachableResources() const
 		}
 	}
 
+        // Read the font name, display and legal, from a DefineFontName tag.
+        void font::read_font_name(stream* in, SWF::tag_type tag,
+            movie_definition* /*m*/) 
+        {
+            assert(tag == SWF::DEFINEFONTNAME);
+            in->read_string(m_display_name);
+            in->read_string(m_copyright_name);
+        }
 
 	// Read additional information about this font, from a
 	// DefineFontInfo tag.  The caller has already read the tag
@@ -449,17 +424,14 @@ GlyphInfo::markReachableResources() const
 			}
 		}
 
-		char* name = in->read_string_with_length();
-		if ( name )
+		in->read_string_with_length(m_name);
+		// TODO: initialize the deviceFontProvider only when needed ?
+		if ( ! m_name.empty() && ! initDeviceFontProvider() )
 		{
-			m_name = name;
-			delete [] name;
-		}
-		else
-		{
-			m_name.clear();
+			log_error(_("Could not initialize device font face '%s'"), m_name.c_str());
 		}
 
+		in->ensureBytes(1);
 		unsigned char	flags = in->read_u8();
 
 		// The following 3 flags are reserved
@@ -485,27 +457,30 @@ GlyphInfo::markReachableResources() const
 
 		assert(_embedded_code_table.empty());
 
+		size_t nGlyphs = _embedGlyphTable.size();
 		if (m_wide_codes)
 		{
-			// Code table is made of uint16_t's.
-			for (size_t i=0, n=_embedGlyphTable.size(); i<n; ++i)
+			in->ensureBytes(2*nGlyphs);
+			// Code table is made of boost::uint16_t's.
+			for (size_t i=0; i<nGlyphs; ++i)
 			{
-				uint16_t code = in->read_u16();
+				boost::uint16_t code = in->read_u16();
 				_embedded_code_table.insert(std::make_pair(code, i));
 			}
 		}
 		else
 		{
 			// Code table is made of bytes.
-			for (int i=0, n=_embedGlyphTable.size(); i<n; ++i)
+			in->ensureBytes(1*nGlyphs);
+			for (size_t i=0; i<nGlyphs; ++i)
 			{
-				uint8_t code = in->read_u8();
+				boost::uint8_t code = in->read_u8();
 				_embedded_code_table.insert(std::make_pair(code, i));
 			}
 		}
 	}
 
-	int	font::get_glyph_index(uint16_t code, bool embedded) const
+	int	font::get_glyph_index(boost::uint16_t code, bool embedded) const
 	{
 		const code_table& ctable = embedded ? _embedded_code_table : _device_code_table;
 
@@ -515,7 +490,7 @@ GlyphInfo::markReachableResources() const
 		{
 			glyph_index = it->second;
 #if 0
-			log_msg(_("get_glyph_index(%u) returning %d"),
+			log_debug(_("get_glyph_index(%u) returning %d"),
 				code, glyph_index);
 #endif
 			return glyph_index;
@@ -527,7 +502,7 @@ GlyphInfo::markReachableResources() const
 			glyph_index = const_cast<font*>(this)->add_os_glyph(code);
 		}
 #if 0
-		log_msg(_("get_glyph_index(%u) returning -1"), code);
+		log_debug(_("get_glyph_index(%u) returning -1"), code);
 #endif
 		return glyph_index;
 	}
@@ -550,7 +525,7 @@ GlyphInfo::markReachableResources() const
 		else
 		{
 			// Bad glyph index.  Due to bad data file?
-			assert(0);
+			abort();
 			return 0;
 		}
 	}
@@ -572,53 +547,28 @@ GlyphInfo::markReachableResources() const
 		return 0;
 	}
 
-
-        void	font::output_cached_data(tu_file* /* out */, const cache_options& /* options */)
-	// Dump our cached data into the given stream.
+	unsigned short int font::unitsPerEM(bool embed) const
 	{
-// @@ Disabled.  Need to fix input_cached_data, so that it has a
-// reliable and cheap way to skip over data for NULL glyphs.
-#if 0
-		// Dump cached shape data for glyphs (i.e. this will
-		// be tesselations used to render larger glyph sizes).
-		int	 n = m_glyphs.size();
-		out->write_le32(n);
-		for (int i = 0; i < n; i++)
+		// the EM square is 1024 x 1024 for DefineFont up to 2
+		// and 20 as much for DefineFont3 up
+		if ( embed )
 		{
-			shape_character_def*	s = m_glyphs[i].get();
-			if (s)
+			if ( m_subpixel_font ) return 1024*20;
+			else return 1024;
+		}
+		else
+		{
+			if ( ! _ftProvider.get() )
 			{
-				s->output_cached_data(out, options);
+				log_error("Device font provider was not initialized, can't get unitsPerEM");
+				return 0; // can't query it..
 			}
+			return _ftProvider->unitsPerEM();
 		}
-#endif // 0
-	}
-
-	
-	void	font::input_cached_data(tu_file* /* in */)
-	// Read our cached data from the given stream.
-	{
-// @@ Disable.  See comment in output_cached_data().
-#if 0
-		// Read cached shape data for glyphs.
-		int	n = in->read_le32();
-		if (n != m_glyphs.size())
-		{
-			log_error(_("error reading cache file in font::input_cached_data(): "
-				  "glyph count mismatch"));
-			in->go_to_end();	// ensure that no more data will be read from this stream.
-			return;
-		}
-
-		for (int i = 0; i < n; i++)
-		{
-			m_glyphs[i]->input_cached_data(in);
-		}
-#endif // 0
 	}
 
 	int
-	font::add_os_glyph(uint16_t code)
+	font::add_os_glyph(boost::uint16_t code)
 	{
 		assert ( _ftProvider.get() );
 		assert(_device_code_table.find(code) == _device_code_table.end());
@@ -672,8 +622,8 @@ GlyphInfo::markReachableResources() const
 /// Mark reachable resources (for the GC)
 //
 /// Reachable resources are:
-///	- texture_glyphs
-///	- shape_character_defs (vector glyphs)
+///	- shape_character_defs (vector glyphs, devide and embeded)
+///
 void
 font::markReachableResources() const
 {
