@@ -1,9 +1,9 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -15,19 +15,11 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-//
-
 #ifndef GNASH_GENERIC_CHARACTER_H
 #define GNASH_GENERIC_CHARACTER_H
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
+#include "smart_ptr.h" // GNASH_USE_GC
 #include "character.h" // for inheritance
-
 #include "shape_character_def.h" // for add_invalidated_bounds 
 
 #include <cassert>
@@ -48,6 +40,21 @@ protected:
 
 	boost::intrusive_ptr<character_def> m_def;
 
+#ifdef GNASH_USE_GC
+	/// Mark reachabe resources (for the GC)
+	//
+	/// These are:
+	///	- this char's definition (m_def)
+	///
+	void markReachableResources() const
+	{
+		assert(isReachable());
+		m_def->setReachable();
+
+		markCharacterReachable();
+	}
+#endif // GNASH_USE_GC
+
 public:
 
 	generic_character(character_def* def, character* parent, int id)
@@ -58,25 +65,33 @@ public:
 	    assert(m_def);
 	}
 
+	/// generic characters can not handle mouse events, so
+	/// the default implementation returns false.
+	/// override in your subclass to change this
 	virtual bool can_handle_mouse_event() const {
-		assert(m_parent != NULL);
-		return m_parent->can_handle_mouse_event();
+		return false;
+		//assert(m_parent != NULL);
+		//return m_parent->can_handle_mouse_event();
 	}
 
-	virtual void	display()
-	{
-//			GNASH_REPORT_FUNCTION;
-	    m_def->display(this);	// pass in transform info
-	    clear_invalidated();
-	    do_display_callback();
-	}
+	virtual void	display();
 
 	geometry::Range2d<float> getBounds() const
 	{
 		return m_def->get_bound().getRange();
 	}
 
-	virtual character* get_topmost_mouse_entity(float x, float y);
+	/// Generic character is NEVER a mouse entity by default, so
+	/// the default implementation of this method always returns NULL.
+	/// Override it from subclasses that do can be mouse entities.
+	///
+	/// If you need to check for a generic character to contain a 
+	/// given point, use the pointInShape() function instead.
+	/// 
+	virtual character* get_topmost_mouse_entity(float /*x*/, float /*y*/)
+	{
+		return NULL;
+	}
 
 	// See dox in character.h
 	virtual bool pointInShape(float x, float y) const;
