@@ -15,20 +15,13 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-/* $Id: impl.h,v 1.52 2008/01/21 20:55:51 rsavoye Exp $ */
 
 #ifndef GNASH_IMPL_H
 #define GNASH_IMPL_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
-#include "tu_config.h"
+#include "dsodefs.h"
 
 #include "gnash.h"
-//#include "cxform.h" // for composition of display_info
-//#include "matrix.h" // for display_info composition
 #include "types.h"
 #include "utility.h"
 #include "smart_ptr.h"
