@@ -7,11 +7,10 @@
 #ifndef GNASH_MOUSE_BUTTON_STATE_H
 #define GNASH_MOUSE_BUTTON_STATE_H
 
-
-//#include "impl.h" // should get rid of this
+#include "smart_ptr.h" // GNASH_USE_GC
 #include "character_def.h"
-#include "sound.h"
 #include "smart_ptr.h" // for composition and inlines
+#include "character.h" // for use in intrusive_ptr
 
 // Forward declarations
 namespace gnash {
@@ -26,11 +25,21 @@ class mouse_button_state
 
 public:
 
+	/// Possible button states
+	enum state {
+
+		/// Button is depressed
+		UP=0,
+
+		/// Button is pressed
+		DOWN=1
+	};
+
 	/// entity that currently owns the mouse pointer
-	smart_ptr<movie>	m_active_entity;
+	boost::intrusive_ptr<character>	m_active_entity;
 
 	/// what's underneath the mouse right now
-	smart_ptr<movie>	m_topmost_entity;
+	boost::intrusive_ptr<character>	m_topmost_entity;
 
 	/// previous state of mouse button
 	bool	m_mouse_button_state_last;	
@@ -43,12 +52,20 @@ public:
 
 	mouse_button_state()
 		:
-		m_mouse_button_state_last(0),
-		m_mouse_button_state_current(0),
+		m_mouse_button_state_last(UP),
+		m_mouse_button_state_current(UP),
 		m_mouse_inside_entity_last(false)
 	{
 	}
 
+#ifdef GNASH_USE_GC
+	/// Mark reachable objects (active and topmost entities)
+	void markReachableResources() const
+	{
+		if ( m_active_entity.get() ) m_active_entity->setReachable();
+		if ( m_topmost_entity.get() ) m_topmost_entity->setReachable();
+	}
+#endif // GNASH_USE_GC
 };
 
 }	// end namespace gnash
