@@ -1,6 +1,6 @@
 // action_buffer.cpp:  holds actions for later execution, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,10 +17,9 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-/* $Id: action_buffer.cpp,v 1.22 2007/08/07 20:53:10 strk Exp $ */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include "action_buffer.h"
@@ -29,6 +28,7 @@
 #include "swf.h"
 #include "ASHandlers.h"
 #include "as_environment.h"
+#include "movie_definition.h"
 
 #include <typeinfo> 
 
@@ -47,50 +47,68 @@ namespace gnash {
 static float convert_float_little(const void *p);
 static double convert_double_wacky(const void *p);
 
-action_buffer::action_buffer()
+action_buffer::action_buffer(const movie_definition& md)
     :
-    m_decl_dict_processed_at(-1)
+    m_decl_dict_processed_at(-1),
+    _src(md)
 {
 //	static int count=0;
 //	printf("Action buffer %d created\n", ++count);
 }
 
-
 void
-action_buffer::read(stream* in)
+action_buffer::read(stream& in, unsigned long endPos)
 {
-    // Read action bytes.
-    for (;;) {
-#if 0
-	size_t instruction_start = m_buffer.size();
-	size_t pc = m_buffer.size();
-#endif
+	unsigned long startPos = in.get_position();
+	assert(endPos <= in.get_tag_end_position());
+	unsigned size = endPos-startPos;
 
-	in->ensureBytes(1); // action_id
-	uint8_t action_id = in->read_u8();
-	m_buffer.push_back(action_id);
-	
-	if (action_id & 0x80) {
-	    // Action contains extra data.  Read it.
-	    in->ensureBytes(2); // action length
-	    uint16_t length = in->read_u16();
-	    m_buffer.push_back(length & 0x0FF);
-	    m_buffer.push_back((length >> 8) & 0x0FF);
-	    in->ensureBytes(length); // action actions 
-	    for (uint16_t i = 0; i < length; i++) {
-		uint8_t b = in->read_u8(); // bytes ensured outside loop
-		m_buffer.push_back(b);
-	    }
-	}
-	
-	if (action_id == SWF::ACTION_END)
+	if ( ! size )
 	{
-	    // end of action buffer.
-	    break;
+		IF_VERBOSE_MALFORMED_SWF(
+		log_swferror(_("Empty action buffer starting at offset %lu"), startPos);
+		);
+		return;
 	}
-    }
-}
 
+	// Allocate the buffer
+	// 
+	// NOTE: a .reserve would be fine here, except GLIBCPP_DEBUG will complain...
+	//
+	m_buffer.resize(size);
+	unsigned char* buf = &m_buffer.front();
+
+	// Read all the bytes in the buffer
+	//
+	// NOTE:
+	// we might be reading more data then we'll actually
+	// use here if the SWF contains Action blocks padded
+	// with data after the terminating END.
+	// This has a cost in memory use, but for the normal
+	// case (non-malformed SWF) not looking for an END
+	// tag should give significant speedup in parsing
+	// large action-based movies.
+	//
+	in.read(reinterpret_cast<char*>(buf), size);
+
+	// Consistency checks here
+	//
+	// NOTE: it is common to find such movies, swfmill is known to write
+	//       DoAction w/out the terminating END tag
+	//
+    if ( m_buffer.back() != SWF::ACTION_END )
+    {
+        // Add a null terminator so read_string won't read off
+        // the end of the buffer.
+        m_buffer.push_back(0x00);
+
+	    IF_VERBOSE_MALFORMED_SWF(
+		    log_swferror(_("Action buffer starting at offset %lu doesn't end with an END tag"),
+			    startPos);
+        );
+    }
+    
+}
 
 /*public*/
 void
@@ -99,7 +117,7 @@ action_buffer::process_decl_dict(size_t start_pc, size_t stop_pc) const
     assert(stop_pc <= m_buffer.size());
     
     if (static_cast<size_t>(m_decl_dict_processed_at) == start_pc) {
-	// We've already processed this decl_dict.
+	// We've already processed this decl_dict. 
 #ifndef NDEBUG
 	int count = read_int16(start_pc+3);
 	assert((int) m_dictionary.size() == count);
@@ -107,22 +125,24 @@ action_buffer::process_decl_dict(size_t start_pc, size_t stop_pc) const
 	return;
     }
     
+#if 0 // debugging
     if (m_decl_dict_processed_at != -1)	{
-	log_msg(_("process_decl_dict(" SIZET_FMT ", " SIZET_FMT "): decl_dict was already processed at %d. "
-		"Skipping (or maybe we should append, or replace?)."),
+	log_debug(_("process_decl_dict(" SIZET_FMT ", " SIZET_FMT "): decl_dict was already processed at %d. "
+		"Overriding."),
 		  start_pc, stop_pc, m_decl_dict_processed_at);
-	return;
+	//return;
     }
+#endif
     
     m_decl_dict_processed_at = start_pc;
     
     // Actual processing.
     size_t i = start_pc;
-    uint16_t length = uint16_t(read_int16(i+1));
-    uint16_t count = uint16_t(read_int16(i+3)); 
+    boost::uint16_t length = boost::uint16_t(read_int16(i+1));
+    boost::uint16_t count = boost::uint16_t(read_int16(i+3)); 
     i += 2;
     
-//log_msg(_("Start at %d, stop at %d, length read was %d, count read was %d"), start_pc, stop_pc, length, count);
+//log_debug(_("Start at %d, stop at %d, length read was %d, count read was %d"), start_pc, stop_pc, length, count);
 
     assert(start_pc + 3 + length == stop_pc);
     
@@ -211,117 +231,159 @@ disasm_instruction(const unsigned char* instruction_data)
     if (action_id > ash.lastType()) {
 	ss << "<unknown>[0x]" <<  action_id << endl;
     } else {
-	ss << ash[action_id].getName() << endl;
-	fmt = ash[action_id].getArgFormat();
+	ss << ash[action_id].getName();
     }
 
     // Show instruction argument(s).
-    if (action_id & 0x80) {
+    if (action_id & 0x80)
+    {
+	ss << " (";
+	fmt = ash[action_id].getArgFormat();
 	assert(fmt != ARG_NONE);
 	int length = instruction_data[1] | (instruction_data[2] << 8);
-	if (fmt == ARG_HEX) {
+	if (fmt == ARG_HEX)
+	{
 	    for (int i = 0; i < length; i++) {
-		hexify(num, (const unsigned char *)&instruction_data[3 + i], 1, false);
-		ss << "0x" << num << " ";
+
+		ss << "0x" << hexify((const boost::uint8_t *)&instruction_data[3 + i], 1, false) << " ";
 	    }
-	    ss << endl;
-	} else if (fmt == ARG_STR) {
+	}
+	else if (fmt == ARG_STR)
+	{
 	    string str;
 	    for (int i = 0; i < length; i++) {
 		str += instruction_data[3 + i];
 	    }
-	    ss << "\"" << str.c_str() << "\"" << endl;
-	} else if (fmt == ARG_U8) {
+	    ss << "\"" << str.c_str() << "\"";
+	}
+	else if (fmt == ARG_U8)
+	{
 	    int	val = instruction_data[3];
-	    ss << " " << val << endl;
-	} else if (fmt == ARG_U16) {
+	    ss << " " << val;
+	}
+	else if (fmt == ARG_U16)
+	{
 	    int	val = instruction_data[3] | (instruction_data[4] << 8);
-	    ss << " " << val << endl;
-	} else if (fmt == ARG_S16) {
+	    ss << " " << val;
+	}
+	else if (fmt == ARG_S16)
+	{
 	    int	val = instruction_data[3] | (instruction_data[4] << 8);
 	    if (val & 0x8000) val |= ~0x7FFF;	// sign-extend
-	    ss << " " << val << endl;
-	} else if (fmt == ARG_PUSH_DATA) {
-	    ss << endl;
+	    ss << " " << val;
+	}
+	else if (fmt == ARG_PUSH_DATA)
+	{
 	    int i = 0;
-	    while (i < length) {
-		int	type = instruction_data[3 + i];
-		i++;
-		if (type == 0) {
+	    while (i < length)
+	    {
+		int type = instruction_data[3 + i];
+		if ( i++ ) ss << ", ";
+
+		if (type == 0)
+		{
 		    // string
 		    string str;
-		    while (instruction_data[3 + i]) {
+		    while (instruction_data[3 + i])
+		    {
 			str += instruction_data[3 + i];
 			i++;
 		    }
 		    i++;
-		    ss << "\t\"" << str.c_str() << "\"" << endl;
-		} else if (type == 1) {
+		    ss << "\"" << str.c_str() << "\"";
+		}
+		else if (type == 1)
+		{
 		    // float (little-endian)
 		    float f = convert_float_little(instruction_data + 3 + i);
 		    i += 4;
-		    ss << "(float) " << f << endl;
-		} else if (type == 2) {
-		    ss << "NULL" << endl;
-		} else if (type == 3) {
-		    ss << "undef" << endl;
-		} else if (type == 4) {
+		    ss << "(float) " << f;
+		}
+		else if (type == 2)
+		{
+		    ss << "NULL";
+		}
+		else if (type == 3)
+		{
+		    ss << "undef";
+		}
+		else if (type == 4)
+		{
 		    // contents of register
 		    int	reg = instruction_data[3 + i];
 		    i++;
-		    ss << "reg[" << reg << "]" << endl;
-		} else if (type == 5) {
+		    ss << "reg[" << reg << "]";
+		}
+		else if (type == 5)
+		{
 		    int	bool_val = instruction_data[3 + i];
 		    i++;
-		    ss << "bool(" << bool_val << ")" << endl;
-		} else if (type == 6) {
+		    ss << "bool(" << bool_val << ")";
+		}
+		else if (type == 6)
+		{
 		    // double in wacky format: 45670123
 		    double d = convert_double_wacky(instruction_data + 3 + i);
 		    i += 8;
-		    ss << "(double) " << d << endl;
-		} else if (type == 7) {
-		    // int32_t
-		    int32_t	val = instruction_data[3 + i]
+		    ss << "(double) " << d;
+		}
+		else if (type == 7)
+		{
+		    // boost::int32_t
+		    boost::int32_t	val = instruction_data[3 + i]
 			| (instruction_data[3 + i + 1] << 8)
 			| (instruction_data[3 + i + 2] << 16)
 			| (instruction_data[3 + i + 3] << 24);
 		    i += 4;
-		    ss << "(int) " << val << endl;
-		} else if (type == 8) {
+		    ss << "(int) " << val;
+		}
+		else if (type == 8)
+		{
 		    int	id = instruction_data[3 + i];
 		    i++;
-		    ss << "dict_lookup[" << id << "]" << endl;
-		} else if (type == 9) {
+		    ss << "dict_lookup[" << id << "]";
+		}
+		else if (type == 9)
+		{
 		    int	id = instruction_data[3 + i] | (instruction_data[3 + i + 1] << 8);
 		    i += 2;
-		    ss << "dict_lookup_lg[" << id << "]" << endl;
+		    ss << "dict_lookup_lg[" << id << "]";
 		}
 	    }
-	} else if (fmt == ARG_DECL_DICT) {
+	}
+	else if (fmt == ARG_DECL_DICT)
+	{
 	    int	i = 0;
 	    int	count = instruction_data[3 + i] | (instruction_data[3 + i + 1] << 8);
 	    i += 2;
 	    
-	    ss << " [" << count << "]" << endl;
+	    ss << " [" << count << "] ";
 	    
 	    // Print strings.
-	    for (int ct = 0; ct < count; ct++) {
-		ss << "\t" << ct << ") "; 
+	    for (int ct = 0; ct < count; ct++)
+	    {
+		if ( ct ) ss << ", ";
+
+		ss << ct << ":"; 
 		
 		string str;
-		while (instruction_data[3 + i]) {
+		while (instruction_data[3 + i])
+		{
 			// safety check.
-		    if (i >= length) {
+		    if (i >= length)
+		    {
 			log_debug("<disasm error -- length exceeded>");
 			break;
 		    }
 		    str += instruction_data[3 + i];
 		    i++;
 		}
-		ss << "\"" << str.c_str() << "\"" << endl;
+		ss << "\"" << str.c_str() << "\"";
 		i++;
 	    }
-	} else if (fmt == ARG_FUNCTION2) {
+	}
+	else if (fmt == ARG_FUNCTION2)
+	{
 	    // Signature info for a function2 opcode.
 	    int	i = 0;
 	    const char*	function_name = (const char*) &instruction_data[3 + i];
@@ -333,11 +395,11 @@ disasm_instruction(const unsigned char* instruction_data)
 	    int	reg_count = instruction_data[3 + i];
 	    i++;
 
-	    ss << "\t\tname = '" << function_name << "'"
+	    ss << "\tname = '" << function_name << "'"
 		       << " arg_count = " << arg_count
-		       << " reg_count = " << reg_count << endl;
+		       << " reg_count = " << reg_count;
 	    
-	    uint16_t	flags = (instruction_data[3 + i]) | (instruction_data[3 + i + 1] << 8);
+	    boost::uint16_t	flags = (instruction_data[3 + i]) | (instruction_data[3 + i + 1] << 8);
 	    i += 2;
 	    
 	    // @@ What is the difference between "super" and "_parent"?
@@ -352,45 +414,36 @@ disasm_instruction(const unsigned char* instruction_data)
 	    bool	suppress_this  = (flags & 0x02) != 0;
 	    bool	preload_this   = (flags & 0x01) != 0;
 	    
-	    log_msg("\t\t        pg = %d\n"
-		    "\t\t        pp = %d\n"
-		    "\t\t        pr = %d\n"
-		    "\t\tss = %d, ps = %d\n"
-		    "\t\tsa = %d, pa = %d\n"
-		    "\t\tst = %d, pt = %d\n",
-		    int(preload_global),
-		    int(preload_parent),
-		    int(preload_root),
-		    int(suppress_super),
-		    int(preload_super),
-		    int(suppress_args),
-		    int(preload_args),
-		    int(suppress_this),
-		    int(preload_this));
-	    
-	    for (int argi = 0; argi < arg_count; argi++) {
+	    ss << " pg=" << preload_global
+		<< " pp=" << preload_parent
+		<< " pr=" << preload_root
+		<< " ss=" << suppress_super
+		<< " ps=" << preload_super
+		<< " sa=" << suppress_args
+		<< " pa=" << preload_args
+		<< " st=" << suppress_this
+		<< " pt=" << preload_this;
+
+	    for (int argi = 0; argi < arg_count; argi++)
+	    {
 		int	arg_register = instruction_data[3 + i];
 		i++;
 		const char*	arg_name = (const char*) &instruction_data[3 + i];
 		i += strlen(arg_name) + 1;
 		
-		ss << "\t\targ[" << argi << "]"
+		ss << "\targ[" << argi << "]"
 			   << " - reg[" << arg_register << "]"
-			   << " - '" << arg_name << "'" << endl;
+			   << " - '" << arg_name << "'";
 	    }
 	    
 	    int	function_length = instruction_data[3 + i] | (instruction_data[3 + i + 1] << 8);
 	    i += 2;
 	    
-	    ss << "\t\tfunction length = " << function_length << endl;
+	    ss << "\t\tfunction length = " << function_length;
 	}
-    } else {
-	ss << endl;
+	ss << ")";
     }
 
-    //dbglogfile.setStamp(false); // why ?
-    //log_msg("%s", ss.str().c_str());
-    //dbglogfile.setStamp(true);
     return ss.str();
 }
 
@@ -423,16 +476,16 @@ convert_float_little(const void *p)
 	// Hairy union for endian detection and munging
 	union {
 		float	f;
-		uint32_t i;
+		boost::uint32_t i;
 		struct {	// for endian detection
-			uint16_t s0;
-			uint16_t s1;
+			boost::uint16_t s0;
+			boost::uint16_t s1;
 		} s;
 		struct {	// for byte-swapping
-			uint8_t c0;
-			uint8_t c1;
-			uint8_t c2;
-			uint8_t c3;
+			boost::uint8_t c0;
+			boost::uint8_t c1;
+			boost::uint8_t c2;
+			boost::uint8_t c3;
 		} c;
 	} u;
 
@@ -443,7 +496,7 @@ convert_float_little(const void *p)
 		break;
 	case 0x3f80:	// big-endian host
 	    {
-		const uint8_t *cp = (const uint8_t *) p;
+		const boost::uint8_t *cp = (const boost::uint8_t *) p;
 		u.c.c0 = cp[3];
 		u.c.c1 = cp[2];
 		u.c.c2 = cp[1];
@@ -452,7 +505,7 @@ convert_float_little(const void *p)
 	    break;
 	default:
 	    log_error(_("Native floating point format not recognised"));
-	    assert(0);
+	    abort();
 	}
 	
 	return u.f;
@@ -464,29 +517,29 @@ convert_float_little(const void *p)
 static double
 convert_double_wacky(const void *p)
 {
-	const uint8_t *cp = (const uint8_t *)p;	// Handy uchar version
+	const boost::uint8_t *cp = (const boost::uint8_t *)p;	// Handy uchar version
 	union {
 		double	d;
-		uint64_t	i;
+		boost::uint64_t	i;
 		struct {
-			uint32_t l0;
-			uint32_t l1;
+			boost::uint32_t l0;
+			boost::uint32_t l1;
 		} l;
 		struct {
-			uint16_t s0;
-			uint16_t s1;
-			uint16_t s2;
-			uint16_t s3;
+			boost::uint16_t s0;
+			boost::uint16_t s1;
+			boost::uint16_t s2;
+			boost::uint16_t s3;
 		} s;
 		struct {
-			uint8_t c0;
-			uint8_t c1;
-			uint8_t c2;
-			uint8_t c3;
-			uint8_t c4;
-			uint8_t c5;
-			uint8_t c6;
-			uint8_t c7;
+			boost::uint8_t c0;
+			boost::uint8_t c1;
+			boost::uint8_t c2;
+			boost::uint8_t c3;
+			boost::uint8_t c4;
+			boost::uint8_t c5;
+			boost::uint8_t c6;
+			boost::uint8_t c7;
 		} c;
 	} u;
 
@@ -528,7 +581,7 @@ convert_double_wacky(const void *p)
 		break;
 	default:
 		log_error(_("Native double floating point format not recognised"));
-		assert(0);
+		abort();
 	}
 
 	return u.d;
@@ -546,6 +599,12 @@ action_buffer::read_double_wacky(size_t pc) const
 	return(convert_double_wacky(&m_buffer[pc]));
 }
 
+const std::string&
+action_buffer::getDefinitionURL() const
+{
+	return _src.get_url();
+}
+
 }
 
 // Local Variables:
