@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,21 +15,13 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-//
-
 #ifndef GNASH_BITMAP_CHARACTER_DEF_H
 #define GNASH_BITMAP_CHARACTER_DEF_H
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
+#include "smart_ptr.h" // GNASH_USE_GC
 #include "gnash.h" // for bitmap_info definition
 #include "ref_counted.h" // for character_def inheritance
 #include "types.h"
-#include "container.h"
 #include "utility.h"
 #include "smart_ptr.h"
 #include "bitmap_info.h" // for dtor visibility by intrusive_ptr
