@@ -1,9 +1,9 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -15,14 +15,11 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-
 #ifndef GNASH_CHARACTER_DEF_H
 #define GNASH_CHARACTER_DEF_H
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include "resource.h" // for inheritance from resource class
@@ -33,6 +30,7 @@ class tu_file;
 namespace gnash {
 	class character;
 	class cache_options;
+	class matrix;
 	class rect;
 }
 
@@ -53,6 +51,8 @@ class character_def : public resource
 private:
 	int	m_id;
 		
+	// don't assign-to
+	character_def& operator= (const character_def&) { abort(); return *this; }
 public:
 	character_def()
 		:
@@ -60,8 +60,9 @@ public:
 		m_render_cache(NULL)
 		{
 		}
+
 	
-	virtual ~character_def() {}
+	virtual ~character_def();
 	
 	virtual void display(character* /*instance_info*/)
 	{
@@ -71,7 +72,11 @@ public:
 	//
 	/// Point coordinates are local coords (TWIPS)
 	///
-	virtual bool point_test_local(float /*x*/, float /*y*/)
+	/// @param wm
+	///	Current world matrix of the instance we want to check.
+	///	This is needed to properly scale non-scalable strokes.
+	///
+	virtual bool point_test_local(float /*x*/, float /*y*/, matrix& /*wm*/)
 	{
 		return false;
 	}
@@ -102,23 +107,47 @@ public:
 	// Declared as virtual here because generic_character needs access to it
 	virtual const rect&	get_bound() const = 0;
 	
-	//
-	// Caching.
-	//
-	
-	virtual void output_cached_data(tu_file* /*out*/,
-			const cache_options& /*options*/)
-	{
-	}
-
-	virtual void	input_cached_data(tu_file* /*in*/)
-	{
-	}
-		
 public:  
   
   /// Cache holder for renderer (contents depend on renderer handler)
+  /// Will be deleted by destructor of the character_def.
+  /// We could store by auto_ptr, but I'm afraid that would mean
+  /// including render_handler.h in this header, which I don't like.
+  /// (REF: PIMPL)
+  ///
   render_cache_manager* m_render_cache;
+
+protected:
+
+	/// Copy a character definition
+	//
+	/// The copy will have a NULL render cache object.
+	/// The only known use of copy constructor is from
+	/// duplicateMovieClip, in particular during copy
+	/// of the drawable object, which is a subclass
+	/// of a shape_character_def
+	///
+	/// The choice of NOT copying the cache manager
+	/// is a choice of simplicity. We can't copy the
+	/// pointer as the character_def destructor will
+	/// destroy it, and we don't want to destroy it twice.
+	/// We don't want to make a copy of the whole cache
+	/// as it might be a waste of resource, we don't want
+	/// to share ownership as some character_def ended up
+	/// NOT being immutable any more !! :(
+	///
+	/// By setting the cache to NULL we'll leave reconstruction
+	/// of a cache to the renderers.
+	///
+	/// TODO: improve by implementing copy on write for the cache ?
+	///
+	character_def(const character_def& o)
+		:
+		resource(), // this is a new resource, nothing to copy
+		m_id(o.m_id),
+		m_render_cache(NULL)
+	{}
+
 	
 };
 
