@@ -18,9 +18,10 @@
 #ifndef _GNASH_EDIT_TEXT_CHARACTER_DEF_H_
 #define _GNASH_EDIT_TEXT_CHARACTER_DEF_H_
 
+#include "smart_ptr.h" // GNASH_USE_GC
+#include "types.h"
 #include "character_def.h" // for inheritance
 #include "rect.h" // for composition
-#include "textformat.h" // for composition
 
 namespace gnash {
 
@@ -49,7 +50,11 @@ public:
 	edit_text_character_def(movie_definition* root_def)
 		:
 		m_root_def(root_def),
-		m_format(),
+
+		m_has_text(true), // For an SWF-defined textfield we'll read
+		                  // this from the tag. Dynamic textfields should
+		                  // behave as always having text by default (not tested).
+
 		m_word_wrap(false),
 		m_multiline(false),
 		m_password(false),
@@ -78,12 +83,6 @@ public:
 		m_color.set(0, 0, 0, 255);
 	}
 
-	/// Set the format of the text
-	void	set_format(text_format &format)
-	{
-		m_format = format;
-	}
-	
 	~edit_text_character_def()
 	{
 	}
@@ -212,6 +211,11 @@ public:
 		return m_word_wrap;
 	}
 
+	/// Has text defined ?
+	bool has_text() const {
+		return m_has_text;
+	}
+
 	/// Get root movie definition
 	movie_definition* get_root_def() {
 		return m_root_def;
@@ -266,7 +270,7 @@ private:
 
 	rect			m_rect;
 	std::string		m_variable_name;
-	text_format		m_format;
+	bool			m_has_text;
 	bool			m_word_wrap;
 	bool			m_multiline;
 	/// show asterisks instead of actual characters
