@@ -21,6 +21,7 @@
 #include "gnashconfig.h"
 #endif
 
+#include "smart_ptr.h" // GNASH_USE_GC
 #include "movie_def_impl.h"
 #include "movie_definition.h" // for inheritance
 #include "sprite_instance.h" // for ??
@@ -41,6 +42,7 @@
 #include "ControlTag.h"
 #include "sound_definition.h" // for sound_sample
 #include <boost/bind.hpp>
+#include <boost/version.hpp>
 
 #include <iomanip>
 #include <memory>
@@ -75,8 +77,9 @@ namespace gnash
 
 MovieLoader::MovieLoader(movie_def_impl& md)
 	:
-	_movie_def(md)
-	,_thread(NULL)
+	_movie_def(md),
+	_thread(NULL),
+	_barrier(2) // us and the main thread..
 {
 }
 
@@ -109,13 +112,20 @@ MovieLoader::isSelfThread() const
 	if (!_thread.get()) {
 		return false;
 	}
+#if BOOST_VERSION < 103500
 	boost::thread this_thread;
 	return this_thread == *_thread;
+#else
+	return boost::this_thread::get_id() == _thread->get_id();
+#endif
+
 }
 
+// static..
 void
-MovieLoader::execute(movie_def_impl* md)
+MovieLoader::execute(MovieLoader& ml, movie_def_impl* md)
 {
+	ml._barrier.wait(); // let _thread assignment happen before going on
 	md->read_all_swf();
 }
 
@@ -132,7 +142,9 @@ MovieLoader::start()
 	// Those tests do seem a bit redundant, though...
 	boost::mutex::scoped_lock lock(_mutex);
 
-	_thread.reset( new boost::thread(boost::bind(execute, &_movie_def)) );
+	_thread.reset( new boost::thread(boost::bind(execute, boost::ref(*this), &_movie_def)) );
+
+	_barrier.wait(); // let execution start befor returning
 
 	return true;
 }
@@ -475,9 +487,27 @@ movie_def_impl::readHeader(std::auto_ptr<tu_file> in, const std::string& url)
 		log_swferror("non-finite movie bounds");
 		);
 	}
+
+	// It seems frame rate is limited to a max 
+	// 84 was found by testing the pp, might be turned into a compile-time define
+	static const int maxfps = 84;
 	m_frame_rate = _str->read_u16();
-	if ( ! m_frame_rate ) m_frame_rate = 65535;
-	else m_frame_rate /= 256.0f;
+	if ( ! m_frame_rate )
+	{
+		log_debug("Frame rate of 0 taken as %d (upper bound)", maxfps);
+		m_frame_rate = maxfps;
+	}
+	else
+	{
+		m_frame_rate /= 256.0f;
+		if ( m_frame_rate > maxfps )
+		{
+			log_debug("Frame rate of %d too high, we'll use %d (upper bound)",
+				m_frame_rate, maxfps);
+			m_frame_rate = maxfps;
+		}
+	}
+
 	m_frame_count = _str->read_u16();
 
 	/* Markus: Probably this is better anyways */
@@ -820,9 +850,10 @@ parse_tag:
 	{
 		IF_VERBOSE_MALFORMED_SWF(
 		log_swferror(_(SIZET_FMT " frames advertised in header, but only " SIZET_FMT " SHOWFRAME tags "
-			"found in stream. Updating total frames count"), m_frame_count, floaded);
+			"found in stream. Pretending we loaded all advertised frames"), m_frame_count, floaded);
 		);
-		m_frame_count = floaded;
+		boost::mutex::scoped_lock lock(_frames_loaded_mutex);
+		_frames_loaded = m_frame_count;
 		// Notify any thread waiting on frame reached condition
 		_frame_reached_condition.notify_all();
 	}
@@ -842,9 +873,6 @@ movie_def_impl::incrementLoadedFrames()
 
 	++_frames_loaded;
 
-	// Close current frame definition in Timeline object
-	_timeline.closeFrame();
-
 	if ( _frames_loaded > m_frame_count )
 	{
 		IF_VERBOSE_MALFORMED_SWF(
@@ -1057,4 +1085,45 @@ movie_def_impl::markReachableResources() const
 }
 #endif // GNASH_USE_GC
 
+void
+movie_def_impl::importResources(boost::intrusive_ptr<movie_definition> source, Imports& imports)
+{
+	size_t importedSyms=0;
+	for (Imports::iterator i=imports.begin(), e=imports.end(); i!=e; ++i)
+	{
+		int id = i->first;
+		const std::string& symbolName = i->second;
+
+	        boost::intrusive_ptr<resource> res = source->get_exported_resource(symbolName);
+	        if (!res)
+	        {
+			log_error(_("import error: could not find resource '%s' in movie '%s'"),
+				symbolName, source->get_url());
+			continue;
+	        }
+	        else if (font* f = res->cast_to_font())
+		{
+			// Add this shared font to the currently-loading movie.
+			add_font(id, f);
+			++importedSyms;
+	        }
+	        else if (character_def* ch = res->cast_to_character_def())
+	        {
+			// Add this character to the loading movie.
+			add_character(id, ch);
+			++importedSyms;
+	        }
+	        else
+	        {
+			log_error(_("importResources error: unsupported import of '%s' from movie '%s' has unknown type"),
+				symbolName, source->get_url());
+	        }
+	}
+
+	if ( importedSyms )
+	{
+		_importSources.insert(source);
+	}
+}
+
 } // namespace gnash
