@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,14 +15,10 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-
 #ifndef GNASH_MOVIE_DEF_IMPL_H
 #define GNASH_MOVIE_DEF_IMPL_H
 
-#include "container.h"
-#include "smart_ptr.h"
+#include "smart_ptr.h" // GNASH_USE_GC
 #include "fontlib.h"
 #include "font.h"
 #include "jpeg.h"
@@ -32,13 +28,15 @@
 #include "bitmap_character_def.h" // for boost::intrusive_ptr visibility of dtor
 #include "resource.h" // for boost::intrusive_ptr visibility of dtor
 #include "stream.h" // for get_bytes_loaded
-#include "Timeline.h" // for composition 
+#include "StringPredicates.h" // for case-insensitive string comparision (ExportMap)
 
 #include <map> // for CharacterDictionary
+#include <set> // for _importSources
 #include <string>
 #include <memory> // for auto_ptr
 #include <boost/thread/thread.hpp>
 #include <boost/thread/condition.hpp>
+#include <boost/thread/barrier.hpp>
 
 //
 // Forward declarations
@@ -117,8 +115,13 @@ private:
 	mutable boost::mutex _mutex;
 	std::auto_ptr<boost::thread> _thread;
 
+	// Barrier to ensure that _thread
+	// is initialized before the loader thread
+	// continues execution
+	boost::barrier _barrier;
+
 	/// Entry point for the actual thread
-	static void execute(movie_def_impl* md);
+	static void execute(MovieLoader& ml, movie_def_impl* md);
 };
 
 /// The Characters dictionary associated with each SWF file.
@@ -188,7 +191,7 @@ private:
 /// Immutable definition of a movie's contents.
 //
 /// It cannot be played directly, and does not hold
-/// current state; for that you need to call create_instance()
+/// current state; for that you need to call create_movie_instance()
 /// to get a movie instance 
 ///
 class movie_def_impl : public movie_definition
@@ -197,6 +200,9 @@ private:
 	/// Characters Dictionary
 	CharacterDictionary	_dictionary;
 
+	/// Mutex protecting _dictionary
+	mutable boost::mutex _dictionaryMutex;
+
 	/// Tags loader table
 	SWF::TagLoadersTable& _tag_loaders;
 
@@ -209,18 +215,23 @@ private:
 	typedef std::map<int, boost::intrusive_ptr<sound_sample> > SoundSampleMap;
 	SoundSampleMap m_sound_samples;
 
-	/// A list of movie control events for each frame.
-	std::vector<PlayList> m_playlist;
+	typedef std::map<size_t, PlayList> PlayListMap;
 
-	/// Init actions for each frame.
-	std::vector<PlayList> m_init_action_list;
+	/// Movie control events for each frame.
+	PlayListMap m_playlist;
 
 	/// 0-based frame #'s
 	typedef std::map<std::string, size_t> NamedFrameMap;
-	NamedFrameMap m_named_frames;
+	NamedFrameMap _namedFrames;
 
-	typedef std::map<std::string, boost::intrusive_ptr<resource> > ExportMap;
-	ExportMap m_exports;
+	// Mutex protecting access to _namedFrames
+	mutable boost::mutex _namedFramesMutex;
+
+	typedef std::map<std::string, boost::intrusive_ptr<resource>, StringNoCaseLessThen > ExportMap;
+	ExportMap _exportedResources;
+
+	// Mutex protecting access to _exportedResources
+	mutable boost::mutex _exportedResourcesMutex;
 
 	/// Items we import.
 	std::vector<import_info> m_imports;
@@ -235,9 +246,6 @@ private:
 	typedef std::vector<boost::intrusive_ptr<bitmap_info> >	BitmapVect;
 	BitmapVect m_bitmap_list;
 
-	create_bitmaps_flag	m_create_bitmaps;
-	create_font_shapes_flag	m_create_font_shapes;
-
 	rect	m_frame_size;
 	float	m_frame_rate;
 	size_t	m_frame_count;
@@ -278,7 +286,7 @@ private:
 
 
 	int	m_loading_sound_stream;
-	uint32_t	m_file_length;
+	boost::uint32_t	m_file_length;
 
 	std::auto_ptr<jpeg::input> m_jpeg_in;
 
@@ -300,7 +308,9 @@ private:
 	///
 	/// NOTE: this method locks _frames_loaded_mutex
 	///
-	void incrementLoadedFrames();
+	/// @return the new value of _frames_loaded
+	///
+	size_t incrementLoadedFrames();
 
 	/// Set number of bytes loaded from input stream
 	//
@@ -312,12 +322,6 @@ private:
 		_bytes_loaded=bytes;
 	}
 
-	/// Timeline depth info
-	//
-	/// TODO: move to base class ?
-	///
-	Timeline _timeline;
-
 	/// A flag set to true when load cancelation is requested
 	bool _loadingCanceled;
 
@@ -325,9 +329,12 @@ private:
 	/// character_id is listed in the import table.
 	bool in_import_table(int character_id) const;
 
+	/// Movies we import resources from
+	std::set< boost::intrusive_ptr<movie_definition> > _importSources;
+
 public:
 
-	movie_def_impl(create_bitmaps_flag cbf, create_font_shapes_flag cfs);
+	movie_def_impl();
 
 	~movie_def_impl();
 
@@ -374,26 +381,6 @@ public:
 		return m_file_length;
 	}
 
-	/// Returns DO_CREATE_BITMAPS if we're supposed to
-	/// initialize our bitmap infos, or DO_NOT_INIT_BITMAPS
-	/// if we're supposed to create blank placeholder
-	/// bitmaps (to be init'd later explicitly by the host
-	/// program).
-	virtual create_bitmaps_flag get_create_bitmaps() const
-	{
-		return m_create_bitmaps;
-	}
-
-	/// Returns DO_LOAD_FONT_SHAPES if we're supposed to
-	/// initialize our font shape info, or
-	/// DO_NOT_LOAD_FONT_SHAPES if we're supposed to not
-	/// create any (vector) font glyph shapes, and instead
-	/// rely on precached textured fonts glyphs.
-	virtual create_font_shapes_flag	get_create_font_shapes() const
-	{
-	    return m_create_font_shapes;
-	}
-
 	/// All bitmap_info's used by this movie should be
 	/// registered with this API.
 	virtual void	add_bitmap_info(bitmap_info* bi)
@@ -438,6 +425,8 @@ public:
 	virtual void resolve_import(const std::string& source_url,
 		movie_definition* source_movie);
 
+	virtual void importResources(boost::intrusive_ptr<movie_definition> source, Imports& imports);
+
 	void add_character(int character_id, character_def* c);
 
 	/// \brief
@@ -449,6 +438,8 @@ public:
 
 	// See dox in movie_definition
 	//
+	// locks _namedFramesMutex
+	//
 	bool get_labeled_frame(const std::string& label, size_t& frame_number);
 
 	void	add_font(int font_id, font* f);
@@ -474,31 +465,34 @@ public:
 	int get_loading_sound_stream_id() { return m_loading_sound_stream; }
 
 	// See dox in movie_definition.h
-	void	add_execute_tag(execute_tag* tag)
+	void	addControlTag(ControlTag* tag)
 	{
 	    assert(tag);
-	    if (_frames_loaded < m_playlist.size()) {
-	      m_playlist[_frames_loaded].push_back(tag);
-	    }
-	}
-
-	/// Need to execute the given tag before entering the
-	/// currently-loading frame for the first time.
-	void	add_init_action(execute_tag* e)
-	{
-	    assert(e);
-	    assert(_frames_loaded < m_init_action_list.size());
-	    m_init_action_list[_frames_loaded].push_back(e);
+	    boost::mutex::scoped_lock lock(_frames_loaded_mutex);
+	    m_playlist[_frames_loaded].push_back(tag);
 	}
 
 	// See dox in movie_definition.h
+	//
+	// locks _namedFramesMutex and _frames_loaded_mutex
+	//
 	void add_frame_name(const std::string& name);
 
 	/// Set an input object for later loading DefineBits
 	/// images (JPEG images without the table info).
 	void	set_jpeg_loader(std::auto_ptr<jpeg::input> j_in)
 	{
-	    assert(m_jpeg_in.get() == NULL);
+	    if (m_jpeg_in.get())
+	    {
+	        /// There should be only one JPEGTABLES tag in an SWF (see: 
+	        /// http://www.m2osw.com/en/swf_alexref.html#tag_jpegtables)
+	        /// Discard any subsequent attempts to set the jpeg loader
+	        /// to avoid crashing on very malformed SWFs. (No conclusive tests
+	        /// for pp behaviour, though one version also crashes out on the
+	        /// malformed SWF that triggers this assert in Gnash).
+	        log_swferror(_("More than one JPEGTABLES tag found: not resetting JPEG loader"));
+	        return;
+	    }
 	    m_jpeg_in = j_in;
 	}
 
@@ -508,17 +502,16 @@ public:
 	    return m_jpeg_in.get();
 	}
 
-	virtual const PlayList& get_playlist(size_t frame_number) const
+	virtual const PlayList* getPlaylist(size_t frame_number) const
 	{
+#ifndef NDEBUG
+		boost::mutex::scoped_lock lock(_frames_loaded_mutex);
 		assert(frame_number <= _frames_loaded);
-		return m_playlist[frame_number];
-	}
+#endif
 
-	virtual const PlayList* get_init_actions(size_t frame_number)
-	{
-		assert(frame_number <= _frames_loaded);
-		//ensure_frame_loaded(frame_number);
-		return &m_init_action_list[frame_number];
+		PlayListMap::const_iterator it = m_playlist.find(frame_number);
+		if ( it == m_playlist.end() ) return NULL;
+		else return &(it->second);
 	}
 
 	/// Calls readHeader() and completeLoad() in sequence.
@@ -572,35 +565,17 @@ public:
 	/// Fill up *fonts with fonts that we own.
 	void get_owned_fonts(std::vector<font*>* fonts);
 
-	/// Generate bitmaps for our fonts, if necessary.
-	void generate_font_bitmaps();
-
-	/// Dump our cached data into the given stream.
-	void output_cached_data(tu_file* out,
-		const cache_options& options);
-
-	/// \brief
-	/// Read in cached data and use it to prime our
-	/// loaded characters.
-	void	input_cached_data(tu_file* in);
-
-	/// \brief
-	/// Create a playable movie_root instance from a def.
+	/// Create an instance of this movie.
 	//
+	/// TOCHECK:
 	/// Make sure you called completeLoad() before this
 	/// function is invoked (calling read() will do that for you).
 	///
+	/// TOCHECK:
 	/// The _root reference of the newly created movie_root
 	/// will be set to a newly created movie_instance.
 	///
-	/// WARNING: the actions in the first frame of the
-	///	     movie will be executed by this function.
-	///         
-	///
-	sprite_instance* create_instance();
-
-	/// Instanc of this definition is a valid movie_instance
-	movie_instance* create_movie_instance();
+	movie_instance* create_movie_instance(character* parent=0);
 
 	virtual const std::string& get_url() const { return _url; }
 	
@@ -610,38 +585,11 @@ public:
     // get_bound() is currently only used by generic_character which normally
     // is used only shape character definitions. See character_def.h to learn
     // why it is virtual anyway.
-    assert(0); // should not be called  
+    abort(); // should not be called  
 		static rect unused;
 		return unused;
 	}
 
-	// See dox in base class
-	//
-	// TODO: implement in base class ?
-	//
-	void addTimelineDepth(int depth, int ratio)
-	{
-		_timeline.addDepth(depth, ratio);
-	}
-
-	// See dox in base class
-	//
-	// TODO: implement in base class ?
-	//
-	void removeTimelineDepth(int depth)
-	{
-		_timeline.removeDepth(depth);
-	}
-
-	// See dox in base class
-	//
-	// TODO: implement in base class ?
-	//
-	void getTimelineDepths(size_t frameno, std::map<int, int>& depths)
-	{
-		_timeline.getFrameDepths(frameno, depths);
-	}
-
 protected:
 
 #ifdef GNASH_USE_GC
