@@ -14,6 +14,7 @@
 #include "gnash.h"
 #include "image.h"
 #include "render_handler.h"
+#include "dsodefs.h"
 
 
 namespace gnash {
@@ -37,20 +38,12 @@ namespace gnash {
 	///
 	namespace render
 	{
-
-		/// See render_handler::create_bitmap_info_alpha (in backend/render_handler.h)
-		bitmap_info*	create_bitmap_info_alpha(int w, int h,
-					unsigned char* data);
-
 		/// See render_handler::create_bitmap_info_rgb (in backend/render_handler.h)
 		bitmap_info*	create_bitmap_info_rgb(image::rgb* im);
 
 		/// See render_handler::create_bitmap_info_rgba (in backend/render_handler.h)
 		bitmap_info*	create_bitmap_info_rgba(image::rgba* im);
 
-		/// See render_handler::get_scale (in backend/render_handler.h)
-		void get_scale(point& scale);
-
 		/// See render_handler::delete_bitmap_info (in backend/render_handler.h)
 		void	delete_bitmap_info(bitmap_info* bi);
 
@@ -73,7 +66,7 @@ namespace gnash {
 
 		/// See render_handler::begin_display (in backend/render_handler.h)
 		void	begin_display(
-			rgba background_color,
+			const rgba& background_color,
 			int viewport_x0, int viewport_y0,
 			int viewport_width, int viewport_height,
 			float x0, float x1, float y0, float y1);
@@ -81,12 +74,6 @@ namespace gnash {
 		/// See render_handler::end_display (in backend/render_handler.h)
 		void	end_display();
 
-		/// See render_handler::set_matrix (in backend/render_handler.h)
-		void	set_matrix(const matrix& m);
-
-		/// See render_handler::set_cxform (in backend/render_handler.h)
-		void	set_cxform(const cxform& cx);
-
 		/// \brief
 		/// Draw triangles using the current fill-style 0.
 		/// Clears the style list after rendering.
@@ -95,16 +82,17 @@ namespace gnash {
 		/// triangle-strip order.  The type of the array should
 		/// be float[vertex_count*2]
 		///
-		void	draw_mesh_strip(const int16_t coords[],
+		void	draw_mesh_strip(const boost::int16_t coords[],
 				int vertex_count);
 
 		/// See render_handler::draw_line_strip (in backend/render_handler.h)
-		void	draw_line_strip(const int16_t coords[],
-				int vertex_count, const rgba color);
+		void	draw_line_strip(const boost::int16_t coords[],
+				int vertex_count, const rgba& color, const matrix& mat);
 
 		/// See render_handler::draw_poly (in backend/render_handler.h)
-		void  draw_poly(const point* corners, int corner_count,
-				const rgba fill, const rgba outline);
+		DSOEXPORT void  draw_poly(const point* corners, int corner_count,
+				const rgba& fill, const rgba& outline, const matrix& mat,
+				bool masked);
       
 		/// See render_handler::draw_shape_character (in backend/render_handler.h)
 		void draw_shape_character(shape_character_def *def,
@@ -112,11 +100,8 @@ namespace gnash {
       
 		/// See render_handler::draw_glyph (in backend/render_handler.h)
 		void draw_glyph(shape_character_def *def, const matrix& mat,
-				rgba color, float pixel_scale);
+				const rgba& color, float pixel_scale);
 
-		/// See render_handler::allow_glyph_textures (in backend/render_handler.h)
-		bool allow_glyph_textures();				
-				
 		/// See render_handler::bounds_in_clipping_area (in backend/render_handler.h)
 		bool bounds_in_clipping_area(const rect& bounds);
 		bool bounds_in_clipping_area(const InvalidatedRanges& ranges);
@@ -131,11 +116,6 @@ namespace gnash {
 		/// See render_handler::disable_mask (in backend/render_handler.h)
 		void	disable_mask();
 
-		/// See render_handler::draw_bitmap (in backend/render_handler.h)
-		void	draw_bitmap(const matrix& m, const bitmap_info* bi,
-				const rect& coords,
-				const rect& uv_coords, rgba color);
-
 	}	// end namespace render
 
 }	// end namespace gnash
