@@ -1,9 +1,10 @@
+// sprite_instance.h:  Stateful live Sprite instance, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -15,20 +16,15 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-
-
 // Stateful live Sprite instance
 
 #ifndef GNASH_SPRITE_INSTANCE_H
 #define GNASH_SPRITE_INSTANCE_H
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h" // GNASH_USE_GC, USE_SWFTREE
 #endif
 
-#include "edit_text_character.h" // temp hack
 #include "movie_definition.h" // for inlines
 #include "dlist.h" // DisplayList 
 #include "log.h"
@@ -36,6 +32,11 @@
 #include "DynamicShape.h" // for composition
 //#include "LoadVariablesThread.h" // for composition
 #include "Range2d.h"
+#include "dsodefs.h" // for DSOEXPORT
+
+#ifdef USE_SWFTREE
+# include "tree.hh"
+#endif
 
 #include <vector>
 #include <list>
@@ -48,6 +49,11 @@ namespace gnash {
 	class swf_event;
 	class drag_state;
 	class LoadVariablesThread;
+	class gradient_record;
+	class edit_text_character;
+	namespace SWF{
+		class PlaceObject2Tag;
+	}
 }
 
 namespace gnash
@@ -67,7 +73,7 @@ public:
 	typedef std::list<const action_buffer*> ActionList;
 
 	// definition must match movie_definition::PlayList
-	typedef std::vector<execute_tag*> PlayList;
+	typedef std::vector<ControlTag*> PlayList;
 
 	typedef std::vector<swf_event*> SWFEventsVector;
 
@@ -94,7 +100,7 @@ public:
 
 	/// Type of execute tags
 	//
-	/// TODO: move to execute_tag.h ?
+	/// TODO: move to ControlTag.h ?
 	///
 	enum control_tag_type
 	{
@@ -105,17 +111,16 @@ public:
 		TAG_DLIST  = 1<<1
 	};
 
-	virtual void has_key_event() {
-		m_has_key_event = true;
-	}
+	// Overridden to use the m_root member
+	virtual movie_instance* get_root() const;
 
-	virtual void has_mouse_event();
-
-	/// \brief
-	/// Return this sprite's relative root as
-	/// specified at contruction time
+	/// Return the _root ActionScript property of this sprite.
+	//
+	/// Relative or absolute is determined by
+	/// the _lockroot property, see getLockRoot
+	/// and setLockRoot.
 	///
-	virtual sprite_instance* get_root_movie();
+	virtual const sprite_instance* getAsRoot() const;
 
 	/// \brief
 	/// Return the sprite_definition (or movie_definition)
@@ -128,7 +133,7 @@ public:
 	/// Return version of the SWF definition of this instance
 	/// as been parsed from.
 	//
-        int getSWFVersion()
+        int getSWFVersion() const
 	{
                 return m_def->get_version();
         }
@@ -142,6 +147,13 @@ public:
 	// See dox in character.h
 	bool pointInVisibleShape(float x, float y) const;
 
+    /// return true if the given point is located in a(this) hitable sprite.
+    ///
+    /// all sprites except mouse-insensitive dynamic masks are hitable.
+    /// _visible property is ignored for hitable characters.
+    ///
+    bool pointInHitableShape(float x, float y) const;
+
 	/// Return 0-based index to current frame
 	size_t get_current_frame() const
 	{
@@ -183,11 +195,11 @@ public:
 	}
 
 	/// Stop or play the sprite.
-	void set_play_state(play_state s)
-	{
-	    //if (m_play_state != s) m_time_remainder = 0;
-	    m_play_state = s;
-	}
+	//
+	/// If stopped, any stream sound associated with this sprite
+	/// will also be stopped.
+	///
+	DSOEXPORT void set_play_state(play_state s);
 
 	play_state get_play_state() const { return m_play_state; }
 
@@ -197,10 +209,7 @@ public:
 	virtual float get_background_alpha() const;
 
 	// delegates to movie_root 
-	virtual float	get_pixel_scale() const;
-
-	// delegates to movie_root 
-	virtual void get_mouse_state(int& x, int& y, int& buttons);
+	//virtual void get_mouse_state(int& x, int& y, int& buttons);
 
 	// delegates to movie_root (possibly wrong)
 	void	set_background_color(const rgba& color);
@@ -234,18 +243,28 @@ public:
 	/// covers that can receive mouse events.  NULL if
 	/// none.  Coords are in parent's frame.
 	virtual character* get_topmost_mouse_entity(float x, float y);
+
+    // see dox in character.h
+	const character* findDropTarget(float x, float y, character* dragging) const;
+
+    void setDropTarget(const std::string& tgt)
+    {
+        _droptarget = tgt;
+    }
+
+    const std::string& getDropTarget() const
+    {
+        return _droptarget;
+    }
 	
-	virtual bool wantsInstanceName()
+	virtual bool wantsInstanceName() const
 	{
 		return true; // sprites can be referenced 
 	}
 
-	virtual void	advance(float delta_time);
-	//virtual void	advance_root(float delta_time);
-	virtual void	advance_sprite(float delta_time);
+	virtual void	advance();
 
-	/// Take care of this frame's actions.
-	void do_actions();
+	void	advance_sprite();
 
 	/// Set the sprite state at the specified frame number.
 	//
@@ -273,6 +292,30 @@ public:
 	///
 	bool get_frame_number(const as_value& frame_spec, size_t& frameno) const;
 
+	/// MovieClip instances need to handle cxform specially 
+	//
+	/// This is to suppor the Color asobject class
+	///
+	cxform	get_world_cxform() const;
+
+	/// Update user-defined color transform
+	//
+	/// This should only be used by the Color AS class
+	///
+	void set_user_cxform(const cxform& cx)
+	{
+		set_invalidated();
+		_userCxform = cx;
+	}
+
+	/// Return the user-defined color transform
+	//
+	/// This should only be used by the Color AS class
+	///
+	cxform  get_user_cxform() const
+	{
+		return _userCxform;
+	}
 
 	/// Look up the labeled frame, and jump to it.
 	bool goto_labeled_frame(const std::string& label);
@@ -280,6 +323,8 @@ public:
 		
 	/// Display (render?) this Sprite/MovieClip, unless invisible
 	void	display();
+	
+	void omit_display();
 
 	/// Swap depth of the given characters in the DisplayList
 	//
@@ -301,78 +346,64 @@ public:
 	boost::intrusive_ptr<character> add_textfield(const std::string& name,
 			int depth, float x, float y, float width, float height);
 
-	/// Place a character or mask to the DisplayList (make a timeline instance) 
+	/// Place a character or mask to the DisplayList.
 	//
-	/// This method instantiates the given character definition (character_id)
-	/// and places it on the stage at the given depth using the specified
-	/// parameters for name, position, color, ratio(?) and clipping depth (if any).
+	/// This method instantiates the given character definition
+	/// and places it on the stage at the given depth.
 	///
-	/// If the specified depth is already occupied by an instance of the same
-	/// character (character_id), the existing instance will be transformed using
-	/// move_display_object() instead.
+	/// If the specified depth is already occupied, it results a no-ops.
+	/// Otherwise, a new character will be created and onload handler will be triggerred.
 	///
-	/// If the specified depth is already occupied by an instance of a different
-	/// character (including any dynamically-created instance), the behaviour is
-	/// controlled by the replace_if_depth_is_occupied parameter. If false, this
-	/// call will result in a no-op. If true, the previously existing character
-	/// will be replaced by the new one, with unload() method invoked on the
-	/// removed character.
+	/// @param tag
+	/// A swf defined placement tag(PlaceObject, or PlaceObject2, or PlaceObject3)
+	/// No ownership transfer, the tag is still owned by the movie_definition class.
 	///
-	/// Any successful new placement triggers invokation of the newly created
-	/// instance's LOAD event.
+	/// @return
+	///     A pointer to the character being added or NULL
 	///
+	character* add_display_object(const SWF::PlaceObject2Tag* tag, DisplayList& dlist);
+	/// Proxy of DisplayList::move_character()
+	void move_display_object(const SWF::PlaceObject2Tag* tag, DisplayList& dlist);
+	/// Proxy of DisplayList::replace_character()
+	void replace_display_object(const SWF::PlaceObject2Tag* tag, DisplayList& dlist);
+	/// Proxy of DisplayList::remove_character()
+	void remove_display_object(const SWF::PlaceObject2Tag* tag, DisplayList& dlist);
+	/// Proxy of DisplayList::remove_character()
 	///
-	/// @param character_id
-	///	The ID of the character to be added.
-	///	It will be seeked in the CharacterDictionary
-	///
-	/// @param name
-	///	The name to give to the newly created instance.
-	///	If NULL, the new instance will be assigned a sequential
-	///	name in the form 'instanceN', where N is incremented
-	///	at each call, starting from 1.
-	///
-	/// @param event_handlers
+	/// @param ch
+	/// new character to be used for replacing.
 	///
 	/// @param depth
-	///	The depth to assign to the newly created instance.
+	/// depth at which the old character is to be replaced.
 	///
-	/// @param replace_if_depth_is_occupied
-	///	This parameter control behaviour in case an existing instance is found at
-	///	the given depth but is NOT an instance of the given character_id.
-	///	In this case: if this paremeter is true, the old instance will be replaced by
-	///	the new one, otherwise nothing happens and this function returns NULL.
+	/// @use_old_cxform
+	/// if true, the cxform of the new character will be set to the old one.
+	/// if false, the cxform of the new character will be untouched.
 	///
-	/// @param color_transform
-	///	The color transform to apply to the newly created instance.
+	/// @use_old_matrix
+	/// if true, the transformation matrix of the new character will be set to the old one.
+	/// if false, the transformation matrix of the new character will be untouched.
 	///
-	/// @param matrix
-	///	The matrix transform to apply to the newly created instance.
+	void replace_display_object(character* ch,	int depth,
+		bool use_old_cxform,
+		bool use_old_matrix);
+
+
+	/// \brief
+	/// Remove the object at the specified depth.
+	//
+	/// NOTE: 
+	/// (1)the id parameter is currently unused, but 
+	/// required to avoid breaking of inheritance from movie.h.
+	/// (2)the id might be used for specifying a character
+	/// in the depth(think about multiple characters within the same
+	/// depth, not tested and a rare case)
 	///
-	/// @param ratio
-	///
-	/// @param clip_depth
-	///	If != character::noClipDepthValue, mark the created instance
-	///	as a clipping layer. The shape of the placed character will be
-	///	used as a mask for all higher depths up to this value.
-	///	TODO: assert(clip_depth > depth) ?
-	///
-	/// @return 
-	///	A pointer to the character being added or NULL
-	///	if this call results in a move of an existing character 
-	///	or in a no-op due to replace_if_depth_is_occupied being
-	///	false.
-	///       
-	character* add_display_object(
-		uint16_t character_id,
-		const char* name,
-		const SWFEventsVector& event_handlers,
-		int depth,
-		bool replace_if_depth_is_occupied,
-		const cxform& color_transform,
-		const matrix& matrix,
-		float ratio,
-		int clip_depth);
+	void	remove_display_object(int depth, int /* id */)
+	{
+	    set_invalidated();
+	    m_display_list.remove_character(depth);
+	}
 
 	/// Attach the given character instance to current display list
 	//
@@ -387,107 +418,72 @@ public:
 	///
 	bool attachCharacter(character& newch, int depth);
 
-	/// Construct this instance as an ActionScript object.
+	/// Handle placement event
 	//
-	/// This function must be called when the sprite is placed on
-	/// stage for the first time. It will take care of invoking
-	/// the constructor of its associated class, either MovieClip
-	/// or any user-specified one (see sprite_definition::registerClass).
+	/// This callback will (not known to be a problem):
 	///
-	/// Make sure this sprite got an instance name before calling
-	/// this method (it's needed for properly setting the "this" pointer
-	/// when calling user-defined constructors).
+	/// (1) Register ourselves with the global instance list
+	/// (2) Take note of our original target path
+	/// (3) Register as listener of core broadcasters
+	/// (4) Execute tags of frame 0
 	///
-	/// TODO: check if we only need to construct "named" instances
-	/// TODO: consider moving this function up to the 'character' class.
-	/// TODO: possibly have this function call the onConstruct() event handler
+	/// The callback will also (known to be bogus):
+	//
+	/// (1) Construct this instance as an ActionScript object.
+	///     See constructAsScriptObject() method.
 	///
-	void construct();
+	virtual void stagePlacementCallback();
 
 	/// Unload all contents in the displaylist and this instance
 	/// See character::unload for more info
-	void unload();
+	bool unload();
 
-	/// See DisplayList::move_display_object, this method is just a proxy to that...
+	/// Mark this sprite as destroyed
 	//
-	/// @param color_xform
-	///	The color tranform to assign to the new character.
-	///	If NULL the default color transform will be kept.
+	/// This is an override of character::destroy()
 	///
-	/// @param mat
-	///	The matrix tranform to assign to the new character.
-	///	If NULL the default matrix will be kept.
+	/// A sprite should be destroyed when is removed from the display
+	/// list and is not more needed for names (target) resolutions.
+	/// Sprites are needed for names resolution whenever themselves
+	/// or a contained object has an onUnload event handler defined, 
+	/// in which case we want the event handler to find the 'this'
+	/// variable w/out attempting to rebind it.
 	///
-	void	move_display_object(
-			int depth,
-			const cxform* color_xform,
-			const matrix* mat,
-			float ratio,
-			int clip_depth)
-	{
-	    m_display_list.move_display_object(depth, color_xform, mat, ratio, clip_depth);
-	}
-
-
+	/// When a sprite is destroyed, all its children are also destroyed.
+	/// 
+	/// Note: this function will release most memory associated with
+	/// the sprite as no members or drawable should be needed anymore.
 	///
-	/// @param color_xform
-	///	The color tranform to assign to the new character.
-	///	If NULL the default color transform will be kept.
-	///
-	/// @param mat
-	///	The matrix tranform to assign to the new character.
-	///	If NULL the default matrix will be kept.
-	///
-	void	replace_display_object(
-			uint16_t character_id,
-			const char* name,
-			int depth,
-			const cxform* color_xform,
-			const matrix* mat,
-			float ratio,
-			int clip_depth);
-
-
-	///
-	/// @param color_xform
-	///	The color tranform to assign to the new character.
-	///	If NULL the default color transform will be kept.
-	//
-	/// @param mat
-	///	The matrix tranform to assign to the new character.
-	///	If NULL the default matrix will be kept.
-	///
-	void	replace_display_object(
-			character* ch,
-			const char* name,
-			int depth,
-			const cxform* color_xform,
-			const matrix* mat,
-			float ratio,
-			int clip_depth);
-
-
-	/// \brief
-	/// Remove the object at the specified depth.
-	//
-	/// NOTE: the id parameter is unused, but currently
-	/// required to avoid break of inheritance from movie.h
-	///
-	void	remove_display_object(int depth, int /* id */)
-	{
-	    set_invalidated();
-	    m_display_list.remove_display_object(depth);
-	}
-
-
+	void destroy();
+		
 	/// Add the given action buffer to the list of action
 	/// buffers to be processed at the end of the next
 	/// frame advance.
 	void	add_action_buffer(const action_buffer* a)
 	{
-	    m_action_list.push_back(a);
+		if ( ! _callingFrameActions ) queueAction(*a);
+		else execute_action(*a);
 	}
 
+	
+	/// \brief
+	/// Execute the given init action buffer, if not done yet
+	/// for the target character id.
+	//
+	/// The action will normally be pushed on queue, but will
+	/// be executed immediately if we are executing actions
+	/// resulting from a callFame instead.
+	///
+	/// @param a
+	///	The action buffer to execute
+	///
+	/// @param cid
+	///	The referenced character id
+	///
+	void execute_init_action_buffer(const action_buffer& a, int cid);
+
+	/// Execute a single action buffer (DOACTION block)
+	void execute_action(const action_buffer& ab);
 
 	/// For debugging -- return the id of the character
 	/// at the specified depth.
@@ -498,9 +494,15 @@ public:
 
 	/// Load a movie in this sprite, replacing it
 	//
+	/// @param url
+	///  The url to load the movie from. Can be a bitmap or an SWF.
+	///
+	/// @param postdata
+	///  IF not NULL, use as the POST body for HTTP requests
+	///
 	/// Return: true if it succeeded, false otherwise
 	///
-	bool loadMovie(const URL& url);
+	bool loadMovie(const URL& url, const std::string* postdata=NULL);
 
 	/// \brief
 	/// Load url-encoded variables from the given url, optionally
@@ -523,35 +525,22 @@ public:
 	///	If 1, GET will be used.
 	///	If 2, POST will be used.
 	///
-	void loadVariables(const URL& url, short sendVarsMethod=0);
+	void loadVariables(URL url, short sendVarsMethod=0);
 
 	//
 	// ActionScript support
 	//
 
-
-	/// Set the named variable to the value
-	virtual void set_variable(const char* path_to_var,
-		const char* new_value);
-
-	/// Set the named variable to the wide value
-	//
-	/// TODO: check if worth deprecating
-	virtual void set_variable(const char* path_to_var,
-		const wchar_t* new_value);
-
 	// See dox in as_object.h
-	bool get_member(const std::string& name, as_value* val);
-
+	bool get_member(string_table::key name, as_value* val, 
+		string_table::key nsname = 0);
 		
-	/// Set the named member to the value. 
-	//
-	/// Return true if we have that member; false otherwise.
-	///
-	virtual void set_member(const std::string& name, const as_value& val);
+	// See dox in as_object.h
+	virtual bool set_member(string_table::key name, const as_value& val,
+		string_table::key nsname = 0, bool ifFound=false);
 
 	/// Overridden to look in DisplayList for a match
-	virtual character* get_relative_target(const std::string& name);
+	as_object* get_path_element(string_table::key key);
 
 	/// Execute the actions for the specified frame. 
 	//
@@ -600,7 +589,7 @@ public:
 	/// FIXME: unimplemented
 	virtual void	on_event_xmlsocket_onxml()
 	{
-	    log_msg("FIXME: %s: unimplemented\n", __FUNCTION__);
+	    log_debug("FIXME: %s: unimplemented\n", __FUNCTION__);
 	    on_event(event_id::SOCK_XML);
 	}
 		
@@ -608,14 +597,14 @@ public:
 	/// specified interval.
 	virtual void	on_event_interval_timer()
 	{
-	    log_msg("FIXME: %s: unimplemented\n", __FUNCTION__);
+	    log_debug("FIXME: %s: unimplemented\n", __FUNCTION__);
 	    on_event(event_id::TIMER);
 	}
 
 	/// Do the events that happen as a MovieClip (swf 7 only) loads.
 	virtual void	on_event_load_progress()
 	{
-	    log_msg("FIXME: %s: unimplemented\n", __FUNCTION__);
+	    log_debug("FIXME: %s: unimplemented\n", __FUNCTION__);
 	    on_event(event_id::LOAD_PROGRESS);
 	}
 
@@ -638,10 +627,12 @@ public:
 			edit_text_character* ch);
 
 	void add_invalidated_bounds(InvalidatedRanges& ranges, bool force);
+	
+	void dump_character_tree(const std::string prefix) const;
 			
 
 	const DisplayList& getDisplayList() const {
-		return m_display_list;
+			return m_display_list;
 	}
 
 	/// Return the next highest available depth
@@ -656,20 +647,22 @@ public:
 
 	void testInvariant() const {
 		assert(m_play_state == PLAY || m_play_state == STOP);
-		assert(m_current_frame < m_def->get_frame_count());
+
+		// m_current_frame may be 0, since this is our initial
+		// condition. Still, frame count might be 0 as well, and
+		// loaded frames too !
+		//assert(m_current_frame < m_def->get_frame_count());
+#ifndef GNASH_USE_GC 
 		assert(get_ref_count() > 0); // or we're constructed but
 		                             // not stored in a boost::intrusive_ptr
+#endif
 	}
 
-	/// Set the current m_sound_stream_id
-	virtual void set_sound_stream_id(int id){ m_sound_stream_id = id; }
-
-	/// Get the current m_sound_stream_id
-	virtual int get_sound_stream_id() { return m_sound_stream_id;}
-
-	/// Override for character::set_name to proprely update
-	/// _target and _target_dot.
-	virtual void set_name(const char* name);
+	/// Set the currently playing m_sound_stream_id
+	// 
+	// TODO: rename to setStreamingSoundId
+	//
+	void setStreamSoundId(int id);
 
 	/// Remove this sprite from the stage.
 	//
@@ -697,9 +690,19 @@ public:
 
 	/// @{ Drawing API
 	
-	void lineStyle(uint16_t thickness, const rgba& color)
+	void lineStyle(boost::uint16_t thickness, const rgba& color,
+		bool vScale=true, bool hScale=true,
+		bool pixelHinting=false,
+		bool noClose=false,
+		cap_style_e startCapStyle=CAP_ROUND,
+		cap_style_e endCapStyle=CAP_ROUND,
+		join_style_e joinStyle=JOIN_ROUND,
+		float miterLimitFactor=1.0f)
 	{
-		_drawable->lineStyle(thickness, color);
+		_drawable->lineStyle(thickness, color, vScale, hScale,
+			pixelHinting, noClose,
+			startCapStyle, endCapStyle, joinStyle,
+			miterLimitFactor);
 	}
 
 	void resetLineStyle()
@@ -712,6 +715,16 @@ public:
 		_drawable->beginFill(color);
 	}
 
+	void beginLinearGradientFill(const std::vector<gradient_record>& grad, const matrix& mat)
+	{
+		_drawable->beginLinearGradientFill(grad, mat);
+	}
+
+	void beginRadialGradientFill(const std::vector<gradient_record>& grad, const matrix& mat)
+	{
+		_drawable->beginRadialGradientFill(grad, mat);
+	}
+
 	void endFill()
 	{
 		_drawable->endFill();
@@ -725,13 +738,13 @@ public:
 	void lineTo(float x, float y)
 	{
 		set_invalidated();
-		_drawable->lineTo(x, y);
+		_drawable->lineTo(x, y, getSWFVersion());
 	}
 
 	void curveTo(float cx, float cy, float ax, float ay)
 	{
 		set_invalidated();
-		_drawable->curveTo(cx, cy, ax, ay);
+		_drawable->curveTo(cx, cy, ax, ay, getSWFVersion());
 	}
 
 	void clear()
@@ -748,8 +761,62 @@ public:
 	/// Set all variables in the given map with their corresponding values
 	DSOEXPORT void setVariables(VariableMap& vars);
 
+	/// Enumerate child characters
+	//
+	/// See as_object::enumerateNonProperties(as_environment&) for more info.
+	///
+	virtual void enumerateNonProperties(as_environment&) const;
+
+	/// Delete characters removed from the stage
+	/// from the display lists
+	void cleanupDisplayList();
+
+        /// Queue the given action buffer
+        //
+        /// The action will be pushed on the current
+        /// global list (see movie_root).
+        ///
+        void queueAction(const action_buffer& buf);
+
+	/// Construct this instance as an ActionScript object
+	//
+	/// This method invokes the constructor associated with our
+	/// definition, either MovieClip or any user-speficied one
+	/// (see sprite_definition::registerClass). 
+	/// It will also invoke the onClipConstruct and onConstruct handlers.
+	///
+	void constructAsScriptObject();
+
+
+	/// Return true if get_root() should return the *relative* root,
+	/// false otherwise.
+	bool getLockRoot() const { return _lockroot; }
+
+	/// Set whether get_root() should return the *relative* root,
+	/// false otherwise. True for relative root.
+	///
+	void setLockRoot(bool lr) { _lockroot=lr; }
+
+	/// Getter-setter for MovieClip._lockroot
+	static as_value lockroot_getset(const fn_call& fn);
+
+#ifdef USE_SWFTREE
+	// Override to append display list info, see dox in character.h
+	virtual InfoTree::iterator getMovieInfo(InfoTree& tr, InfoTree::iterator it);
+#endif
+
 private:
 
+	void stopStreamSound();
+
+	/// Register this sprite as a listener of core broadcasters
+	//
+	/// This is currently only used for key events broadcaster
+	/// and it's supposed to be called only once (or should we
+	/// also call it whenever onKey* user-defined function is defined ?
+	///
+	void registerAsListener();
+
 	/// \brief
 	/// Return value of the 'enabled' property, casted to a boolean value.
 	/// True if not found (undefined to bool evaluates to false).
@@ -761,49 +828,34 @@ private:
 	///
 	bool isEnabled() const;
 
+	// See dox in character.h
+	bool allowHandCursor() const;
 
 	/// Forbid copy
-	sprite_instance(const sprite_instance&)
-		:
-		character(NULL, 0)
-	{
-		assert(0);
-	}
+	sprite_instance(const sprite_instance&);
 
 	/// Forbid assignment
-	sprite_instance& operator=(const sprite_instance&) { assert(0); return *this; }
+	sprite_instance& operator=(const sprite_instance&);
 
-	/// \brief
-	/// Call has_key_event() or has_mouse_event()
-	/// if the given string correspond to an event handler
-	/// for which registering as a listener of Mouse or Key is needed
+	/// Advance to a previous frame.
 	//
+	/// This function will basically restore the DisplayList as it supposedly
+	/// was *before* executing tags in target frame and then execute target
+	/// frame tags (both DLIST and ACTION ones).
 	///
-	/// @param name
-	///	Member name. 
-	///
-	void checkForKeyOrMouseEvent(const std::string& name);
-
-	/// Duplicate the object with the specified name
-	/// and add it with a new name  at a new depth.
-	void clone_display_object(const std::string& name,
-		const std::string& newname, int depth);
-
-	/// Reset the DisplayList for proper loop-back or goto_frame
-	//
-	/// The DisplayList is cleared by all but dynamic characters
-	///
-	void resetDisplayList();
-
-	/// Reconstruct the DisplayList for proper loop-back operations
-	//
-	/// This function will:
+	/// In practice, it will:
 	///
 	/// - Remove from current DisplayList:
 	///	- Timeline instances constructed after target frame 
 	///	- Timeline instances constructed before or at the target frame but no more at the original depth
 	///	- Dynamic instances found in the static depth zone
-	/// - Execute all displaylist tags from first to target frame, incrementing m_current_frame as it goes
+	/// - Execute all displaylist tags from first to one-before target frame,
+	///   appropriately setting m_current_frame as it goes, finally execute both displaylist and action
+	///   tags for target frame.
+	///
+	/// Callers of this methods are:
+	///	- goto_frame (for jump-backs)
+	///	- advance_sprite (for loop-back)
 	///
 	/// See: http://www.gnashdev.org/wiki/index.php/TimelineControl#Timeline_instances
 	///
@@ -815,8 +867,6 @@ private:
 	///
 	///	- m_current_frame == targetFrame
 	///
-	/// NOTES: resetDisplayList() above should likely just call restoreDisplayList(0);
-	///
 	/// TODO: consider using this same function for jump-forward too,
 	///       with some modifications...
 	///
@@ -829,9 +879,6 @@ private:
 	///
 	void queueActions(ActionList& action_list);
 
-	/// Execute a single action buffer (DOACTION block)
-	void execute_action(const action_buffer& ab);
-
 	/// Execute the actions in the action list
 	//
 	/// The list of action will be consumed starting from the first
@@ -845,13 +892,6 @@ private:
 	/// Current Display List contents.
 	DisplayList	m_display_list;
 
-	/// oldDisplayList is a backup of current DisplayList
-	/// (m_display_list) updated at each call to ::advance
-	/// and at first call to ::construct (it's empty in
-	/// this latter case).
-	/// It will be used to control actions execution order.
-	DisplayList	oldDisplayList;
-
 	/// The canvas for dynamic drawing
 	//
 	/// WARNING: since DynamicShape is a character_def, which is
@@ -870,8 +910,6 @@ private:
 	///
 	boost::intrusive_ptr<character> _drawable_inst;
 
-	ActionList	m_action_list;
-
 	// this is deprecated, we'll be pushing gotoframe target
 	// actions to the global action queue
 	//ActionList	m_goto_frame_action_list;
@@ -884,8 +922,8 @@ private:
 	// true if this sprite reached the last frame and restarted
 	bool		m_has_looped;
 
-	// a bit-array class would be ideal for this
-	std::vector<bool>	m_init_actions_executed;
+	// true is we're calling frame actions
+	bool _callingFrameActions;
 
 	/// This timeline's variable scope
 	as_environment	m_as_environment;
@@ -893,21 +931,19 @@ private:
 	/// Increment m_current_frame, and take care of looping.
 	void increment_frame_and_check_for_loop();
 
-	/// key events are: KEY_DOWN, KEY_PRESS, KEY_UP
-	bool m_has_key_event;
-
-	bool m_has_mouse_event;
+	typedef boost::intrusive_ptr< edit_text_character > TextFieldPtr;
+	typedef std::vector< TextFieldPtr > TextFieldPtrVect;
 
 	/// A container for textfields, indexed by their variable name
-	typedef std::map< std::string, boost::intrusive_ptr<edit_text_character> > TextfieldMap;
+	typedef std::map< std::string, TextFieldPtrVect > TextFieldMap;
 
 	/// We'll only allocate Textfield variables map if
 	/// we need them (ie: anyone calls set_textfield_variable)
 	///
-	std::auto_ptr<TextfieldMap> _text_variables;
+	std::auto_ptr<TextFieldMap> _text_variables;
 
 	/// \brief
-	/// Returns a TextField given it's variable name,
+	/// Returns a vector of TextField associated with the given variable name,
 	/// or NULL if no such variable name is known.
 	//
 	/// A TextField variable is a variable that acts
@@ -917,37 +953,31 @@ private:
 	///
 	/// @todo find out wheter we should be case sensitive or not
 	///
-	edit_text_character* get_textfield_variable(const std::string& name);
+	/// @return a pointer inside a vector, will be invalidated by modifications
+	///         of the vector (set_textfield_variable)
+	///
+	TextFieldPtrVect* get_textfield_variable(const std::string& name);
+
+	/// Unregister textfield variables bound to unloaded TextFields
+	void cleanup_textfield_variables();
 
 	/// soundid for current playing stream. If no stream set to -1
 	int m_sound_stream_id;
 
+	cxform _userCxform;
 
-	/// The DisplayList resulting by execution of tags in first frame.
-	//
-	/// It will be used to reinitialized actual DisplayList on restart.
-	/// See execute_frame_tags.
-	///
-	DisplayList _frame0_chars;
+	std::string _droptarget;
+
+	bool _lockroot;
 
 protected:
 
-	void place_character(character* ch, int depth,
-			const cxform& color_transform, const matrix& mat,
-			float ratio, int clip_depth)
-	{
-		m_display_list.place_character(ch, depth, color_transform, mat, ratio, clip_depth);
+	void place_character(character* ch, int depth)	
+	{		
+		m_display_list.place_character(ch, depth);	
 	}
 
 	/// Execute the tags associated with the specified frame.
-	//
-	/// Execution of 1st frame tags is specially handled:
-	///
-	/// - After executing them for the first time
-	///   the DisplayList state is saved into the _frame0_chars
-	///   member, which is used by resetDisplayList() on loop-back
-	///   TODO: drop this _frame0_chars thing ? See comments in
-	///   resetDisplayList()
 	///
 	/// @param frame
 	///	Frame number. 0-based
@@ -956,15 +986,13 @@ protected:
 	///     Which kind of control tags we want to execute. 
 	///	See control_tag_type enum.
 	///
-	void execute_frame_tags(size_t frame, int typeflags=TAG_DLIST|TAG_ACTION); // bool state_only = false;
+	void execute_frame_tags(size_t frame, DisplayList& dlist, int typeflags=TAG_DLIST|TAG_ACTION);
 
 	/// \brief
 	/// This is either sprite_definition (for sprites defined by
 	/// DefineSprite tag) or movie_def_impl (for the top-level movie).
 	boost::intrusive_ptr<movie_definition>	m_def;
 
-	bool m_on_event_load_called;
-
 	/// List of loadVariables requests
 	typedef std::list<LoadVariablesThread*> LoadVariablesThreads;
 
@@ -980,6 +1008,22 @@ protected:
 
 	/// Process a completed loadVariables request
 	void processCompletedLoadVariableRequest(LoadVariablesThread& request);
+
+#ifdef GNASH_USE_GC
+	/// Mark sprite-specific reachable resources and invoke
+	/// the parent's class version (markCharacterReachable)
+	//
+	/// sprite-specific reachable resources are:
+	/// 	- DisplayList items (current, backup and frame0 ones)
+	///	- Canvas for dynamic drawing (_drawable)
+	///	- Drawable instance (_drawable_inst)
+	///	- sprite environment
+	///	- definition the sprite has been instantiated from
+	///	- Textfields having an associated variable registered in this instance.
+	///	- Relative root of this instance (m_root)
+	///
+	virtual void markReachableResources() const;
+#endif // GNASH_USE_GC
 };
 
 /// Initialize the global MovieClip class
