@@ -67,7 +67,6 @@ public:
                   action_callback_t func, as_arg_t format);
     ActionHandler(action_type type, std::string name, 
                   action_callback_t func, as_arg_t format, int nargs);
-    ~ActionHandler();
 
     /// Execute the action
     void execute(ActionExec& thread) const;
@@ -157,12 +156,12 @@ private:
 	static void CommonSetTarget(ActionExec& thread, 
 			const std::string& target_name);
 
-        /// Common code for guessing at the encoding of random text, between
-        // Shift-Jis, UTF8, and other. Puts the character count in length,
-        // and the offsets to the characters in offsets, if offsets is not NULL.
-        // If not NULL, offsets should be at least s.length().
-        // offsets are not accurate if the return value is GUESSENC_OTHER
-        static as_encoding_guess_t GuessEncoding(std::string& s, int& length,
+    /// Common code for guessing at the encoding of random text, between
+    // Shift-Jis, UTF8, and other. Puts the character count in length,
+    // and the offsets to the characters in offsets, if offsets is not NULL.
+    // If not NULL, offsets should be at least s.length().
+    // offsets are not accurate if the return value is GUESSENC_OTHER
+    static as_encoding_guess_t guessEncoding(const std::string& s, int& length,
             std::vector<int>& offsets);
 
 	static void ActionEnd(ActionExec& thread);
