@@ -23,7 +23,7 @@
 #include "gnashconfig.h"
 #endif
 
-#include "smart_ptr.h" // for boost::intrusive_ptr
+#include "smart_ptr.h" // GNASH_USE_GC
 #include "movie_root.h" // for composition
 #include "GC.h" // for ineritance of VmGcRoot
 #include "string_table.h" // for the string table
@@ -274,12 +274,7 @@ public:
 	///
 	void markReachableResources() const;
 
-	void registerNative(as_c_function_ptr fun, unsigned int x, unsigned int y)
-	{
-		assert(fun);
-		assert(!_asNativeTable[x][y]);
-		_asNativeTable[x][y]=fun;
-	}
+	void registerNative(as_c_function_ptr fun, unsigned int x, unsigned int y);
 
 	/// Return a newly created builtin_function or null
 	builtin_function* getNative(unsigned int x, unsigned int y);
