@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,18 +15,9 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-//
-
 #ifndef GNASH_WITH_STACK_ENTRY_H
 #define GNASH_WITH_STACK_ENTRY_H
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-
 #include "as_object.h" // for dtor visibility by boost::intrusive_ptr
 #include "smart_ptr.h"
 
