@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+ *   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +24,6 @@
 #include "sprite_instance.h"
 #include "character.h"
 #include "dlist.h"
-#include "container.h"
 #include "log.h"
 
 #include "check.h"
@@ -38,9 +37,13 @@ void
 test_mouse_activity(MovieTester& tester, const character* text, const character* text2, bool covered, bool enabled)
 {
 	rgba red(255,0,0,255);
+	rgba dark_red(128,0,0,255);
 	rgba covered_red(127,126,0,255); // red, covered by 50% black
+	rgba covered_dark_red(64,120,0,255); // dark red, covered by 50% black
 	rgba yellow(255,255,0,255);
+	rgba dark_yellow(128,128,0,255);
 	rgba covered_yellow(128,255,0,255); // yellow, covered by 50% black
+	rgba covered_dark_yellow(64,184,0,255); // dark yellow, covered by 50% black
 	rgba green(0,255,0,255);
 
 	// roll over the middle of the square, this should change
@@ -50,16 +53,38 @@ test_mouse_activity(MovieTester& tester, const character* text, const character*
 		check_equals(string(text->get_text_value()), string("MouseOver"));
 		check_equals(string(text2->get_text_value()), string("RollOver"));
 		check(tester.isMouseOverMouseEntity());
-		// check that pixel @ 60,60 is yellow !
-		if ( covered ) { check_pixel(60, 60, 2, covered_yellow, 2);  }
-		else { check_pixel(60, 60, 2, yellow, 2);  }
+		if ( covered )
+		{
+			// check that pixel @ 60,60 is yellow (covered)
+			check_pixel(60, 60, 2, covered_yellow, 2); 
+			// check that pixel @ 72,64 is dark_yellow (covered)
+			check_pixel(72, 64, 2, covered_dark_yellow, 2); 
+		}
+		else
+		{
+			// check that pixel @ 60,60 is yellow 
+			check_pixel(60, 60, 2, yellow, 2); 
+			// check that pixel @ 72,64 is dark_yellow 
+			check_pixel(72, 64, 2, dark_yellow, 2); 
+		}
 	} else {
-		check_equals(string(text->get_text_value()), string("MouseOut"));
-		check_equals(string(text2->get_text_value()), string("RollOut"));
+		check_equals(string(text->get_text_value()), string("MouseUpOutside"));
+		check_equals(string(text2->get_text_value()), string("ReleaseOutside"));
 		check(!tester.isMouseOverMouseEntity());
-		// check that pixel @ 60,60 is red !
-		if ( covered ) { check_pixel(60, 60, 2, covered_red, 2);  }
-		else { check_pixel(60, 60, 2, red, 2);  }
+		if ( covered )
+		{
+			// check that pixel @ 60,60 is red  (covered)
+			check_pixel(60, 60, 2, covered_red, 2); 
+			// check that pixel @ 72,64 is dark_red  (covered)
+			check_pixel(72, 64, 2, covered_dark_red, 2); 
+		}
+		else
+		{
+			// check that pixel @ 60,60 is red 
+			check_pixel(60, 60, 2, red, 2); 
+			// check that pixel @ 72,64 is dark_red 
+			check_pixel(72, 64, 2, dark_red, 2); 
+		}
 	}
 
 	// press the mouse button, this should change
@@ -72,8 +97,8 @@ test_mouse_activity(MovieTester& tester, const character* text, const character*
 		// check that pixel @ 60,60 is green !
 		check_pixel(60, 60, 2, green, 2);
 	} else {
-		check_equals(string(text->get_text_value()), string("MouseOut"));
-		check_equals(string(text2->get_text_value()), string("RollOut"));
+		check_equals(string(text->get_text_value()), string("MouseUpOutside"));
+		check_equals(string(text2->get_text_value()), string("ReleaseOutside"));
 		check(!tester.isMouseOverMouseEntity());
 		// check that pixel @ 60,60 is red !
 		if ( covered ) { check_pixel(60, 60, 2, covered_red, 2);  }
@@ -91,8 +116,8 @@ test_mouse_activity(MovieTester& tester, const character* text, const character*
 		if ( covered ) { check_pixel(60, 60, 2, covered_yellow, 2);  }
 		else { check_pixel(60, 60, 2, yellow, 2);  }
 	} else {
-		check_equals(string(text->get_text_value()), string("MouseOut"));
-		check_equals(string(text2->get_text_value()), string("RollOut"));
+		check_equals(string(text->get_text_value()), string("MouseUpOutside"));
+		check_equals(string(text2->get_text_value()), string("ReleaseOutside"));
 		check(!tester.isMouseOverMouseEntity());
 		// check that pixel @ 60,60 is red !
 		if ( covered ) { check_pixel(60, 60, 2, covered_red, 2);  }
@@ -102,8 +127,13 @@ test_mouse_activity(MovieTester& tester, const character* text, const character*
 	// roll off the square, this should change
 	// the textfield value, if enabled
 	tester.movePointerTo(39, 60);
-	check_equals(string(text->get_text_value()), string("MouseOut"));
-	check_equals(string(text2->get_text_value()), string("RollOut"));
+	if ( enabled ) {
+		check_equals(string(text->get_text_value()), string("MouseOut"));
+		check_equals(string(text2->get_text_value()), string("RollOut"));
+	} else {
+		check_equals(string(text->get_text_value()), string("MouseUpOutside"));
+		check_equals(string(text2->get_text_value()), string("ReleaseOutside"));
+	}
 	check(!tester.isMouseOverMouseEntity());
 	// check that pixel @ 60,60 is red !
 	if ( covered ) { check_pixel(60, 60, 2, covered_red, 2); }
@@ -112,8 +142,13 @@ test_mouse_activity(MovieTester& tester, const character* text, const character*
 	// press the mouse button, this should not change anything
 	// as we're outside of the button.
 	tester.pressMouseButton();
-	check_equals(string(text->get_text_value()), string("MouseOut"));
-	check_equals(string(text2->get_text_value()), string("RollOut"));
+	if ( enabled ) {
+		check_equals(string(text->get_text_value()), string("MouseOut"));
+		check_equals(string(text2->get_text_value()), string("RollOut"));
+	} else {
+		check_equals(string(text->get_text_value()), string("MouseUpOutside"));
+		check_equals(string(text2->get_text_value()), string("ReleaseOutside"));
+	}
 	check(!tester.isMouseOverMouseEntity());
 	// check that pixel @ 60,60 is red !
 	if ( covered ) { check_pixel(60, 60, 2, covered_red, 2); }
@@ -122,8 +157,13 @@ test_mouse_activity(MovieTester& tester, const character* text, const character*
 	// depress the mouse button, this should not change anything
 	// as we're outside of the button.
 	tester.depressMouseButton();
-	check_equals(string(text->get_text_value()), string("MouseOut"));
-	check_equals(string(text2->get_text_value()), string("RollOut"));
+	if ( enabled ) {
+		check_equals(string(text->get_text_value()), string("MouseOut"));
+		check_equals(string(text2->get_text_value()), string("RollOut"));
+	} else {
+		check_equals(string(text->get_text_value()), string("MouseUpOutside"));
+		check_equals(string(text2->get_text_value()), string("ReleaseOutside"));
+	}
 	check(!tester.isMouseOverMouseEntity());
 	// check that pixel @ 60,60 is red !
 	if ( covered ) { check_pixel(60, 60, 2, covered_red, 2); }
@@ -141,8 +181,8 @@ test_mouse_activity(MovieTester& tester, const character* text, const character*
 		if ( covered ) { check_pixel(60, 60, 2, covered_yellow, 2);  }
 		else { check_pixel(60, 60, 2, yellow, 2);  }
 	} else {
-		check_equals(string(text->get_text_value()), string("MouseOut"));
-		check_equals(string(text2->get_text_value()), string("RollOut"));
+		check_equals(string(text->get_text_value()), string("MouseUpOutside"));
+		check_equals(string(text2->get_text_value()), string("ReleaseOutside"));
 		check(!tester.isMouseOverMouseEntity());
 		// check that pixel @ 60,60 is red !
 		if ( covered ) { check_pixel(60, 60, 2, covered_red, 2); }
@@ -158,8 +198,8 @@ test_mouse_activity(MovieTester& tester, const character* text, const character*
 		// check that pixel @ 60,60 is green !
 		check_pixel(60, 60, 2, rgba(0,255,0,255), 2);
 	} else {
-		check_equals(string(text->get_text_value()), string("MouseOut"));
-		check_equals(string(text2->get_text_value()), string("RollOut"));
+		check_equals(string(text->get_text_value()), string("MouseUpOutside"));
+		check_equals(string(text2->get_text_value()), string("ReleaseOutside"));
 		check(!tester.isMouseOverMouseEntity());
 		// check that pixel @ 60,60 is red !
 		if ( covered ) { check_pixel(60, 60, 2, covered_red, 2); }
@@ -173,11 +213,11 @@ test_mouse_activity(MovieTester& tester, const character* text, const character*
 	tester.depressMouseButton();
 
 	if ( enabled ) {
-		xcheck_equals(string(text->get_text_value()), string("MouseUpOutside"));
-		xcheck_equals(string(text2->get_text_value()), string("ReleaseOutside"));
+		check_equals(string(text->get_text_value()), string("MouseUpOutside"));
+		check_equals(string(text2->get_text_value()), string("ReleaseOutside"));
 	} else {
-		check_equals(string(text->get_text_value()), string("MouseOut"));
-		check_equals(string(text2->get_text_value()), string("RollOut"));
+		check_equals(string(text->get_text_value()), string("MouseUpOutside"));
+		check_equals(string(text2->get_text_value()), string("ReleaseOutside"));
 	}
 }
 
@@ -193,14 +233,10 @@ main(int /*argc*/, char** /*argv*/)
 	sprite_instance* root = tester.getRootMovie();
 	assert(root);
 
-	check_equals(root->get_frame_count(), 4);
+	check_equals(root->get_frame_count(), 7);
 
 	check_equals(root->get_current_frame(), 0);
 
-	const character* mc1 = tester.findDisplayItemByName(*root, "square1");
-	check(mc1);
-	check_equals(mc1->get_depth(), 2+character::staticDepthOffset);
-
 	const character* text = tester.findDisplayItemByName(*root, "textfield");
 	check(text);
 
@@ -213,27 +249,45 @@ main(int /*argc*/, char** /*argv*/)
 	check_equals(string(text->get_text_value()), idleString);
 	check_equals(string(text2->get_text_value()), idleString);
 	check_equals(string(text3->get_text_value()), idleString);
+
+	tester.advance();
+	check_equals(root->get_current_frame(), 1);
+
+	const character* mc1 = tester.findDisplayItemByName(*root, "square1");
+	check(mc1);
+	check_equals(mc1->get_depth(), 2+character::staticDepthOffset);
+
 	check(!tester.isMouseOverMouseEntity());
 	// check that pixel @ 60,60 is red !
 	rgba red(255,0,0,255);
 	check_pixel(60, 60, 2, red, 2);
 
-	for (size_t fno=0; fno<root->get_frame_count(); fno++)
+	tester.advance();
+	check_equals(root->get_current_frame(), 2);
+	tester.advance();
+	check_equals(root->get_current_frame(), 2); // need to roll out
+
+	tester.movePointerTo(60, 60); // roll over the square
+	check_equals(root->get_current_frame(), 2); // need to roll out
+	tester.movePointerTo(0, 0); // roll out, should go to next frame
+	check_equals(root->get_current_frame(), 3); 
+
+	for (size_t fno=root->get_current_frame(); fno<root->get_frame_count(); fno++)
 	{
 		const character* square_back = tester.findDisplayItemByDepth(*root, 1+character::staticDepthOffset);
 		const character* square_front = tester.findDisplayItemByDepth(*root, 3+character::staticDepthOffset);
 
 		switch (fno)
 		{
-			case 0:
+			case 3:
 				check(!square_back);
 				check(!square_front);
 				break;
-			case 1:
+			case 4:
 				check(square_back);
 				check(!square_front);
 				break;
-			case 2:
+			case 5:
 				check(square_back);
 				check(square_front);
 				break;
@@ -251,8 +305,9 @@ main(int /*argc*/, char** /*argv*/)
 
 	}
 
-	// last advance should restart the loop...
-	check_equals(root->get_current_frame(), 0);
+	// last advance should not restart the loop (it's in STOP mode)
+        check_equals(root->get_play_state(), sprite_instance::STOP);
+	check_equals(root->get_current_frame(), 6);
 
 }
 
