@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2007 Free Software Foundation, Inc.
+ *   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +24,6 @@
 #include "sprite_instance.h"
 #include "character.h"
 #include "dlist.h"
-#include "container.h"
 #include "log.h"
 
 #include "check.h"
@@ -41,6 +40,9 @@ main(int /*argc*/, char** /*argv*/)
 	string filename = string(TGTDIR) + string("/") + string(INPUT_FILENAME);
 	MovieTester tester(filename);
 
+	gnash::RcInitFile& rc = gnash::RcInitFile::getDefaultInstance();
+	rc.addLocalSandboxPath(MEDIADIR);
+
 	gnash::LogFile& dbglogfile = gnash::LogFile::getDefaultInstance();
 	dbglogfile.setVerbosity(1);
 
