@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2007 Free Software Foundation, Inc.
+ *   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +24,6 @@
 #include "sprite_instance.h"
 #include "character.h"
 #include "dlist.h"
-#include "container.h"
 #include "log.h"
 
 #include "check.h"
@@ -58,44 +57,67 @@ main(int /*argc*/, char** /*argv*/)
 	// TODO: change the scaling of the window/canvas
 	int scale_x = 1;
 	int scale_y = 1;
+
+	size_t framecount = root->get_frame_count();
+	check_equals(framecount, 11);
 	
 	// Just loop twice, so to catch crashes...
 	for (int j = 0; j < 2; ++j) {
+
+		if ( ! tester.canTestVideo() )
+		{
+			// no pixel checking, but we'd still be interested in crashes...
+			tester.advance();
+			continue;
+		}
+		
 		// Frame 1
 
-		// Check the color in (1,1) - should be red
-		check_pixel(1, 1, 1, red, 5);
+		check_equals(root->get_current_frame(), 0); // 0-based
 
-		// Check the color in (30,1) - should be yellow
-		check_pixel(35*scale_x, 1, 1, yellow, 5);
+		// Check the color in (5,5) - should be red
+		check_pixel(5, 5, 2, red, 5);
 
-		// Check the color in (1,30) - should be yellow
-		check_pixel(1, 35*scale_y, 1, yellow, 5);
+		// Check the color in (35,5) - should be yellow
+		check_pixel(35*scale_x, 5, 2, yellow, 5);
 
-		size_t framecount = root->get_frame_count();
-		int frame = 1;
-		int i = 0;
-		while (frame++ < framecount) {
+		// Check the color in (5,35) - should be yellow
+		check_pixel(5, 35*scale_y, 2, yellow, 5);
+
+		while (true)
+		{
 			// Frame X
 			tester.advance();
+			int framenum = root->get_current_frame();
+			cout << "---- Pixel checking in frame " << framenum+1 << " play state " << root->get_play_state() << endl;
+			
+			int i = (framenum-1)*10;
 
-			// Check the color in (9+i,1) - should be yellow
-			check_pixel((5 + i)*scale_x, 1, 1, yellow, 5);
+			// Check the color in (5+i,5) - should be yellow
+			check_pixel((5 + i)*scale_x, 5, 2, yellow, 5);
 
-			// Check the color in (25+i,1) - should be red
-			check_pixel((25 + i)*scale_x, 1, 1, red, 5);
+			// Check the color in (25+i,5) - should be red
+			check_pixel((25 + i)*scale_x, 5, 2, red, 5);
 
-			// Check the color in (25+i,30) - should be yellow
+			// Check the color in (25+i,35) - should be yellow
 			check_pixel((25 + i)*scale_x, 35*scale_y, 1, yellow, 5);
 
 			// The video is 128x96 so we don't want to check beyond that
 			if (45+i <= 128) {
-				// Check the color in (40+i,1) - should be yellow
-				check_pixel((45 + i)*scale_x, 1, 1, yellow, 5);
+				// Check the color in (45+i,5) - should be yellow
+				check_pixel((45 + i)*scale_x, 5, 2, yellow, 5);
 			}
 
-			i += 10;
+			if ( framenum == framecount-1 )
+			{
+				// check we're playing, or we'll never get to next loop...
+				check_equals(root->get_play_state(), sprite_instance::PLAY);
+				break;
+			}
+
+			tester.click();
 		}
+
 		tester.advance();
 	}
 
