@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+ *   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,8 +24,8 @@
 #include "sprite_instance.h"
 #include "character.h"
 #include "dlist.h"
-#include "container.h"
 #include "log.h"
+#include "VM.h"
 
 #include "check.h"
 #include <string>
@@ -79,23 +79,25 @@ main(int /*argc*/, char** /*argv*/)
 	// check that the pixel under the mouse is white
 	check_pixel(100, 30, 2, rgba(255,255,255,255), 2);
 
-	root->get_member("mousedown", &tmp);
+	string_table& st = VM::get().getStringTable();
+
+	root->get_member(st.find("mousedown"), &tmp);
 	check(tmp.is_undefined());
-	root->get_member("mouseup", &tmp);
+	root->get_member(st.find("mouseup"), &tmp);
 	check(tmp.is_undefined());
 
 	// Note that we are *not* on an active entity !
 	tester.pressMouseButton();
 
-	root->get_member("mousedown", &tmp);
+	root->get_member(st.find("mousedown"), &tmp);
 	check_equals(tmp.to_number(), 1);
-	check ( ! root->get_member("mouseup", &tmp) );
+	check ( ! root->get_member(st.find("mouseup"), &tmp) );
 
 	tester.depressMouseButton();
 
-	root->get_member("mousedown", &tmp);
+	root->get_member(st.find("mousedown"), &tmp);
 	check_equals(tmp.to_number(), 1);
-	root->get_member("mouseup", &tmp);
+	root->get_member(st.find("mouseup"), &tmp);
 	check_equals(tmp.to_number(), 1);
 
 	tester.advance();
@@ -152,7 +154,7 @@ main(int /*argc*/, char** /*argv*/)
 	// Note that we are *not* on an active entity !
 	tester.pressMouseButton();
 
-	root->get_member("mousedown", &tmp);
+	root->get_member(st.find("mousedown"), &tmp);
 	check_equals(tmp.to_number(), 5);
 
 }
