@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2007 Free Software Foundation, Inc.
+ *   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,8 +24,8 @@
 #include "sprite_instance.h"
 #include "character.h"
 #include "dlist.h"
-#include "container.h"
 #include "log.h"
+#include "VM.h"
 
 #include "check.h"
 #include <string>
@@ -53,6 +53,7 @@ main(int /*argc*/, char** /*argv*/)
 	check_equals(root->get_current_frame(), 0);
 	check_equals(root->get_play_state(), sprite_instance::PLAY);
 
+	tester.advanceClock(50); // "sleep" 50 milliseconds
 	tester.advance(); // execute actions in second frame frame
 
 	check_equals(root->get_current_frame(), 1);
@@ -60,65 +61,76 @@ main(int /*argc*/, char** /*argv*/)
 
 	// Now timers are set and counters initialized
 
-	root->get_member("this_counter", &tmp);
+	string_table& st = VM::get().getStringTable();
+	root->get_member(st.find("this_counter"), &tmp);
 	check_equals(tmp.to_number(), 0);
-	root->get_member("that_counter", &tmp);
+	root->get_member(st.find("that_counter"), &tmp);
 	check_equals(tmp.to_number(), 0);
 
-	usleep(500000); tester.advance(); // run expired timers
+	tester.advanceClock(500); // "sleep" 500 milliseconds
+	tester.advance(); // run expired timers
 
-	root->get_member("this_counter", &tmp);
+	root->get_member(st.find("this_counter"), &tmp);
 	check_equals(tmp.to_number(), 1);
-	root->get_member("that_counter", &tmp);
+	root->get_member(st.find("that_counter"), &tmp);
 	check_equals(tmp.to_number(), 0);
 
-	usleep(500000); tester.advance(); // run expired timers
+	tester.advanceClock(600); // "sleep" 500 milliseconds
+	tester.advance(); // run expired timers
 
-	root->get_member("this_counter", &tmp);
+	root->get_member(st.find("this_counter"), &tmp);
 	check_equals(tmp.to_number(), 2);
-	root->get_member("that_counter", &tmp);
+	root->get_member(st.find("that_counter"), &tmp);
 	check_equals(tmp.to_number(), 1);
 
-	usleep(500000); tester.advance(); // run expired timers
+	tester.advanceClock(500); // "sleep" 500 milliseconds
+	tester.advance(); // run expired timers
 
-	root->get_member("this_counter", &tmp);
+	root->get_member(st.find("this_counter"), &tmp);
 	check_equals(tmp.to_number(), 3);
-	root->get_member("that_counter", &tmp);
+	root->get_member(st.find("that_counter"), &tmp);
 	check_equals(tmp.to_number(), 1);
 
-	usleep(500000); tester.advance(); // run expired timers
+	tester.advanceClock(520); // "sleep" 520 milliseconds
+	tester.advance(); // run expired timers
 
-	root->get_member("this_counter", &tmp);
+	root->get_member(st.find("this_counter"), &tmp);
 	check_equals(tmp.to_number(), 4);
-	root->get_member("that_counter", &tmp);
+	root->get_member(st.find("that_counter"), &tmp);
 	check_equals(tmp.to_number(), 2);
 
-	usleep(1000000); tester.advance(); // run expired timers
+	tester.advanceClock(1020); // "sleep" 1020 milliseconds
+	tester.advance(); // run expired timers
 
-	root->get_member("this_counter", &tmp);
+	root->get_member(st.find("this_counter"), &tmp);
 	check_equals(tmp.to_number(), 4);
-	root->get_member("that_counter", &tmp);
+	root->get_member(st.find("that_counter"), &tmp);
 	check_equals(tmp.to_number(), 3);
 
-	usleep(1000000); tester.advance(); // run expired timers
+	tester.advanceClock(1020); // "sleep" 1020 milliseconds
+	tester.advance(); // run expired timers
 
-	root->get_member("this_counter", &tmp);
+	root->get_member(st.find("this_counter"), &tmp);
 	check_equals(tmp.to_number(), 4);
-	root->get_member("that_counter", &tmp);
+	root->get_member(st.find("that_counter"), &tmp);
 	check_equals(tmp.to_number(), 4);
 
-	usleep(500000); tester.advance(); // run expired timers
+	tester.advanceClock(520); // "sleep" 520 milliseconds
+	tester.advance(); // run expired timers
 
-	root->get_member("this_counter", &tmp);
+	root->get_member(st.find("this_counter"), &tmp);
 	check_equals(tmp.to_number(), 5);
-	root->get_member("that_counter", &tmp);
+	root->get_member(st.find("that_counter"), &tmp);
 	check_equals(tmp.to_number(), 4);
 
-	root->get_member("pushed_args", &tmp);
+	root->get_member(st.find("pushed_args"), &tmp);
 	as_environment env; // needed for proper to_string()
-	check_equals(tmp.to_string(&env), std::string("8,9,10"));
+	check_equals(tmp.to_string(), std::string("8,9,10"));
 
-	root->get_member("test_completed", &tmp);
+	tester.advanceClock(100); // "sleep" another 100 milliseconds
+	tester.advance(); // run expired timers
+
+	root->get_member(st.find("test_completed"), &tmp);
 	check_equals(tmp.to_number(), 1);
 
 
