@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include "dlist.h"
@@ -27,6 +27,7 @@
 #include "DummyMovieDefinition.h"
 #include "DummyCharacter.h"
 #include "movie_definition.h"
+#include "ManualClock.h"
 
 #include <iostream>
 #include <sstream>
@@ -41,10 +42,6 @@ using namespace gnash;
 int
 main(int /*argc*/, char** /*argv*/)
 {
-	cxform color;
-	matrix mat;
-
-
 	gnash::LogFile& dbglogfile = gnash::LogFile::getDefaultInstance();
 	dbglogfile.setVerbosity();
 
@@ -54,7 +51,12 @@ main(int /*argc*/, char** /*argv*/)
 	// Initialize a VM
 	boost::intrusive_ptr<movie_definition> md5 ( new DummyMovieDefinition(5) );
 	boost::intrusive_ptr<movie_definition> md6 ( new DummyMovieDefinition(6) );
-	VM::init(*md5);
+
+	ManualClock clock;
+	VM& vm = VM::init(*md5, clock);
+
+	movie_instance* root = md5->create_movie_instance();
+    	vm.getRoot().setRootMovie( root );
 
 	DisplayList dlist1;
 
@@ -65,16 +67,16 @@ main(int /*argc*/, char** /*argv*/)
 	check_equals(dlist1, dlist2);
 
 	// just a couple of characters
-	boost::intrusive_ptr<character> ch1 ( new DummyCharacter() );
-	boost::intrusive_ptr<character> ch2 ( new DummyCharacter() );
+	boost::intrusive_ptr<character> ch1 ( new DummyCharacter(root) );
+	boost::intrusive_ptr<character> ch2 ( new DummyCharacter(root) );
 
-	dlist1.place_character( ch1.get(), 1, color, mat, 0, 0);
-	dlist1.place_character( ch2.get(), 2, color, mat, 0, 0);
+	dlist1.place_character( ch1.get(), 1);
+	dlist1.place_character( ch2.get(), 2);
 
 	check(dlist1 != dlist2);
 
-	dlist2.place_character( ch2.get(), 1, color, mat, 0, 0);
-	dlist2.place_character( ch1.get(), 2, color, mat, 0, 0);
+	dlist2.place_character( ch2.get(), 1);
+	dlist2.place_character( ch1.get(), 2);
 
 	// Resort dlist1 as depth of it's chars has been changed
 	// by place_character calls above :/
